#!/usr/bin/python
# Copyright 2016, 2017, Cumulus Networks, Inc.  All rights reserved.

import os

class UnknownPluggableType(Exception):
    pass

COMPATIBLE_MACS = ('bcm56850-mac', 'bcm56854-mac')
COMPATIBLE_SERDES = ('bcm56850-serdes', 'bcm56854-serdes')

def _read_str(*args, **kwargs):
    path = os.path.join(*args)
    if kwargs.get('cached', False):
        return __read_str_cached(path)
    else:
        return open(path, 'r').read().strip()

_read_str_cache = {}
def __read_str_cached(path):
    if not _read_str_cache.has_key(path):
        _read_str_cache[path] = _read_str(path, cached=False)
    return _read_str_cache[path]

def _read_int(*args, **kwargs):
    return int(_read_str(*args, **kwargs), 0)

def _read_int_arr(*args, **kwargs):
    strs = _read_str(*args, **kwargs).split()
    ints = []
    for e in strs:
        ints.append(int(e, 0))
    return tuple(ints)

#
# validate --
#     Determine if the ACPI switch description is complete.
#
def validate():
    # XXX - use the existence of an ONIE EEPROM ACPI node.  This function
    # should do much more checking and should be the first place to report
    # obvious errors in ACPI platform descriptions.
    path = '/sys/bus/acpi/devices'
    devices = os.listdir(path)
    for device in devices:
        try:
            if _read_str(path, device, 'description', cached=True) == 'ONIE EEPROM':
                return True
        except IOError:
            pass
    return False

def pci_devices():
    path = '/sys/bus/pci/devices'
    devices = os.listdir(path)
    for device in [ path + '/' + x for x in devices ]:
        yield device

def find_pci_device(vendor_id, device_id):
    for device in pci_devices():
        ven = int(_read_str(device, 'vendor', cached=True), 0)
        dev = int(_read_str(device, 'device', cached=True), 0)
        if (ven == vendor_id and dev == device_id):
            return device

_pluggable_cache = {}
def find_pluggable(label):
    cached = _pluggable_cache.get(label, None)
    if cached is not None:
        return cached

    path = '/sys/class/swp'
    ports = os.listdir(path)
    for port in [ path + '/' + x for x in ports ]:
        lab = _read_str(port, 'label', cached=True)
        if lab == label:
            _pluggable_cache[label] = port + '/pluggable'
            return _pluggable_cache[label]

_pluggables_cache = None
def find_pluggables():
    path = '/sys/class/swp'
    ports = os.listdir(path)
    if _pluggables_cache is None:
        _pluggables_cache = tuple([ os.path.join(path, port, 'pluggable')
                                    for port in ports ])
    return _pluggables_cache

_pluggable_type_cache = {}
def find_pluggable_type(label):
    pluggable = find_pluggable(label)
    if not pluggable:
        # XXX - log warning
        return None

    cached = _pluggable_type_cache.get(label, None)
    if cached is not None:
        return cached

    compatible = _read_str(pluggable, 'firmware_node', 'properties',
                           'compatible', cached=True)
    if compatible.startswith('sff-sfpp'):
        _pluggable_type_cache[label] = 'sfp+'
    elif compatible.startswith('sff-qsfpp'):
        _pluggable_type_cache[label] = 'qsfp+'
    elif compatible.startswith('sff-qsfp28'):
        _pluggable_type_cache[label] = 'qsfp28'
    else:
        raise UnknownPluggableType(compatible)

    return _pluggable_type_cache[label]

def find_pluggable_mem(label):
    pluggable = find_pluggable(label)
    if not pluggable:
        # XXX - log warning
        return None

    return '/sys/class/swp%s/pluggable/serial-interface/mem' % label

_pluggable_path_cache = {}
def find_pluggable_path(label):
    pluggable = find_pluggable(label)
    if not pluggable:
        # XXX - log warning
        return None

    cached = _pluggable_path_cache.get(label, None)
    if cached is not None:
        return cached

    _pluggable_path_cache[label] = _read_str(pluggable, 'firmware_node', 'path', cached=True)
    return _pluggable_path_cache[label]

_child_cache = {}
def find_child(path, address):
    cache_key = '%s:%s' % (path, address)

    if _child_cache.get(cache_key, None) is None:
        _child_cache[cache_key] = 'NOT_FOUND'
        for path in os.listdir(path):
            adr = _read_str(os.path.join(path, 'adr'), cached=True)
            if adr == address:
                _child_cache[cache_key] = path

    if _child_cache[cache_key] == 'NOT_FOUND':
        return None
    else:
        return _child_cache[cache_key]


_compatible_children_cache = {}
def find_compatible_children(path, compatible):
    cache_key = '%s:%s' % (path, compatible)

    if _compatible_children_cache.get(cache_key, None) is None:
        children = []
        for child in os.listdir(path):
            compatible_file = os.path.join(path, child, 'properties', 'compatible')
            if not os.path.exists(compatible_file):
                continue
            if _read_str(compatible_file) in compatible:
                children.append(os.path.join(path, child))

        _compatible_children_cache[cache_key] = tuple(children)
    return _compatible_children_cache[cache_key]

def find_bcm_interfaces():
    pluggable = find_pluggable(label)
    if not pluggable:
        # XXX - log warning
        return None

def pluggable_eeprom_file(label):
    pluggable = find_pluggable(label)
    if not pluggable:
        raise RuntimeError('no such port: %s' % label)
    return os.path.join(pluggable, 'eeprom')

class SWP(object):
    @classmethod
    def get_swps(self, device):
        path = '/sys/class/swp'
        swp_paths = os.listdir(path)

        swps = []
        for swp_path in swp_paths:
            # XXX - make sure this swp belongs to 'device'
            swps.append(SWP(_read_str(os.path.join(path, swp_path, 'label'), cached=True)))

        return tuple(sorted(swps, key=lambda x: int(x.label)))

    def __init__(self, label):
        self.label = label

    @property
    def pluggable_type(self):
        return find_pluggable_type(self.label)

    @property
    def pluggable_path(self):
        return find_pluggable_path(self.label)

    @property
    def channels(self):
        if self.pluggable_type == 'sfp+':
            return 1
        elif self.pluggable_type in ('qsfp+', 'qsfp28'):
            return 4
        else:
            raise NotImplementedError('unknown pluggable type: %s' % self.pluggable_type)

    def find_xrs(self, xrs):
        out = [None] * self.channels

        for xr in xrs:
            if xr.pluggable_path == self.pluggable_path:
                out[xr.pluggable_lane] = xr

        return out

# XXX - report the found vs. compatible macs/serdes
class CompatibleSerdesNotFound(Exception):
    pass
class CompatibleMacNotFound(Exception):
    pass
class TooManyMacs(Exception):
    pass
class NoSuchSerdes(Exception):
    pass

class BCMXR(object):
    @classmethod
    def _enum_serdes(self, device):
        fw = os.path.join(device, 'firmware_node')
        macs = find_compatible_children(fw, COMPATIBLE_MACS)
        if len(macs) == 0:
            raise CompatibleMacNotFound(fw)
        elif len(macs) > 1:
            raise TooManyMacs(fw)
        else:
            mac = macs[0]

        children = find_compatible_children(mac, COMPATIBLE_SERDES)

        if len(children) == 0:
            raise CompatibleSerdesNotFound(mac)

        return children

    @classmethod
    def get_xrs(self, device):
        xrs = []
        for serdes in self._enum_serdes(device):
            xrs.append(BCMXR(device, _read_int(os.path.join(serdes, 'adr'), cached=True)))
        return tuple(xrs)

    def __init__(self, device, serdes_adr):
        self.device = device
        self.adr = serdes_adr

        # cached
        self._path = None

    @property
    def index(self):
        if self.tx_index != self.rx_index:
            raise NotImplementedError('lane swap not implemented')
        return self.tx_index

    @property
    def path(self):
        if self._path is None:
            for serdes in self._enum_serdes(self.device):
                # XXX - dont use 'adr' get lowest tx_index
                adr = _read_int(os.path.join(serdes, 'adr'), cached=True)
                if adr == self.adr:
                    self._path = serdes

        if self._path is None:
            raise NoSuchSerdes(path)

        return self._path

    @property
    def tx_index(self):
        return _read_int(os.path.join(self.path, 'properties', 'tx-index'), cached=True)

    @property
    def rx_index(self):
        return _read_int(os.path.join(self.path, 'properties', 'rx-index'), cached=True)

    @property
    def pluggable_path(self):
        return _read_str(os.path.join(self.path, 'properties', 'pluggable', 'path'), cached=True)

    @property
    def pluggable_lane(self):
        try:
            return _read_int(os.path.join(self.path, 'properties', 'pluggable_args'), cached=True)
        except:
            # XXX - narrow this down to a missing exception
            return 0
