#!/usr/bin/python
# Copyright 2017 Cumulus Networks Inc, all rights reserved
#
# asic monitor module
#
#
#
#
#
#
#

import sys
import argparse
import getopt
import os
import subprocess
import json
import math
import time
import logging
import logging.handlers
import traceback
import datetime
import importlib

from cumulus.asic_exception import ChipNotSupportedError
from cumulus.asic_exception import ConfigFileError

from cumulus.asic_manager import AsicManager
import cumulus.asic_manager

# ==============================================================
class AsicMonitor(object):

    # ----------------------------------------------------------
    #
    def __init__(self, signal_handler):
        super(AsicMonitor,self).__init__()
        self.asic_manager = None
        self.signal_handler = signal_handler
        self.logger = logging.getLogger('asic-monitor-module')
        fmt = logging.Formatter(fmt='%(name)s %(levelname)s: %(message)s')
        handler = logging.handlers.SysLogHandler('/dev/log')
        handler.setFormatter(fmt)
        self.logger.setLevel(logging.INFO)
        self.logger.addHandler(handler)

    # ----------------------------------------------------------
    #
    def take_action(self, state_desc, results, collect_dict, depth=0):

        depth_value = depth
        if depth_value > 3:
            # safety net
            self.logger.error('take_action: recursed too far: desc is %s' % state_desc.name)
            return
        if results.trigger_flags & state_desc.snapshot_flag:
            # convert output to json and write to the file
            output_file = state_desc.snapshot.file + '_%d' % state_desc.curr_snapshot_idx
            state_desc.curr_snapshot_idx +=1
            if state_desc.curr_snapshot_idx >= state_desc.snapshot.file_count :
                state_desc.curr_snapshot_idx = 0
            cumulus.asic_manager.write_results_to_file(results, output_file)

        if results.trigger_flags & state_desc.log_flag:
            log_message = '%s: ' % str(results.end_time)
            queue_threshold      = getattr(state_desc.log, 'queue_bytes', None)
            congestion_threshold = state_desc.congestion_trigger_dict.get(state_desc.log_flag)
            error_threshold      = state_desc.error_trigger_dict.get(state_desc.log_flag)
            if queue_threshold != None:
                log_message += 'Egress queue(s) greater than %d bytes in monitor port group %s' % (queue_threshold,
                                                                                                   state_desc.name)
            elif congestion_threshold != None:
                log_message += 'Packet buffer congestion drops greater than %d packets in monitor port group %s' % (congestion_threshold,
                                                                                                                    state_desc.name)
            elif error_threshold != None:
                log_message += 'Packet buffer error drops greater than %d packets' % (error_threshold,
                                                                                      state_desc.name)
            else:
                log_message += 'no log threshold found for %s' % state_desc
            self.logger.info(log_message)

        if results.trigger_flags & state_desc.collect_flag:
            # start the data collection
            depth_value += 1
            for port_group_name in state_desc.collect.port_group_list:
                collect_pg = collect_dict[port_group_name]
                collect_results = self.asic_manager.read_state(collect_pg)
                if collect_results.trigger_flags:
                    self.take_action(collect_pg, collect_results, collect_dict, depth=depth_value)

    # ----------------------------------------------------------
    #
    def run(self):
        try:
            self.asic_manager = cumulus.asic_manager.initialize_asic_manager()
        except Exception as e:
            raise

        port_group_dict = self.asic_manager.config_manager.monitor.get_port_group_dict()
        port_group_count = len(port_group_dict.keys())
        if port_group_count == 0:
            # exit without error
            self.logger.info('No port groups configured')
            return

        if self.asic_manager.asic_is_supported == False:
            raise ChipNotSupportedError(self.asic_manager.chip_name)

        poll_state = {}
        poll_clock = {}
        state_desc_dict = {}
        populated_group_count = 0
        for port_group_name, port_group in port_group_dict.items():
            if len(port_group.linux_port_list) == 0:
                continue
            populated_group_count += 1
            new_state_desc = self.asic_manager.create_state_desc(port_group_name, port_group)
            state_desc_dict[new_state_desc.name] = new_state_desc
            if port_group.trigger_type == 'timer':
                multiplier = 1
                unit = port_group.timer[-1]
                timer_value = int(port_group.timer[0:-1])
                if unit == 'm':
                    multiplier = 60
                elif unit == 'h':
                    multiplier = 60 * 60
                elif unit == 'd':
                    multiplier = 24 * 60 * 60
                frequency = timer_value * multiplier
                if frequency not in poll_state.keys():
                    poll_state[frequency] = []
                    poll_clock[frequency] = 0
                poll_state[frequency].append(new_state_desc)

        if populated_group_count == 0:
            self.logger.info('Every port group has an empty port set')
            return
        if len(poll_state) == 0:
            self.logger.info('No timer actions configured on the ASIC monitor')
            return
        frequency_list = sorted(poll_state)

        # start the timers
        self.logger.info('ASIC monitor starting')
        num_frequencies = len(frequency_list)
        wait_time = 1
        while True:
            for curr_idx in range(num_frequencies):
                curr_frequency = frequency_list[curr_idx]
                if poll_clock[curr_frequency] == curr_frequency:
                    for state_desc in poll_state[curr_frequency]:
                        if self.signal_handler.exit_flag:
                            return
                        results = self.asic_manager.read_state(state_desc)
                        if results.trigger_flags:
                            self.take_action(state_desc, results, state_desc_dict, depth=0)
                    poll_clock[curr_frequency] = 0
                poll_clock[curr_frequency] += wait_time
            time.sleep(wait_time)

    # ----------------------------------------------------------
    #
    def clean_up(self):
        if self.asic_manager:
            self.asic_manager.clear_manager()
        self.logger.info('ASIC monitor stopping')
