#
# Copyright (C) 2017 Cumulus Networks, Inc.
#

#----------------------------------------------------------------------------
#
#   Accton OMP800 chassis
#
#   Do not instantiate this class or import this module directly.
#   Use the parent module's probe() function.  See __init__.py.
#
#----------------------------------------------------------------------------

import os.path
from cumulus.chassis.base import chassisBase

class chassisImpl(chassisBase):
    """
    Functions specific to the Accton OMP800 Chassis
    """

    # Constants
    _SYSFS_DEVICE       = "/sys/bus/i2c/devices/12-0060"
    _SYSFS_CARD_TYPE    = _SYSFS_DEVICE + "/card_type_id"
    _SYSFS_SLOT         = _SYSFS_DEVICE + "/slot_id"
    _SYSFS_CPU          = _SYSFS_DEVICE + "/cpu_id"
    _SYSFS_LOWER        = _SYSFS_DEVICE + "/lower_subchassis"

    def __init__(self):
        chassisBase.__init__(self)
        self._slot = None
        if not os.path.isdir(self._SYSFS_DEVICE):
            return
        try:
            # Line cards are 1 to 32.  Fabric cards are 33 to 48.
            # Upper chassis are the first half (1-16 for LC, 33-40 for FC) and
            # Lower chassis (not supported) are second half (17-32 for LC and
            # 41-48 for FC). Cards are in this order: 1-A, 1-B, 2-A, 2-B, ...
            card_type = self._readint(self._SYSFS_CARD_TYPE) # 0 = LC, 1 = FC
            hw_slot   = self._readint(self._SYSFS_SLOT)      # 0-7
            cpu       = self._readint(self._SYSFS_CPU)       # A = 0, B = 1
            lowerChas = self._readint(self._SYSFS_LOWER)  # Upper = 0, Lower = 1
            sw_slot   = (card_type * 32) + (2 - card_type)*8*lowerChas + (hw_slot * 2) + cpu + 1
        except (IOError, ValueError, TypeError) as e:
            raise RuntimeError("Could not determine chassis slot number: %s" %
                               str(e))
        self._slot = sw_slot

    def HasSlotNum(self):
        return self._slot is not None

    def GetSlotNum(self):
        """
        Return the slot number of the module on which this code is running.
        The slot number is computed once at instance creation time,
        and not expected to change (duh).
        """
        if self._slot is None:
            raise RuntimeError("Platform driver not loaded")
        return self._slot

    def GetMgmtAddr(self):
        """
        Get the IPv6 link local management address of this chassis card
        """
        slot = self.GetSlotNum()
        if self.isFabricCard():
            # fabric card
            return "fe80::%x0%d:2" % ((slot - 31)/2, 2 - (slot % 2))
        else:
            return "fe80::%x0%d:3" % ((slot + 1)/2, 2 - (slot % 2))

    def GetV4MgmtAddr(self):
        """
        Get the IPv4 link local management address of this chassis card
        """
        return "169.254.251.%d" % (self.GetSlotNum(),)

    def GetMgmtIf(self):
        """
        Return the name of the management interface
        """
        return "eth0.4088"

    def GetNumFabIntfs(self):
        """
        Return the number of fabric interfaces on this card.
        """
        if self.isFabricCard():
            return 32
        else:
            return 16

        return 0

    def isFabricCard(self):
        """
        Return true if this is a fabric card. Line cards return False.
        """
        return self.GetSlotNum() > 32

    def GetModuleName(self, slot = None):
        """
        Return the name of a slot (fc[1-4]0[12] or lc[1-8]0[12]).
        Default slot is this module.
        """
        if slot is None:
            slot = self.GetSlotNum()
        card = "fc" if self.isFabricCard() else "lc"
        lcfcslot = (slot - 31) / 2 if self.isFabricCard() else (slot + 1) / 2
        ab = 2 - (slot % 2)
        return card + str(lcfcslot) + "0" + str(ab)

    def GetMasterChassisBmcNum(self):
        """
        Retrieve the current master chassis management BMC number.
        """
        return None

    def GetChassisBmc(self, number):
        """
        Return a platform BMC object (an instanace of a subclass
        of cumulus.platform.BMC) for the indicated chassis
        management BMC.
        """
        return None

    def GetChassisName(self):
        """
        Return the name of the chassis.
        """
        return "accton_omp800"

    def GetV4Hosts(self):
        """
        Return the IPv4 host entries for all slots of the chassis.
        """

        base_ip = "169.254.251."
        hosts = ""
        index = 1
        for lc in range(1, 9):
            for swe in range(1,3):
                ip = "%s%d" % (base_ip, index)
                name = "lc%d0%d" % (lc, swe)
                hosts += "%-20s %-15s %s.chassis\n" % (ip, name, name)
                index += 1

        index = 33
        for fc in range(1, 5):
            for swe in range(1,3):
                ip = "%s%d" % (base_ip, index)
                name = "fc%d0%d" % (fc, swe)
                hosts += "%-20s %-15s %s.chassis\n" % (ip, name, name)
                index += 1

        return hosts

    def GetV6Hosts(self):
        """
        Return the IPv6 host entries for all slots of the chassis.
        """

        base_ip = "fe80::"
        hosts = ""
        for fc in range(1, 5):
            for swe in range(1,3):
                ip = "%s%d0%d:2" % (base_ip, fc, swe)
                name = "fc%d0%d-v6" % (fc, swe)
                hosts += "%-20s %-15s %s.chassis\n" % (ip, name, name)

        for lc in range(1, 9):
            for swe in range(1,3):
                ip = "%s%d0%d:3" % (base_ip, lc, swe)
                name = "lc%d0%d-v6" % (lc, swe)
                hosts += "%-20s %-15s %s.chassis\n" % (ip, name, name)

        return hosts
