#
# Copyright 2016.  Cumulus Networks, Inc.
#

import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class AcctonAS670x_32XQSFPp(cumulus.platform.TridentTwo40GPort):
    _set_led = '''
;;; AcctonAS670x_32XQSFPp
;;; Two bits per port: yellow(active low), green(active low)
set_led:
        port a

        pushst  PORTSTATUS_ONE
        pack

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        tinv
        pack

        ret
'''
    _set_led_4x10 = '''
;;;
;;; Input : register A, logical port
;;; Input / Output : register B, accumulated port status

;;; For each port accumulate 2 bits of information in register B:
;;;   register B[0] - activity (TX _OR_ RX)
;;;   register B[1] - link enabled

SUB_INTF_ACTIVITY	equ	0x0  ; activity
SUB_INTF_LINKEN		equ	0x1  ; link enabled

sub_intf_info:

        ;;; push OR of current TX/RX activity onto stack
        port a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor

        ;;; push accumlated link activity onto stack
        tst B, SUB_INTF_ACTIVITY
        push cy

        ;;; OR accumlated activity with current activity and store
        ;;; result back in register B
        tor
        pop
        bit B, SUB_INTF_ACTIVITY

        ;;; push current LINKEN onto stack
        pushst  PORTSTATUS_LINKEN

        ;;; push accumlated link enable onto stack
        tst B, SUB_INTF_LINKEN
        push cy

        ;;; OR accumlated LINKEN with current LINKEN and store
        ;;; result back in register B
        tor
        pop
        bit B, SUB_INTF_LINKEN

        ret

;;; AcctonAS670x_32XQSFPp_4x10
;;; OR together status of 4 ports for 4x10G QSFP port
;;; Two LED bits per port: yellow(active low), green(active low)
;;; Input: register A - first logical port of 4-port group
;;; Modified: register B - used as scratch space
set_led:
        ;;; disable yellow LED
        pushst  PORTSTATUS_ONE
        pack

        ;;; initialize B
        ld	b,0

        ;;; load sub-0 info
        call	sub_intf_info

        ;;; load sub-1 info
        inc	a
        call	sub_intf_info

        ;;; load sub-2 info
        inc	a
        call	sub_intf_info

        ;;; load sub-3 info
        inc	a
        call	sub_intf_info

        ;;; push accumlated link activity onto stack
        tst	B, SUB_INTF_ACTIVITY
        push	cy

        ;;; combine with BLINK state
        push	(BLINK_STATE_P)
        tand
        tinv	; flash off with activity

        ;;; push accumlated link enable onto stack
        tst	B, SUB_INTF_LINKEN
        push	cy

        ;;; AND activity-blink state with accumlated LINKEN
        tand

        ;;; LEDs are active low
        tinv
        pack

        ret
'''
    _serdes_intf_map = { 40:'XGMII', 10:'XFI' }

    def __init__(self, label, base_serdes, leds, xr_lane_map=None):
        serdes = [ base_serdes + x for x in range(4) ]
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp+')
        cumulus.platform.TridentTwo40GPort.__init__(self,
                                                    label=label,
                                                    serdes=serdes,
                                                    phy_types=None,
                                                    phy_addrs=None,
                                                    phy_layer=sfp.type.upper(),
                                                    leds=leds,
                                                    rx_lane_map=xr_lane_map,
                                                    rx_polarity_flip=None,
                                                    tx_lane_map=xr_lane_map,
                                                    tx_polarity_flip=None,
                                                    oversub=True,
                                                    unit=0)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if (mode == "4x10G") and (sub != 0):
            # For 4x10G mode only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = self.__class__.__name__
        if mode == "4x10G":
            call = (fn_name + "_4x10", self._set_led_4x10, led_port)
        elif mode == "40G":
            call = (fn_name, self._set_led, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0] + 1] = None

        return set_calls

class AcctonAS670x_32XLED(cumulus.ledup.TridentTwoLEDController):
    num_leds = 32

class AcctonAS670x_32XSwitch(cumulus.platform.BCMSwitch):
    ledup = AcctonAS670x_32XLED
    _qsfp = AcctonAS670x_32XQSFPp

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (
        _qsfp(label="1" , base_serdes=12,  leds=(0, 0)),
        _qsfp(label="2" , base_serdes=16,  leds=(0, 2)),
        _qsfp(label="3" , base_serdes=20,  leds=(0, 4)),
        _qsfp(label="4" , base_serdes=24,  leds=(0, 6)),
        _qsfp(label="5" , base_serdes=28,  leds=(0, 8)),
        _qsfp(label="6" , base_serdes=32,  leds=(0, 10)),
        _qsfp(label="7" , base_serdes=36,  leds=(0, 12)),
        _qsfp(label="8" , base_serdes=40,  leds=(0, 14)),
        _qsfp(label="9" , base_serdes=44,  leds=(0, 16)),
        _qsfp(label="10", base_serdes=48,  leds=(0, 18)),
        _qsfp(label="11", base_serdes=64,  leds=(1, 0)),
        _qsfp(label="12", base_serdes=68,  leds=(1, 2)),
        _qsfp(label="13", base_serdes=72,  leds=(1, 4)),
        _qsfp(label="14", base_serdes=76,  leds=(1, 6)),
        _qsfp(label="15", base_serdes=80,  leds=(1, 8)),
        _qsfp(label="16", base_serdes=96,  leds=(1, 10)),
        _qsfp(label="17", base_serdes=100, leds=(1, 12)),
        _qsfp(label="18", base_serdes=104, leds=(1, 14)),
        _qsfp(label="19", base_serdes=108, leds=(1, 16)),
        _qsfp(label="20", base_serdes=112, leds=(1, 18)),
        _qsfp(label="21", base_serdes=116, xr_lane_map=0x2301, leds=(1, 26)),
        _qsfp(label="22", base_serdes=120, xr_lane_map=0x2301, leds=(1, 28)),
        _qsfp(label="23", base_serdes=124, xr_lane_map=0x2301, leds=(1, 30)),
        _qsfp(label="24", base_serdes=0,   xr_lane_map=0x2301, leds=(0, 20)),
        _qsfp(label="25", base_serdes=4,   xr_lane_map=0x2301, leds=(0, 22)),
        _qsfp(label="26", base_serdes=8,   xr_lane_map=0x2301, leds=(0, 24)),
        _qsfp(label="27", base_serdes=84,  xr_lane_map=0x2301, leds=(1, 20)),
        _qsfp(label="28", base_serdes=88,  xr_lane_map=0x2301, leds=(1, 22)),
        _qsfp(label="29", base_serdes=92,  xr_lane_map=0x2301, leds=(1, 24)),
        _qsfp(label="30", base_serdes=52,  xr_lane_map=0x2301, leds=(0, 26)),
        _qsfp(label="31", base_serdes=56,  xr_lane_map=0x2301, leds=(0, 28)),
        _qsfp(label="32", base_serdes=60,  xr_lane_map=0x2301, leds=(0, 30)),
    )

class AcctonAS670xSYSCPLD(cumulus.cpld.SysFS):
    path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-0031'

class AcctonAS670xFANCPLD(cumulus.cpld.SysFS):
    path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-0035'

class AcctonAS670xPSU1(cumulus.sensors.PSU_Unit):
    name = 'PSU1'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-0031'
    driver_hwmon = ['psu_pwr1']

class AcctonAS670xPSU2(cumulus.sensors.PSU_Unit):
    name = 'PSU2'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-0031'
    cpld_hwmon = ['psu_pwr2']

class AcctonAS670xTemp1(cumulus.sensors.Temp_Unit):
    name = 'Temp1'
    driver_path = '/sys/devices/soc.0/ffe03000.i2c/i2c-0/i2c-5/5-004d'
    driver_hwmon = ['temp1']

class AcctonAS670xTemp2(cumulus.sensors.Temp_Unit):
    name = 'Temp2'
    driver_path = '/sys/devices/soc.0/ffe03000.i2c/i2c-0/i2c-5/5-004d'
    driver_hwmon = ['temp2']

class AcctonAS670xTemp3(cumulus.sensors.Temp_Unit):
    name = 'Temp3'
    driver_path = '/sys/devices/soc.0/ffe03000.i2c/i2c-0/i2c-5/5-004d'
    driver_hwmon = ['temp3']

class AcctonAS670xTemp4(cumulus.sensors.Temp_Unit):
    name = 'Temp4'
    driver_path = '/sys/devices/soc.0/ffe03000.i2c/i2c-0/i2c-5/5-004d'
    driver_hwmon = ['temp4']

class AcctonAS670xTemp5(cumulus.sensors.Temp_Unit):
    name = 'Temp5'
    driver_path = '/sys/devices/soc.0/ffe03000.i2c/i2c-0/i2c-5/5-004d'
    driver_hwmon = ['temp5']

class AcctonAS670xTemp6(cumulus.sensors.Temp_Unit):
    name = 'Temp6'
    driver_path = '/sys/devices/soc.0/ffe03000.i2c/i2c-0/i2c-5/5-004d'
    driver_hwmon = ['temp6']

class AcctonAS670xTemp7(cumulus.sensors.Temp_Unit):
    name = 'Temp7'
    driver_path = '/sys/devices/soc.0/ffe03000.i2c/i2c-0/i2c-5/5-004d'
    driver_hwmon = ['temp7']

class AcctonAS670xTemp8(cumulus.sensors.Temp_Unit):
    name = 'Temp8'
    driver_path = '/sys/devices/soc.0/ffe03000.i2c/i2c-0/i2c-5/5-004d'
    driver_hwmon = ['temp8']

class AcctonAS670xFan1(cumulus.sensors.Fan_Unit):
    name = 'Fan1'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-0035'
    driver_hwmon = ['fan1']

class AcctonAS670xFan2(cumulus.sensors.Fan_Unit):
    name = 'Fan2'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-0035'
    driver_hwmon = ['fan2']

class AcctonAS670xFan3(cumulus.sensors.Fan_Unit):
    name = 'Fan3'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-0035'
    driver_hwmon = ['fan3']

class AcctonAS670xFan4(cumulus.sensors.Fan_Unit):
    name = 'Fan4'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-0035'
    driver_hwmon = ['fan4']

class AcctonAS670xFan5(cumulus.sensors.Fan_Unit):
    name = 'Fan5'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-0035'
    driver_hwmon = ['fan5']

class AcctonAS670xFan6(cumulus.sensors.Fan_Unit):
    name = 'Fan6'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-0035'
    driver_hwmon = ['fan6']

class AcctonAS670xFan7(cumulus.sensors.Fan_Unit):
    name = 'Fan7'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-0035'
    driver_hwmon = ['fan7']

class AcctonAS670xFan8(cumulus.sensors.Fan_Unit):
    name = 'Fan8'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-0035'
    driver_hwmon = ['fan8']

class AcctonAS670xFan9(cumulus.sensors.Fan_Unit):
    name = 'Fan9'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-0035'
    driver_hwmon = ['fan9']

class AcctonAS670xFan10(cumulus.sensors.Fan_Unit):
    name = 'Fan10'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-0035'
    driver_hwmon = ['fan10']

class AcctonAS670xSensors():
    units = ( AcctonAS670xPSU1(),
              AcctonAS670xPSU2(),
              AcctonAS670xTemp1(),
              AcctonAS670xTemp2(),
              AcctonAS670xTemp3(),
              AcctonAS670xTemp4(),
              AcctonAS670xTemp5(),
              AcctonAS670xTemp6(),
              AcctonAS670xTemp7(),
              AcctonAS670xTemp8(),
              AcctonAS670xFan1(),
              AcctonAS670xFan2(),
              AcctonAS670xFan3(),
              AcctonAS670xFan4(),
              AcctonAS670xFan5(),
              AcctonAS670xFan6(),
              AcctonAS670xFan7(),
              AcctonAS670xFan8(),
              AcctonAS670xFan9(),
              AcctonAS670xFan10(),
              )
class AcctonAS670xPWM1(cumulus.sensors.PWM_Unit):
    fans = (  AcctonAS670xFan1(),
	      AcctonAS670xFan2(),
              AcctonAS670xFan3(),
              AcctonAS670xFan4(),
              AcctonAS670xFan5(),
              AcctonAS670xFan6(),
              AcctonAS670xFan7(),
              AcctonAS670xFan8(),
              AcctonAS670xFan9(),
              AcctonAS670xFan10(),
            )
    temps = ( AcctonAS670xTemp1(),
              AcctonAS670xTemp2(),
              AcctonAS670xTemp3(),
              AcctonAS670xTemp4(),
              AcctonAS670xTemp5(),
              AcctonAS670xTemp6(),
              AcctonAS670xTemp7(),
              AcctonAS670xTemp8(),
            )

class AcctonAS670xPWMs():
    units = (AcctonAS670xPWM1(),
             )

class AcctonAS670xSystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_location = 'led_sys'
    bad_led_color = 'green_blinking'

class AcctonAS670xLEDs():
    units = ( AcctonAS670xSystemLED(),
             )
class AcctonAS670x_32X(cumulus.platform.Platform):
    chip = cumulus.platform.TridentTwo_56850_Chip()
    switch = AcctonAS670x_32XSwitch(chip)
    cpld = cumulus.cpld.CPLDMerged((AcctonAS670xSYSCPLD(), AcctonAS670xFANCPLD()))
    sensors = AcctonAS670xSensors()
    pwms = AcctonAS670xPWMs()
    statusleds = AcctonAS670xLEDs()
