#
# Copyright 2017.  Cumulus Networks, Inc.
#

import cumulus.platform
import cumulus.chassis
import cumulus.sensors
import subprocess
import json

sensor_count = 0


class AcctonOMP800Temp(cumulus.sensors.Temp_Unit):
    hwmon_search = False
    no_shutdown = True
    driver_hwmon = ['temp1']


class AcctonOMP800CPUTemp(object, AcctonOMP800Temp):
    temp_max = 64
    temp_crit = 95

    def __init__(self):
        AcctonOMP800Temp.__init__(self)


class AcctonOMP800ASICTemp(object, AcctonOMP800Temp):
    temp_max = 82
    temp_crit = 105

    def __init__(self):
        AcctonOMP800Temp.__init__(self)


class AcctonOMP800LM75ATemp(object, AcctonOMP800Temp):
    temp_max = 60
    temp_crit = 85

    def __init__(self):
        AcctonOMP800Temp.__init__(self)


class AcctonOMP800LM75BTemp(object, AcctonOMP800Temp):
    temp_max = 65
    temp_crit = 90

    def __init__(self):
        AcctonOMP800Temp.__init__(self)


class AcctonOMP800LM75CTemp(object, AcctonOMP800Temp):
    temp_max = 69
    temp_crit = 94

    def __init__(self):
        AcctonOMP800Temp.__init__(self)


class AcctonOMP800LM75DTemp(object, AcctonOMP800Temp):
    temp_max = 62
    temp_crit = 87

    def __init__(self):
        AcctonOMP800Temp.__init__(self)


class AcctonOMP800CoreTemp(cumulus.sensors.Temp_Unit):
    driver_path = '/sys/devices/platform/coretemp.0'
    temp_max = 98
    temp_crit = 98
    no_shutdown = True


class AcctonOMP800Fan(object, cumulus.sensors.Fan_Unit):
    hwmon_search = False
    no_pwm = True
    driver_hwmon = ['fan1']

    def __init__(self):
        cumulus.sensors.Fan_Unit.__init__(self)

# Dynamically generate sensor classes
def create_sensor_classes():
    vhwmon_path = '/sys/class/misc/vhwmon/'
    global sensor_count
    sensor_count = 0
    base_class = None
    sensor_devices = ["lm75%s_temp" % x for x in ('a', 'b', 'c', 'd')] + ["asic_temp", "cpu_temp"]
    cards = ["fc%s" % i for i in xrange(101, 501, 100)] + ["fc%s" % i for i in xrange(102, 502, 100)] + \
            ["lc%s" % x for x in xrange(101, 901, 100)] + ["lc%s" % x for x in xrange(102, 902, 100)]
    for card in cards:
        for sensor in sensor_devices:
            if 'lm75a' in sensor:
                base_class = AcctonOMP800LM75ATemp
            if 'lm75b' in sensor:
                base_class = AcctonOMP800LM75BTemp
            if 'lm75c' in sensor:
                base_class = AcctonOMP800LM75CTemp
            if 'lm75d' in sensor:
                base_class = AcctonOMP800LM75DTemp
            if 'asic_temp' in sensor:
                base_class = AcctonOMP800CPUTemp
            if 'cpu_temp' in sensor:
                base_class = AcctonOMP800ASICTemp

            globals()["AcctonOMP800Temp" + str(sensor_count)] = type("AcctonOMP800Temp" + str(sensor_count),
                                                                     (base_class, ),
                                                                     dict(name='Temp' + str(sensor_count),
                                                                          driver_path=vhwmon_path + '%s:%s' % (card, sensor),
                                                                          description='%s %s' %(card, sensor)))
            sensor_count += 1
    # 16 fans in total on the Accton chassis
    for fan_num in xrange(1, 17):
        globals()["AcctonOMP800Fan" + str(fan_num) + "Front"] = type("AcctonOMP800Fan" + str(fan_num) + "Front",
                                                                     (AcctonOMP800Fan, ),
                                                                     dict(name="Fan" + str(fan_num) + "Front",
                                                                          driver_path=vhwmon_path + "chassis:Fan" +
                                                                          str(fan_num) + "Front",
                                                                          description="Fan" + str(fan_num) + " Front"))

        globals()["AcctonOMP800Fan" + str(fan_num) + "Rear"] = type("AcctonOMP800Fan" + str(fan_num) + "Rear",
                                                                    (AcctonOMP800Fan, ),
                                                                    dict(name="Fan" + str(fan_num) + "Rear",
                                                                         driver_path=vhwmon_path + "chassis:Fan" +
                                                                         str(fan_num) + "Rear",
                                                                         description="Fan" + str(fan_num) + " Rear"))


# Dynamically generate objects for classes above
# This will be used to populate "units" in the
# Sensor class below.
def sensor_obj_factory():
    sensor_instances = ()

    create_sensor_classes()
    for i in xrange(1, sensor_count):
        sensor_instances = sensor_instances + (globals()["AcctonOMP800Temp" + str(i)](), )
    for fan_num in xrange(1, 17):
        sensor_instances = sensor_instances + (globals()["AcctonOMP800Fan" + str(fan_num) + "Front"](), )
        sensor_instances = sensor_instances + (globals()["AcctonOMP800Fan" + str(fan_num) + "Rear"](), )
    return sensor_instances


class AcctonOMP800BMC(cumulus.platform.BMC):
    type = "virtbmc"

    def __init__(self, chassis):
        if "__init__" in cumulus.platform.BMC.__dict__:
            cumulus.platform.BMC.__init__(self)
        if not chassis.HasSlotNum():
            return
        self.chassis_obj = chassis
        try:
            leaders = subprocess.check_output(["/usr/bin/chassisctl", "-j", "localmasters"])
        except Exception:
            return
        self.leaders_list = json.loads(leaders)

    def twos_complement(self, value, bitwidth):
        if value >= 2 ** bitwidth:
            # This catches when someone tries to give a value that is out of range
            raise ValueError("Value: {} out of range of {}-bit value.".format(value, bitwidth))
        else:
            return value - int((value << 1) & 2 ** bitwidth)

    # Query should only run on one fabric card on each side.
    # A leader is elected on each side by chassisd to run i2c
    # transactions. We use that leader for querying sensors.
    def query(self):
        node_str = ''
        slot = self.chassis_obj.GetSlotNum()
        name = self.chassis_obj.GetModuleName(slot)

        # For the accton chassis, sensors are read only from
        # the CPLD present on a Fan Control Unit(FCU). FCU is accessible only
        # from CPU-A of fabric cards.
        if not self.chassis_obj.isFabricCard():
            return None

        if slot not in self.leaders_list:
            return None

        sensors_dict = {'temp': [], 'fan': [], }
        sensor_paths = {"channel-6": "/sys/bus/i2c/devices/0-0076/channel-6/16-0066/",
                        "channel-0": "/sys/bus/i2c/devices/0-0076/channel-0/10-0066/"}
        # Temp sensors present on line cards and fabric cards. there are 6 in total - lm75a-d,
        # asic_temp and cpu_temp.
        sensor_devices = ["lm75%s_temp" % x for x in ('a', 'b', 'c', 'd')] + ["asic_temp", "cpu_temp"]
        # FC and LC endpoints, each of which have the above temp sensors
        node_list = ["fc%s" % x for x in ('a', 'b')] + ["lc%s" % x for x in ('a', 'b', 'c', 'd')]
        # remove "fc" from the module Name and get the actual fabric card number
        # minus the "01" or "02" which denotes the side.
        fabric_card_num = int(name[2:]) / 100

        for channel, path in sensor_paths.iteritems():
            for node in node_list:
                for sensor in sensor_devices:
                    sensor_dev = path + node + "_" + sensor
                    sysfs_file = open(sensor_dev, 'r')
                    label = node + "_" + sensor
                    # Figure out if the sensor device is part of lc
                    # or fc by looking at first 2 chars of the sysfs file
                    # name.
                    sensor_dev_card_type = node[:2]
                    if sensor_dev_card_type == 'lc' or sensor_dev_card_type == 'fc':
                        if channel == "channel-6":
                            if fabric_card_num == 3 or fabric_card_num == 4:
                                # channel-6 on fc4 or fc3 maps as
                                # lca -> lc502
                                # lcb -> lc602
                                # lcc -> lc702
                                # lcd -> lc802
                                if node == 'fca':
                                    node_str = 'fc301'
                                elif node == 'fcb':
                                    node_str = 'fc401'
                                else:
                                    node_str = "lc%s" % ((int(node[2:], 16) - int('a', 16)) * 100 + 502)

                            if fabric_card_num == 1 or fabric_card_num == 2:
                                # channel-6 on fc1 or fc2 maps as
                                # lca -> lc101
                                # lcb -> lc201
                                # lcc -> lc301
                                # lcd -> lc401
                                if node == 'fca':
                                    node_str = 'fc101'
                                elif node == 'fcb':
                                    node_str = 'fc201'
                                else:
                                    node_str = "lc%s" % ((int(node[2:], 16) - int('a', 16)) * 100 + 101)

                        if channel == "channel-0":
                            if fabric_card_num == 3 or fabric_card_num == 4:
                                # channel-0 on fc4 or fc3 maps as
                                # lca -> lc102
                                # lcb -> lc202
                                # lcc -> lc302
                                # lcd -> lc402
                                if node == 'fca':
                                    node_str = 'fc302'
                                elif node == 'fcb':
                                    node_str = 'fc402'
                                else:
                                    node_str = "lc%s" % ((int(node[2:], 16) - int('a', 16)) * 100 + 102)

                            if fabric_card_num == 1 or fabric_card_num == 2:
                                # channel-0 on fc1 or fc2 maps as
                                # lca -> lc501
                                # lcb -> lc601
                                # lcc -> lc701
                                # lcd -> lc801
                                if node == 'fca':
                                    node_str = 'fc102'
                                elif node == 'fcb':
                                    node_str = 'fc202'
                                else:
                                    node_str = "lc%s" % ((int(node[2:], 16) - int('a', 16)) * 100 + 501)

                        label = "%s_" % node_str + sensor
                    # temp sensor values are stored in lm75 format.
                    # It consists of 8 bits encoded in two's complement.
                    value = self.twos_complement(int(sysfs_file.read().strip('\n'), 16), 8)
                    if value <= -127:
                        continue
                    sensors_dict['temp'].append({'sensor_name': sensor,
                                                 'sensor_node': node_str,
                                                 'label': label,
                                                 'description': '%s %s' % (node_str, sensor),
                                                 'value': value})
                    sysfs_file.close()
        # Referencing the Fan zone diagram sent to us by Accton, Fan1 is actually Fan3
        # in the lower right zone. Since, the real chassis in our labs is a 90 deg flip
        # of the diagram, that fan, physically, is the top leftmost one.
        # The fans are numbered linearly from 1 to 16 from there on.
        for fan_num in xrange(1, 5):
            fans = [sensor_paths["channel-6"] + "fan%s_" % fan_num + "front_tach",
                    sensor_paths["channel-6"] + "fan%s_" % fan_num + "rear_tach",
                    sensor_paths["channel-0"] + "fan%s_" % fan_num + "front_tach",
                    sensor_paths["channel-0"] + "fan%s_" % fan_num + "rear_tach"]
            card_start_num = 1

            for fan in fans:
                fp = open(fan, 'r')
                tach_value = int(fp.read().strip('\n'), 0)

                if 'rear' in fan:
                    pos = 'Rear'
                else:
                    pos = 'Front'
                if 'channel-6' in fan:
                    zone_inc = 0
                else:
                    zone_inc = 2

                if fabric_card_num == 1 or fabric_card_num == 2:
                    card_start_num = 9

                row_num = 1 - (fan_num % 4) / 2
                fan_pos = 1 - (fan_num % 2)

                fan_id = card_start_num + zone_inc + 4 * row_num + fan_pos
                sensor_name = "Fan%s" % fan_id + "%s" % pos

                sensors_dict['fan'].append({'sensor_name': sensor_name,
                                            'sensor_node': 'chassis',
                                            'label': 'chassis',
                                            'description': "Fan%s %s" % (fan_id, pos),
                                            'value': tach_value * 100})

        return sensors_dict


class AcctonOMP800Sensors():
    units = () + sensor_obj_factory()




        

