#
# Copyright 2017.  Cumulus Networks, Inc.
#
import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
import cumulus.chassis
from cumulus.platforms.accton_omp800_common import *

class AcctonOMP800Fabric(cumulus.platform.Tomahawk100GPort):
    '''
    Fabric ports are 100G Tomahawk ports using KR4 serdes and
    no LEDs.
    '''
    is_fabric = True
    create_fabric = True
    def __init__(self, label, serdes, tx_polarity_flip,
                 rx_polarity_flip, tx_lane_map, rx_lane_map):
        cumulus.platform.Tomahawk100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   leds=None,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=None,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   rx_lane_map=rx_lane_map,
                                                   oversub=True,
                                                   unit=0)
        self._serdes_intf_map = { 100 : 'KR4' }

    @property
    def max_logical_ports(self):
        return 1

class AcctonOMP800QSFP28(cumulus.platform.Tomahawk100GPort):
    '''
    QSFP28 ports have 6 LED bits, 3 per LED, per port. The second LED is not
    used. The first LED indicates Link and Activity.
    '''
    _set_led_common = '''
;;;
;;; common LED routines
;;;
;;; Basic formula for an LED is: L & !(A & B)
;;;
;;; where:
;;;     L is link enable
;;;     A is transmit or receive activity
;;;     B is the blink state
;;; When a port is split into multiple ports whose status must be indicated by
;;; this single LED, the link enable is the OR of all of the link enables and
;;; the activity is the OR of all of the tx and rx activity on all ports.
;;;

;;;
;;; set_port_led - This routine outputs the LED bits for the specified port(s).
;;; Each pair of ports has four LEDs. Only the two outermost LEDs are used. And
;;; even though the ports support 8 RGB colors, only green and off are used.
;;; This routine is called with the port number in register a and the number of
;;; ports to include (1, 2, or 4) in register b.
;;;
set_port_led:
        tst     a, 2                ; Lower port?
        jnc     get_port_activity   ; No, don't turn off the first LED

        push    1                   ; This LED isn't used, turn it off
        pack
        push    1
        pack
        push    1
        pack

get_port_activity:
        push    1                   ; First bit (always off)
        pack

        ld      (0xf2), b           ; Save the number of ports
        add     b, a
        push    0                   ; Initial state, no activity

accum_activity:
        dec     b                   ; Select the next port
        port    b                   ; The port upon which we are operating
        pushst  PORTSTATUS_RX       ; Any RX activity recently?
        tor                         ; Accumulate activity
        pushst  PORTSTATUS_TX       ; Any TX activity recently?
        tor                         ; Accumulate activity
        cmp     a, b                ; Are we done?
        jnz     accum_activity      ; Nope, go add in another port

        push    (BLINK_STATE_P)     ; If we are blinking, turn it off
        tand
        tinv                        ; Invert the result

set_led_link:
        add     b, (0xf2)
        push    0                   ; Initial state, no link

accum_link:
        dec     b                   ; Select the next port
        port    b                   ; The port upon which we are operating
        pushst  PORTSTATUS_LINKEN   ; Is there any link on this port?
        tor                         ; Accumulate link
        cmp     a, b                ; Are we done?
        jnz     accum_link          ; Nope, go add in another port

        tand                        ; AND link with activity and blink
        tinv
        pack                        ; Set the bit to turn on/off green

        push    1                   ; Last bit (always off)
        pack

        tst     a, 2                ; Lower port?
        jc      all_done            ; Yes, turn off the last LED

        push    1                   ; This LED isn't used, turn it off
        pack
        push    1
        pack
        push    1
        pack

all_done:
        ret
'''

    _set_led_1x = '''
;;;
;;; In 1x mode, specified port's status is used.
;;;
set_led:
        ld      b, 1                ; Number of ports to combine into this LED
        call    set_port_led
        ret
'''

    _set_led_2x = '''
;;;
;;; In 2x mode, two port's status is OR'd together
;;;
set_led:
        ld      b, 2                ; Number of ports to combine into this LED
        call    set_port_led
        ret
'''

    _set_led_4x = '''
;;;
;;; In 4x mode, four port's status is OR'd together
;;;
set_led:
        ld      b, 4                ; Number of ports to combine into this LED
        call    set_port_led
        ret
'''

    def __init__(self, label, serdes, leds, tx_polarity_flip, 
                 rx_polarity_flip, tx_lane_map, rx_lane_map):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.Tomahawk100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   leds=leds,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   rx_lane_map=rx_lane_map,
                                                   oversub=True,
                                                   unit=0)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if sub != 0:
            # For breakout modes only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        num_lp = self.num_logical_ports(mode)
        if num_lp == 4:
            call = (fn_name + "_4x", self._set_led_4x, led_port)
        elif num_lp == 2:
            call = (fn_name + "_2x", self._set_led_2x, led_port)
        else:
            call = (fn_name + "_1x", self._set_led_1x, led_port)

        for led in led_index_list:
            set_calls[led] = call
            for x in range(1,6):
                set_calls[led + x] = None

        return set_calls

class AcctonOMP800LED(cumulus.ledup.TomahawkLEDController):
    led_functions = (AcctonOMP800QSFP28._set_led_common)
    num_leds = 96

class AcctonOMP800Switch(cumulus.platform.BCMSwitch):
    ledup = AcctonOMP800LED
    led_controllers = set([1]) # 0 and 2 are unused
    _qsfp28 = AcctonOMP800QSFP28
    _fab = AcctonOMP800Fabric
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (

        # Line Card Tomahawk A front panel ports
        _qsfp28(label="1",  serdes=( 32,  33,  34,  35), leds=(1,   0),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x3201, rx_lane_map=0x0213),
        _qsfp28(label="2",  serdes=( 36,  37,  38,  39), leds=(1,   6),
              tx_polarity_flip=0x0007, rx_polarity_flip=0x000f,
              tx_lane_map=0x0213, rx_lane_map=0x2310),
        _qsfp28(label="3",  serdes=( 40,  41,  42,  43), leds=(1,  12),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x3201, rx_lane_map=0x0213),
        _qsfp28(label="4",  serdes=( 44,  45,  46,  47), leds=(1,  18),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x000f,
              tx_lane_map=0x0132, rx_lane_map=0x2013),
        _qsfp28(label="5",  serdes=( 48,  49,  50,  51), leds=(1,  24),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x000f,
              tx_lane_map=0x0132, rx_lane_map=0x0213),
        _qsfp28(label="6",  serdes=( 52,  53,  54,  55), leds=(1,  30),
              tx_polarity_flip=0x0008, rx_polarity_flip=0x0000,
              tx_lane_map=0x0213, rx_lane_map=0x1023),
        _qsfp28(label="7",  serdes=( 56,  57,  58,  59), leds=(1,  36),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x000f,
              tx_lane_map=0x0132, rx_lane_map=0x0213),
        _qsfp28(label="8",  serdes=( 60,  61,  62,  63), leds=(1,  42),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x0132, rx_lane_map=0x1023),
        _qsfp28(label="9",  serdes=( 64,  65,  66,  67), leds=(1,  48),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x3201, rx_lane_map=0x3120),
        _qsfp28(label="10", serdes=( 68,  69,  70,  71), leds=(1,  54),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0004,
              tx_lane_map=0x0132, rx_lane_map=0x2031),
        _qsfp28(label="11", serdes=( 72,  73,  74,  75), leds=(1,  60),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x0132, rx_lane_map=0x1023),
        _qsfp28(label="12", serdes=( 76,  77,  78,  79), leds=(1,  66),
              tx_polarity_flip=0x0002, rx_polarity_flip=0x0004,
              tx_lane_map=0x1302, rx_lane_map=0x1032),
        _qsfp28(label="13", serdes=( 80,  81,  82,  83), leds=(1,  72),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x000f,
              tx_lane_map=0x0132, rx_lane_map=0x3201),
        _qsfp28(label="14", serdes=( 84,  85,  86,  87), leds=(1,  78),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000f,
              tx_lane_map=0x3201, rx_lane_map=0x0132),
        _qsfp28(label="15", serdes=( 88,  89,  90,  91), leds=(1,  84),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x000f,
              tx_lane_map=0x0132, rx_lane_map=0x3201),
        _qsfp28(label="16", serdes=( 92,  93,  94,  95), leds=(1,  90),
              tx_polarity_flip=0x0002, rx_polarity_flip=0x000b,
              tx_lane_map=0x1302, rx_lane_map=0x3120),

        # Line Card Tomahawk A backplane right
        _fab(label="0",  serdes=( 96,  97,  98,  99),
              tx_polarity_flip=0x000d, rx_polarity_flip=0x000e,
              tx_lane_map=0x3210, rx_lane_map=0x2301),
        _fab(label="1",  serdes=(100, 101, 102, 103),
              tx_polarity_flip=0x0001, rx_polarity_flip=0x000f,
              tx_lane_map=0x0213, rx_lane_map=0x0123),
        _fab(label="2",  serdes=(104, 105, 106, 107),
              tx_polarity_flip=0x000a, rx_polarity_flip=0x000d,
              tx_lane_map=0x3210, rx_lane_map=0x3210),
        _fab(label="3",  serdes=(108, 109, 110, 111),
              tx_polarity_flip=0x000b, rx_polarity_flip=0x0007,
              tx_lane_map=0x0213, rx_lane_map=0x1032),

        # Line Card Tomahawk A whisper cable right
        _fab(label="4",  serdes=(112, 113, 114, 115),
              tx_polarity_flip=0x000c, rx_polarity_flip=0x0000,
              tx_lane_map=0x1032, rx_lane_map=0x0213),
        _fab(label="5",  serdes=(116, 117, 118, 119),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000f,
              tx_lane_map=0x2301, rx_lane_map=0x1032),
        _fab(label="6",  serdes=(120, 121, 122, 123),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000f,
              tx_lane_map=0x2301, rx_lane_map=0x1032),
        _fab(label="7",  serdes=(124, 125, 126, 127),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000f,
              tx_lane_map=0x1032, rx_lane_map=0x2301),

        # Line Card Tomahawk A whisper cable left
        _fab(label="8",  serdes=(  0,   1,   2,   3),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000f,
              tx_lane_map=0x2301, rx_lane_map=0x1032),
        _fab(label="9",  serdes=(  4,   5,   6,   7),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000f,
              tx_lane_map=0x2301, rx_lane_map=0x1032),
        _fab(label="10", serdes=(  8,   9,  10,  11),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000f,
              tx_lane_map=0x2301, rx_lane_map=0x1032),
        _fab(label="11", serdes=( 12,  13,  14,  15),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000f,
              tx_lane_map=0x2301, rx_lane_map=0x1032),

        # Line Card Tomahawk A backplane left
        _fab(label="12", serdes=( 16,  17,  18,  19),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0001,
              tx_lane_map=0x0213, rx_lane_map=0x1302),
        _fab(label="13", serdes=( 20,  21,  22,  23),
              tx_polarity_flip=0x0001, rx_polarity_flip=0x0007,
              tx_lane_map=0x3210, rx_lane_map=0x3210),
        _fab(label="14", serdes=( 24,  25,  26,  27),
              tx_polarity_flip=0x000a, rx_polarity_flip=0x000f,
              tx_lane_map=0x0213, rx_lane_map=0x0123),
        _fab(label="15", serdes=( 28,  29,  30,  31),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0007,
              tx_lane_map=0x3210, rx_lane_map=0x3210),
    )


class AcctonOMP800(cumulus.platform.Platform):
    name = 'accton,omp800_lca'
    chip = cumulus.platform.TomahawkChip()
    switch = AcctonOMP800Switch(chip)
    cpld = None
    statusleds = None
    pwms = None
    sensors = AcctonOMP800Sensors()
    chassis = cumulus.chassis.probe()
    bmc = AcctonOMP800BMC(chassis)

Platform = AcctonOMP800
