#
# Copyright 2016.  Cumulus Networks, Inc.
#
import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class Wedge100QSFP28(cumulus.platform.Tomahawk100GPort):
    '''
;;;
;;; Each pair of ports (top and bottom) has four RGB LEDs (six LEDs per
;;; port).  Only the two outer LEDs are used to indicate link/activity,
;;; and the LED will only be used in green, so the red and blue LEDs
;;; will always be off.  The inner RGB LEDs will always be off.
;;;
    '''
    _set_led_common = '''
;;;
;;; common LED routines
;;;
set_led_green_1x:
        push    PORTSTATUS_ONE  ; red always off
        pack
        push    PORTSTATUS_ONE  ; blue always off
        pack

	port	a
	pushst	PORTSTATUS_RX
	pushst	PORTSTATUS_TX
	tor
	push	(BLINK_STATE_P)
	tand
	tinv
	pushst	PORTSTATUS_LINKEN
	tand
	tinv
        pack  ; green indicates link/activity
        ret
    
set_led_green_2x:
        push    PORTSTATUS_ONE  ; red always off
        pack
        push    PORTSTATUS_ONE  ; blue always off
        pack

	port	a
	pushst	PORTSTATUS_RX
	pushst	PORTSTATUS_TX
	tor

	inc	a
	inc	a
        port    a
	pushst	PORTSTATUS_RX
	pushst	PORTSTATUS_TX
	tor

	tor

	push	(BLINK_STATE_P)
	tand
	tinv

	pushst	PORTSTATUS_LINKEN

	dec	a
	dec	a
	port	a
	pushst	PORTSTATUS_LINKEN
	tor

        tand
	tinv
        
        pack  ; green indicates link/activity
        ret
    
set_led_green_4x:
        push    PORTSTATUS_ONE  ; red always off
        pack
        push    PORTSTATUS_ONE  ; blue always off
        pack

	port	a
	pushst	PORTSTATUS_RX
	pushst	PORTSTATUS_TX
	tor

	inc	a
	port	a
	pushst	PORTSTATUS_RX
	pushst	PORTSTATUS_TX
	tor
	tor

	inc	a
	port	a
	pushst	PORTSTATUS_RX
	pushst	PORTSTATUS_TX
	tor
	tor

	inc	a
	port	a
	pushst	PORTSTATUS_RX
	pushst	PORTSTATUS_TX
	tor
	tor

	push	(BLINK_STATE_P)
	tand
	tinv
	
	pushst	PORTSTATUS_LINKEN

	dec	a
	port	a
	pushst	PORTSTATUS_LINKEN
	tor	

	dec	a
	port	a
	pushst	PORTSTATUS_LINKEN
	tor	

	dec	a
	port	a
	pushst	PORTSTATUS_LINKEN
	tor	

	tand
	tinv
        pack  ; green indicates link/activity
        ret
    
set_led_off:
        push    PORTSTATUS_ONE  ; red always off
        pack
        push    PORTSTATUS_ONE  ; blue always off
        pack
        push    PORTSTATUS_ONE  ; green off
        pack
        ret
'''

    _set_led_1x = '''
;;;
;;; In 1x mode, only the outer RGB LED is used.  The LED will be green
;;; to indicate link/activity. The inner RGB LED will always be off.
;;;
set_led:
        call    set_led_green_1x
        call    set_led_off
        ret
'''
    _set_led_1x_swapped = '''
;;;
;;; These 1x mode LEDs are swapped.
;;;
set_led:
        call    set_led_off
        call    set_led_green_1x
        ret
'''
    
    _set_led_2x = '''
;;;
;;; 2x mode.
;;;
set_led:
        call    set_led_green_2x
        call    set_led_off
        ret
'''
    _set_led_2x_swapped = '''
;;;
;;; These 2x mode LEDs are swapped.
;;;
set_led:
        call    set_led_off
        call    set_led_green_2x
        ret
'''

    _set_led_4x = '''
;;;
;;; 4x mode.
;;;
set_led:
        call    set_led_green_4x
        call    set_led_off
        ret
'''
    _set_led_4x_swapped = '''
;;;
;;; These 4x mode LEDs are swapped.
;;;
set_led:
        call    set_led_off
        call    set_led_green_4x
        ret
'''

    def __init__(self, label, serdes, rx_lane_map, tx_lane_map, leds):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.Tomahawk100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   rx_lane_map=rx_lane_map,
                                                   tx_lane_map=tx_lane_map,
                                                   leds=leds,
                                                   oversub=True,
                                                   unit=0)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if sub != 0:
            # For breakout modes only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        num_lp = self.num_logical_ports(mode)
        swapped_leds = [4,20,36,40,52]
        if num_lp == 4:
            if led_port in swapped_leds:
                call = (fn_name + "_4x_swapped", self._set_led_4x_swapped, led_port)
            else:
            	call = (fn_name + "_4x", self._set_led_4x, led_port)
        elif num_lp == 2:
            if led_port in swapped_leds:
                call = (fn_name + "_2x_swapped", self._set_led_2x_swapped, led_port)
            else:
                call = (fn_name + "_2x", self._set_led_2x, led_port)
        else:
            if led_port in swapped_leds:
                call = (fn_name + "_1x_swapped", self._set_led_1x_swapped, led_port)
            else:
                call = (fn_name + "_1x", self._set_led_1x, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0]+1] = None
        set_calls[led_index_list[0]+2] = None
        set_calls[led_index_list[0]+3] = None
        set_calls[led_index_list[0]+4] = None
        set_calls[led_index_list[0]+5] = None

        return set_calls

class Wedge100LED(cumulus.ledup.TomahawkLEDController):
    led_functions = (Wedge100QSFP28._set_led_common)
    num_leds = 96

class Wedge100Switch(cumulus.platform.BCMSwitch):
    ledup = Wedge100LED
    led_controllers = set([0, 1])
    _qsfp28 = Wedge100QSFP28
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (
        # FC28-31
        _qsfp28(label="1",  serdes=(116, 117, 118, 119), rx_lane_map=0x2301, tx_lane_map=0x3210, leds=(0,18)),
        _qsfp28(label="2",  serdes=(112, 113, 114, 115), rx_lane_map=0x3210, tx_lane_map=0x1032, leds=(0,12)),
        _qsfp28(label="3",  serdes=(124, 125, 126, 127), rx_lane_map=0x1032, tx_lane_map=0x1032, leds=(0,6)),
        _qsfp28(label="4",  serdes=(120, 121, 122, 123), rx_lane_map=0x3210, tx_lane_map=0x3210, leds=(0,0)),

        # FC0-3
        _qsfp28(label="5",  serdes=(4, 5, 6, 7),         rx_lane_map=0x3210, tx_lane_map=0x3210, leds=(0,90)),
        _qsfp28(label="6",  serdes=(0, 1, 2, 3),         rx_lane_map=0x3210, tx_lane_map=0x3210, leds=(0,84)),
        _qsfp28(label="7",  serdes=(12, 13, 14, 15),     rx_lane_map=0x3210, tx_lane_map=0x3210, leds=(0,78)),
        _qsfp28(label="8",  serdes=(8, 9, 10, 11),       rx_lane_map=0x3210, tx_lane_map=0x3210, leds=(0,72)),

        # FC4-7
        _qsfp28(label="9",  serdes=(20, 21, 22, 23),     rx_lane_map=0x2301, tx_lane_map=0x1032, leds=(0,66)),
        _qsfp28(label="10", serdes=(16, 17, 18, 19),     rx_lane_map=0x1032, tx_lane_map=0x3210, leds=(0,60)),
        _qsfp28(label="11", serdes=(28, 29, 30, 31),     rx_lane_map=0x3210, tx_lane_map=0x1032, leds=(0,54)),
        _qsfp28(label="12", serdes=(24, 25, 26, 27),     rx_lane_map=0x1032, tx_lane_map=0x3210, leds=(0,48)),

        # FC8-11
        _qsfp28(label="13", serdes=(36, 37, 38, 39),     rx_lane_map=0x2301, tx_lane_map=0x1032, leds=(1,90)),
        _qsfp28(label="14", serdes=(32, 33, 34, 35),     rx_lane_map=0x1032, tx_lane_map=0x3210, leds=(1,84)),
        _qsfp28(label="15", serdes=(44, 45, 46, 47),     rx_lane_map=0x3210, tx_lane_map=0x1032, leds=(1,78)),
        _qsfp28(label="16", serdes=(40, 41, 42, 43),     rx_lane_map=0x1032, tx_lane_map=0x3210, leds=(1,72)),
        
        # FC12-15
        _qsfp28(label="17", serdes=(52, 53, 54, 55),     rx_lane_map=0x1032, tx_lane_map=0x1032, leds=(1,66)),
        _qsfp28(label="18", serdes=(48, 49, 50, 51),     rx_lane_map=0x1032, tx_lane_map=0x1032, leds=(1,60)),
        _qsfp28(label="19", serdes=(60, 61, 62, 63),     rx_lane_map=0x1032, tx_lane_map=0x1032, leds=(1,54)),
        _qsfp28(label="20", serdes=(56, 57, 58, 59),     rx_lane_map=0x1032, tx_lane_map=0x1032, leds=(1,48)),

        # FC16-19
        _qsfp28(label="21", serdes=(68, 69, 70, 71),     rx_lane_map=0x1032, tx_lane_map=0x1032, leds=(1,42)),
        _qsfp28(label="22", serdes=(64, 65, 66, 67),     rx_lane_map=0x3210, tx_lane_map=0x3210, leds=(1,36)),
        _qsfp28(label="23", serdes=(76, 77, 78, 79),     rx_lane_map=0x0213, tx_lane_map=0x3210, leds=(1,30)),
        _qsfp28(label="24", serdes=(72, 73, 74, 75),     rx_lane_map=0x1032, tx_lane_map=0x1032, leds=(1,24)),

        # FC20-23
        _qsfp28(label="25", serdes=(84, 85, 86, 87),     rx_lane_map=0x0123, tx_lane_map=0x3210, leds=(1,18)),
        _qsfp28(label="26", serdes=(80, 81, 82, 83),     rx_lane_map=0x3210, tx_lane_map=0x1032, leds=(1,12)),
        _qsfp28(label="27", serdes=(92, 93, 94, 95),     rx_lane_map=0x1032, tx_lane_map=0x3210, leds=(1,6)),
        _qsfp28(label="28", serdes=(88, 89, 90, 91),     rx_lane_map=0x3210, tx_lane_map=0x1032, leds=(1,0)),
        
        # FC24-27
        _qsfp28(label="29", serdes=(100, 101, 102, 103), rx_lane_map=0x0123, tx_lane_map=0x1032, leds=(0,42)),
        _qsfp28(label="30", serdes=(96, 97, 98, 99),     rx_lane_map=0x1032, tx_lane_map=0x1032, leds=(0,36)),
        _qsfp28(label="31", serdes=(108, 109, 110, 111), rx_lane_map=0x1032, tx_lane_map=0x3210, leds=(0,30)),
        _qsfp28(label="32", serdes=(104, 105, 106, 107), rx_lane_map=0x3210, tx_lane_map=0x1032, leds=(0,24)),
    )

vhwmon_path = '/sys/class/misc/vhwmon/bmc:'

class Wedge100CoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    temp_max = 90
    temp_crit = 95
    no_shutdown = True

class Wedge100Temp1(Wedge100CoreTemp):
    name = 'Temp1'
    description = 'Core 0'
    driver_hwmon = ['temp2']

class Wedge100Temp2(Wedge100CoreTemp):
    name = 'Temp2'
    description = 'Core 1'
    driver_hwmon = ['temp3']

class Wedge100Temp3(Wedge100CoreTemp):
    name = 'Temp3'
    description = 'Core 2'
    driver_hwmon = ['temp4']

class Wedge100Temp4(Wedge100CoreTemp):
    name = 'Temp4'
    description = 'Core 3'
    driver_hwmon = ['temp5']

class Wedge100Temp5(cumulus.sensors.Temp_Unit):
    name = 'Temp5'
    description = 'Outlet Right Temp'
    driver_path = vhwmon_path + 'tmp75_i2c_8_48'
    driver_hwmon = ['temp1']

class Wedge100Temp6(cumulus.sensors.Temp_Unit):
    name = 'Temp6'
    description = 'Inlet Right Temp'
    driver_path = vhwmon_path + 'tmp75_i2c_3_4c'
    driver_hwmon = ['temp1']

class Wedge100Temp7(cumulus.sensors.Temp_Unit):
    name = 'Temp7'
    description = 'Switch Temp'
    driver_path = vhwmon_path + 'tmp75_i2c_3_4b'
    driver_hwmon = ['temp1']

class Wedge100Temp8(cumulus.sensors.Temp_Unit):
    name = 'Temp8'
    description = 'Inlet Left Temp'
    driver_path = vhwmon_path + 'tmp75_i2c_3_4a'
    driver_hwmon = ['temp1']

class Wedge100Temp9(cumulus.sensors.Temp_Unit):
    name = 'Temp9'
    description = 'Outlet Left Temp'
    driver_path = vhwmon_path + 'tmp75_i2c_8_49'
    driver_hwmon = ['temp1']

class Wedge100Temp10(cumulus.sensors.Temp_Unit):
    name = 'Temp10'
    description = 'Inlet Middle Temp'
    driver_path = vhwmon_path + 'tmp75_i2c_3_49'
    driver_hwmon = ['temp1']

class Wedge100Temp11(cumulus.sensors.Temp_Unit):
    name = 'Temp11'
    description = 'Outlet Middle Temp'
    driver_path = vhwmon_path + 'tmp75_i2c_3_48'
    driver_hwmon = ['temp1']

class Wedge100Temp12(cumulus.sensors.Temp_Unit):
    name = 'Temp12'
    description = 'Memory Temp'
    driver_path = vhwmon_path + 'com_e_driver_i2c_4_33'
    driver_hwmon = ['temp1']

class Wedge100Temp13(cumulus.sensors.Temp_Unit):
    name = 'Temp13'
    description = 'CPU Temp'
    driver_path = vhwmon_path + 'com_e_driver_i2c_4_33'
    driver_hwmon = ['temp2']

class Wedge100Fan(cumulus.sensors.Fan_Unit):
    driver_path = vhwmon_path + 'fancpld_i2c_8_33'
    no_pwm = True

class Wedge100Fan1(Wedge100Fan):
    name = 'Fan1'
    description = 'Fan 1 front'
    driver_hwmon = ['fan5']

class Wedge100Fan2(Wedge100Fan):
    name = 'Fan2'
    description = 'Fan 1 rear'
    driver_hwmon = ['fan6']

class Wedge100Fan3(Wedge100Fan):
    name = 'Fan3'
    description = 'Fan 2 front'
    driver_hwmon = ['fan3']

class Wedge100Fan4(Wedge100Fan):
    name = 'Fan4'
    description = 'Fan 2 rear'
    driver_hwmon = ['fan8']

class Wedge100Fan5(Wedge100Fan):
    name = 'Fan5'
    description = 'Fan 3 front'
    driver_hwmon = ['fan2']

class Wedge100Fan6(Wedge100Fan):
    name = 'Fan6'
    description = 'Fan 3 rear'
    driver_hwmon = ['fan9']

class Wedge100Fan7(Wedge100Fan):
    name = 'Fan7'
    description = 'Fan 4 front'
    driver_hwmon = ['fan1']

class Wedge100Fan8(Wedge100Fan):
    name = 'Fan8'
    description = 'Fan 4 rear'
    driver_hwmon = ['fan7']

class Wedge100Fan9(Wedge100Fan):
    name = 'Fan9'
    description = 'Fan 5 front'
    driver_hwmon = ['fan4']

class Wedge100Fan10(Wedge100Fan):
    name = 'Fan10'
    description = 'Fan 5 rear'
    driver_hwmon = ['fan10']

class Wedge100PSU(cumulus.sensors.PSU_Unit):
    cpld_path = '/sys/bus/i2c/devices/0-0032'

    def update_attrs(self, path, hwmon):
        '''
        the Wedge100 CPLD does not provide a psu_pwrX_all_ok which smond
         expects to see, we will construct one using the available data.
        '''
        ret = cumulus.sensors.PSU_Unit.update_attrs(self, path, hwmon)
        oks = ['psu_pwr%u_dc_ok' % self.psu_num,
               'psu_pwr%u_ac_ok' % self.psu_num,]
        status = '1'
        for ok in oks:
            try:
                if int(self.attrs.get(ok)) != 1:
                    status = '0'
            except:
                continue
        if int(self.attrs.get('psu_pwr%u_alarm') == 1):
            status = '0'
        self.attrs['psu_pwr%u_all_ok' % self.psu_num] = status
        return ret

class Wedge100PSU1(Wedge100PSU):
    name = 'PSU1'
    psu_num = 1
    cpld_hwmon = ['psu_pwr1']

class Wedge100PSU2(Wedge100PSU):
    name = 'PSU2'
    psu_num = 2
    cpld_hwmon = ['psu_pwr2']

class Wedge100SystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_path = '/sys/bus/i2c/devices/0-0032'
    led_location = 'led_system1'

class Wedge100LEDs():
    units = (
        Wedge100SystemLED(),
    )

class Wedge100Sensors():
    units = (Wedge100Temp1(),
             Wedge100Temp2(),
             Wedge100Temp3(),
             Wedge100Temp4(),
             Wedge100Temp5(),
             Wedge100Temp6(),
             Wedge100Temp7(),
             Wedge100Temp8(),
             Wedge100Temp9(),
             Wedge100Temp10(),
             Wedge100Temp11(),
             Wedge100Temp12(),
             Wedge100Temp13(),
             Wedge100Fan1(),
             Wedge100Fan2(),
             Wedge100Fan3(),
             Wedge100Fan4(),
             Wedge100Fan5(),
             Wedge100Fan6(),
             Wedge100Fan7(),
             Wedge100Fan8(),
             Wedge100Fan9(),
             Wedge100Fan10(),
             Wedge100PSU1(),
             Wedge100PSU2(),
            )

class Wedge100BMC(cumulus.platform.BMC):
    name = "bmc"
    type = "openbmc"
    ipv6_addr = "fe80::1%usb0"

class Wedge100(cumulus.platform.Platform):
    name = 'accton,wedge100_32x'
    chip = cumulus.platform.TomahawkChip()
    switch = Wedge100Switch(chip)
    cpld = None
    statusleds = None
    pwms = None
    sensors = Wedge100Sensors()
    bmc = Wedge100BMC()

Platform = Wedge100
