#
# Copyright (C) 2017 Cumulus Networks, Inc. All rights reserved
#
# This software is subject to the Cumulus Networks End User License
# Agreement available  at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt
#

import subprocess
import json
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sfp
import cumulus.sensors

class CelRedstoneVSFPp(cumulus.platform.Maverick10GPort):

    _set_led = '''
;;; CelRedstoneVSFPp
;;
;;; Each port on the Redstone-V uses two bits (SFP and QSFP), they
;;;  behave as follows:
;;;
;;;                  bit 0
;;;                0        1
;;;            +--------+-------+
;;;          0 | green  |  off  |
;;;   bit 1    +--------+-------+
;;;          1 |  red   |  off  |
;;;            +--------+-------+
set_led:
set_any_led:
;;; The first bit will indicate link/activity (low active)
;;; The second bit will always be 0
;;;
        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        tinv
        pack

        port 1
        pushst  PORTSTATUS_ZERO ; disable yellow LED
        pack

        ret
'''
    def __init__(self, label, serdes, leds, rx_polarity_flip=None, tx_polarity_flip=None,
                 tx_lane_map=None, rx_lane_map=None):
        cumulus.platform.Maverick10GPort.__init__(self,
                                                  label=label,
                                                  serdes=serdes,
                                                  phy_types=None,
                                                  phy_addrs=None,
                                                  phy_layer='SFP+',
                                                  leds=leds,
                                                  rx_lane_map=rx_lane_map,
                                                  tx_lane_map=tx_lane_map,
                                                  rx_polarity_flip=rx_polarity_flip,
                                                  tx_polarity_flip=tx_polarity_flip,
                                                  unit=0,
                                                  oversub=True)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        call = (fn_name, self._set_led, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0] + 1] = None

        return set_calls

class CelRedstoneVQSFP28(cumulus.platform.Maverick100GPort):
    _set_led = '''
;;; CelRedstoneVQSFPp
;;;
;;; For 40G mode, we use only the first LED to indicate link/activity.
;;; The other 3 LEDs will be off.
;;;
set_led:
        call    set_any_led    ; set the first led green/off

        push    PORTSTATUS_ONE ; 1, 1 => led 2 off
        pack
        push    PORTSTATUS_ONE
        pack

        push    PORTSTATUS_ONE ; 1, 1 => led 3 off
        pack
        push    PORTSTATUS_ONE
        pack

        push    PORTSTATUS_ONE ; 1, 1 => led 4 off
        pack
        push    PORTSTATUS_ONE
        pack

        ret
'''
    _set_led_4x = '''
;;; CelRedstoneVQSFPp_4x
;;
;;  For 4xG mode, we use an LED for each port.
;;
set_led:
        call    set_any_led
        inc     a
        call    set_any_led
        inc     a
        call    set_any_led
        inc     a
        call    set_any_led

        ret
'''
    def __init__(self, label, serdes, leds, rx_lane_map=None, tx_lane_map=None, rx_polarity_flip=None, tx_polarity_flip=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.Maverick100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   leds=leds,
                                                   rx_lane_map=rx_lane_map,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   unit=0,
                                                   oversub=True)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if mode.startswith("4x") and sub != 0:
            # For 4x mode only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        if mode.startswith("4x"):
            call = (fn_name + "_4x", self._set_led_4x, led_port)
        else:
            call = (fn_name, self._set_led, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0]+1] = None
        set_calls[led_index_list[0]+2] = None
        set_calls[led_index_list[0]+3] = None
        set_calls[led_index_list[0]+4] = None
        set_calls[led_index_list[0]+5] = None
        set_calls[led_index_list[0]+6] = None
        set_calls[led_index_list[0]+7] = None


        return set_calls

class CelRedstoneVLED(cumulus.ledup.TridentTwoLEDController):
    num_leds = 72

class CelRedstoneVSwitch(cumulus.platform.BCMSwitch):
    ledup = CelRedstoneVLED
    _sfp = CelRedstoneVSFPp
    _qsfp28 = CelRedstoneVQSFP28

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (
        _sfp(label="1",  serdes=( 4,), leds=(0,  0), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="2",  serdes=( 5,), leds=(0,  2), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp(label="3",  serdes=( 6,), leds=(0,  4), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="4",  serdes=( 7,), leds=(0,  6), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True ),

        _sfp(label="5",  serdes=( 8,), leds=(0,  8), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="6",  serdes=( 9,), leds=(0, 10), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp(label="7",  serdes=(10,), leds=(0, 12), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="8",  serdes=(11,), leds=(0, 14), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True ),

        _sfp(label="9",  serdes=(12,), leds=(0, 16), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="10", serdes=(13,), leds=(0, 18), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp(label="11", serdes=(14,), leds=(0, 20), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="12", serdes=(15,), leds=(0, 22), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True ),

        _sfp(label="13", serdes=(16,), leds=(0, 24), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="14", serdes=(17,), leds=(0, 26), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="15", serdes=(18,), leds=(0, 28), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="16", serdes=(19,), leds=(0, 30), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True ),
        
        _sfp(label="17", serdes=(20,), leds=(0, 32), tx_lane_map=0x2301, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp(label="18", serdes=(21,), leds=(0, 34), tx_lane_map=0x2301, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp(label="19", serdes=(22,), leds=(0, 36), tx_lane_map=0x2301, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp(label="20", serdes=(23,), leds=(0, 38), tx_lane_map=0x2301, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True ),
        
        _sfp(label="21", serdes=(24,), leds=(0, 40), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp(label="22", serdes=(25,), leds=(0, 42), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="23", serdes=(26,), leds=(0, 44), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp(label="24", serdes=(27,), leds=(0, 46), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False),
        
        _sfp(label="25", serdes=(36,), leds=(1, 0), tx_lane_map=0x1032, rx_lane_map=0x2031, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="26", serdes=(37,), leds=(1, 2), tx_lane_map=0x1032, rx_lane_map=0x2031, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="27", serdes=(38,), leds=(1, 4), tx_lane_map=0x1032, rx_lane_map=0x2031, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="28", serdes=(39,), leds=(1, 6), tx_lane_map=0x1032, rx_lane_map=0x2031, tx_polarity_flip=True,  rx_polarity_flip=True ),
        
        _sfp(label="29", serdes=(40,), leds=(1, 8), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="30", serdes=(41,), leds=(1, 10), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="31", serdes=(42,), leds=(1, 12), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="32", serdes=(43,), leds=(1, 14), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        
        _sfp(label="33", serdes=(44,), leds=(1, 16), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="34", serdes=(45,), leds=(1, 18), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="35", serdes=(46,), leds=(1, 20), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="36", serdes=(47,), leds=(1, 22), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        
        _sfp(label="37", serdes=(48,), leds=(1, 24), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="38", serdes=(49,), leds=(1, 26), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="39", serdes=(50,), leds=(1, 28), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="40", serdes=(51,), leds=(1, 30), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True ),
        
        _sfp(label="41", serdes=(52,), leds=(1, 32), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp(label="42", serdes=(53,), leds=(1, 34), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp(label="43", serdes=(54,), leds=(1, 36), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp(label="44", serdes=(55,), leds=(1, 38), tx_lane_map=0x1032, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False),
        
        _sfp(label="45", serdes=(56,), leds=(1, 40), tx_lane_map=0x2301, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp(label="46", serdes=(57,), leds=(1, 42), tx_lane_map=0x2301, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp(label="47", serdes=(58,), leds=(1, 44), tx_lane_map=0x2301, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp(label="48", serdes=(59,), leds=(1, 46), tx_lane_map=0x2301, rx_lane_map=0x0321, tx_polarity_flip=True,  rx_polarity_flip=True ),

        _qsfp28(label="49", serdes=(32, 33, 34, 35), leds=(0, 56), tx_lane_map=0x1203, rx_lane_map=0x3012, tx_polarity_flip=0xa, rx_polarity_flip=0x0),
        _qsfp28(label="50", serdes=(28, 29, 30, 31), leds=(0, 48), tx_lane_map=0x2031, rx_lane_map=0x1230, tx_polarity_flip=0xf, rx_polarity_flip=0xb),
        _qsfp28(label="51", serdes=(68, 69, 70, 71), leds=(1, 56), tx_lane_map=0x2130, rx_lane_map=0x2031, tx_polarity_flip=0x5, rx_polarity_flip=0xb),
        _qsfp28(label="52", serdes=(64, 65, 66, 67), leds=(1, 48), tx_lane_map=0x2031, rx_lane_map=0x1203, tx_polarity_flip=0xa, rx_polarity_flip=0xd),
    )

# sysfs paths


CelRedstoneV_cpld_path = '/sys/bus/platform/devices/cel_redstone_v_cpld.0'
CelRedstoneV_core_path = '/sys/devices/platform/coretemp.0'
CelRedstoneV_lm75_u6_path = '/sys/bus/i2c/devices/11-004c'
CelRedstoneV_lm75_u8_path = '/sys/bus/i2c/devices/11-0049'
CelRedstoneV_lm75_u11_path = '/sys/bus/i2c/devices/11-004a'
CelRedstoneV_lm75_u24_path = '/sys/bus/i2c/devices/11-004b'
CelRedstoneV_psu1_path = '/sys/bus/i2c/devices/10-0058'
CelRedstoneV_psu2_path = '/sys/bus/i2c/devices/12-0059'

class CelRedstoneVCPLD(cumulus.cpld.SysFS):
    path = CelRedstoneV_cpld_path

# Intel Denverton CPU

class CelRedstoneVTemp1(cumulus.sensors.Temp_Unit):
    name = 'Temp1'
    description = 'Intel CPU die sensor'
    driver_path = CelRedstoneV_core_path
    driver_hwmon = ['temp1']
    temp_crit = 95

class CelRedstoneVTemp2(cumulus.sensors.Temp_Unit):
    name = 'Temp2'
    description = 'Intel CPU die sensor'
    driver_path = CelRedstoneV_core_path
    driver_hwmon = ['temp8']
    temp_crit = 95

class CelRedstoneVTemp3(cumulus.sensors.Temp_Unit):
    name = 'Temp3'
    description = 'Intel CPU die sensor'
    driver_path = CelRedstoneV_core_path
    driver_hwmon = ['temp14']
    temp_crit = 95

# U6

class CelRedstoneVTemp4(cumulus.sensors.Temp_Unit):
    name = 'Temp4'
    description = 'Outlet Sensor'
    driver_path = CelRedstoneV_lm75_u6_path
    temp_crit = 60
    temp_max = 48

# U8

class CelRedstoneVTemp5(cumulus.sensors.Temp_Unit):
    name = 'Temp5'
    description = 'Near SFPs'
    driver_path = CelRedstoneV_lm75_u8_path
    temp_crit = 90
    temp_max = 80

# U11

class CelRedstoneVTemp6(cumulus.sensors.Temp_Unit):
    name = 'Temp6'
    description = 'Inlet'
    driver_path = CelRedstoneV_lm75_u11_path
    temp_crit = 60
    temp_max = 48

# U24

class CelRedstoneVTemp7(cumulus.sensors.Temp_Unit):
    name = 'Temp7'
    description = 'Switch Asic'
    driver_path = CelRedstoneV_lm75_u24_path
    temp_crit = 90
    temp_max = 80

# Maverick Internal Temp Sensors
class CelRedstoneVBCMTemp(cumulus.sensors.Temp_Unit):
    description  = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path  = '/cumulus/switchd/run'
    temp_max     = 100
    temp_crit    = 105
    temp_fan_min = 42
    temp_fan_max = 69

class CelRedstoneVBCMTemp1(CelRedstoneVBCMTemp):
    name = 'Temp8'
    driver_hwmon = ['temp1']

class CelRedstoneVBCMTemp2(CelRedstoneVBCMTemp):
    name = 'Temp9'
    driver_hwmon = ['temp2']

class CelRedstoneVBCMTemp3(CelRedstoneVBCMTemp):
    name = 'Temp10'
    driver_hwmon = ['temp3']

class CelRedstoneVBCMTemp4(CelRedstoneVBCMTemp):
    name = 'Temp11'
    driver_hwmon = ['temp4']

class CelRedstoneVBCMTemp5(CelRedstoneVBCMTemp):
    name = 'Temp12'
    driver_hwmon = ['temp5']

class CelRedstoneVBCMTemp6(CelRedstoneVBCMTemp):
    name = 'Temp13'
    driver_hwmon = ['temp6']

class CelRedstoneVBCMTemp7(CelRedstoneVBCMTemp):
    name = 'Temp14'
    driver_hwmon = ['temp7']

class CelRedstoneVBCMTemp8(CelRedstoneVBCMTemp):
    name = 'Temp15'
    driver_hwmon = ['temp8']

class CelRedstoneVBCMTemp9(CelRedstoneVBCMTemp):
    name = 'Temp16'
    driver_hwmon = ['temp9']

class CelRedstoneVFan(cumulus.sensors.Fan_Unit):
    fan_min = 5200
    fan_max = 18000
    hwmon_search = False
    cpld_path = CelRedstoneV_cpld_path
    driver_path = cpld_path

class CelRedstoneVFan1(CelRedstoneVFan):
    name = 'Fan1'
    description = 'Fan Tray 1'
    pwm_hwmon = ['pwm1']
    driver_hwmon = ['fan1']
    fan_present = 'fan1_present'

class CelRedstoneVFan2(CelRedstoneVFan):
    name = 'Fan2'
    description = 'Fan Tray 2'
    pwm_hwmon = ['pwm2']
    driver_hwmon = ['fan2']
    fan_present = 'fan2_present'

class CelRedstoneVFan3(CelRedstoneVFan):
    name = 'Fan3'
    description = 'Fan Tray 3'
    pwm_hwmon = ['pwm3']
    driver_hwmon = ['fan3']
    fan_present = 'fan3_present'

class CelRedstoneVFan4(CelRedstoneVFan):
    name = 'Fan4'
    description = 'Fan Tray 4'
    pwm_hwmon = ['pwm4']
    driver_hwmon = ['fan4']
    fan_present = 'fan4_present'

class CelRedstoneVFan5(CelRedstoneVFan):
    name = 'Fan5'
    description = 'Fan Tray 5'
    pwm_hwmon = ['pwm5']
    driver_hwmon = ['fan5']
    fan_present = 'fan5_present'

class CelRedstoneV_PSU(cumulus.sensors.PSU_Unit):
    init_done = False
    def __init__(self):
        cumulus.sensors.PSU_Unit.__init__(self)
        self.add_state('OK', self.ok_handler)
        self.is_hw_managed = self._check_hw_mgmt()

    def ok_handler(self):
        if self.init_done:
            return
        err_states = ['BAD', 'ABSENT'];
        if self.attrs['prev_state'] in err_states:
            cmd = 'rmmod dps460 && modprobe dps460'
            subprocess.call(cmd, shell=True)
            self.init_done = True

    def find_and_add_attrs(self):
        if self.get_cpld_value(self.psu_present) == '1':
            self.attrs[self.psu_present] = 1
        else:
            self.attrs[self.psu_present] = 0
        if self.get_cpld_value(self.psu_all_ok) == '1':
            self.attrs[self.psu_all_ok] = 1
        else:
            self.attrs[self.psu_all_ok] = 0
        return 0

    def _check_hw_mgmt(self):
        try:
            data = json.loads(subprocess.check_output(['/usr/cumulus/bin/decode-syseeprom', '-j', '-t', self.name.lower()]))
            name = data.get('product_info').get('name').strip()
        except:
            name = ''
        if name in ['DPS550AB6A',]:
            return True
        return False

class CelRedstoneVPSU1(CelRedstoneV_PSU):
    name = 'PSU1'
    psu_present = 'psu_pwr1_present'
    psu_all_ok = 'psu_pwr1_all_ok'

class CelRedstoneVPSU2(CelRedstoneV_PSU):
    name = 'PSU2'
    psu_present = 'psu_pwr2_present'
    psu_all_ok = 'psu_pwr2_all_ok'

class CelRedstoneVPSUTemp(cumulus.sensors.Temp_Unit):
    cpld_path = CelRedstoneV_cpld_path
    temp_max = 60
    temp_crit = 100

    def find_and_add_attrs(self):
        if ((self.get_cpld_value(self.temp_all_ok) == '1')):
            return cumulus.sensors.Temp_Unit.find_and_add_attrs(self)
        else:
            return -1

class CelRedstoneVTemp9(CelRedstoneVPSUTemp):
    name = 'PSU1Temp1'
    description = 'PSU1 Inlet Temp Sensor'
    driver_path = CelRedstoneV_psu1_path
    driver_hwmon = ['temp1']
    temp_all_ok = 'psu_pwr1_all_ok'

class CelRedstoneVTemp10(CelRedstoneVPSUTemp):
    name = 'PSU1Temp2'
    description = 'PSU1 Max Temp Sensor'
    driver_path = CelRedstoneV_psu1_path
    driver_hwmon = ['temp2']
    temp_all_ok = 'psu_pwr1_all_ok'

class CelRedstoneVTemp11(CelRedstoneVPSUTemp):
    name = 'PSU2Temp1'
    description = 'PSU2 Inlet Temp Sensor'
    driver_path = CelRedstoneV_psu2_path
    driver_hwmon = ['temp1']
    temp_all_ok = 'psu_pwr2_all_ok'

class CelRedstoneVTemp12(CelRedstoneVPSUTemp):
    name = 'PSU2Temp2'
    description = 'PSU2 Max Temp Sensor'
    driver_path = CelRedstoneV_psu2_path
    driver_hwmon = ['temp2']
    temp_all_ok = 'psu_pwr2_all_ok'

class CelRedstoneVPSUFan(cumulus.sensors.Fan_Unit):
    cpld_path = CelRedstoneV_cpld_path
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    minpwm = 102
    fan_max = 18000
    fan_min = 7000

    def find_and_add_attrs(self):
        if ((self.get_cpld_value(self.fan_present) == '1') and
            (self.get_cpld_value(self.fan_all_ok) == '1')):
            return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)
        else:
            return -1

class CelRedstoneVFan9(CelRedstoneVPSUFan):
    name = 'PSU1Fan1'
    description = 'PSU1 Fan'
    driver_path = CelRedstoneV_psu1_path
    fan_present = 'psu_pwr1_present'
    fan_all_ok = 'psu_pwr1_all_ok'
    if CelRedstoneVPSU1().is_hw_managed:
        fan_min = 4000

class CelRedstoneVFan10(CelRedstoneVPSUFan):
    name = 'PSU2Fan1'
    description = 'PSU2 Fan'
    driver_path = CelRedstoneV_psu2_path
    fan_present = 'psu_pwr2_present'
    fan_all_ok = 'psu_pwr2_all_ok'
    if CelRedstoneVPSU2().is_hw_managed:
        fan_min = 4000

class CelRedstoneVPWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'
    fans = (CelRedstoneVFan1(),
            CelRedstoneVFan2(),
            CelRedstoneVFan3(),
            CelRedstoneVFan4(),
            CelRedstoneVFan5(),
    )
    temps = (CelRedstoneVTemp1(),
             CelRedstoneVTemp2(),
             CelRedstoneVTemp3(),
             CelRedstoneVTemp4(),
             CelRedstoneVTemp5(),
             CelRedstoneVTemp6(),
             CelRedstoneVTemp7(),
             CelRedstoneVBCMTemp1(),
             CelRedstoneVBCMTemp2(),
             CelRedstoneVBCMTemp3(),
             CelRedstoneVBCMTemp4(),
             CelRedstoneVBCMTemp5(),
             CelRedstoneVBCMTemp6(),
             CelRedstoneVBCMTemp7(),
             CelRedstoneVBCMTemp8(),
             CelRedstoneVBCMTemp9(),
             )

class CelRedstoneVPWM2(cumulus.sensors.PWM_Unit):
    name = 'PSU1 PWM'
    fans = (CelRedstoneVFan9(),
            )
    temps = (CelRedstoneVTemp9(),
             CelRedstoneVTemp10(),
             )

class CelRedstoneVPWM3(cumulus.sensors.PWM_Unit):
    name = 'PSU2 PWM'
    fans = (CelRedstoneVFan10(),
            )
    temps = (CelRedstoneVTemp11(),
             CelRedstoneVTemp12(),
             )

class CelRedstoneVSensors():
    units = (CelRedstoneVPSU1(),
             CelRedstoneVPSU2(),
             CelRedstoneVTemp1(),
             CelRedstoneVTemp2(),
             CelRedstoneVTemp3(),
             CelRedstoneVTemp4(),
             CelRedstoneVTemp5(),
             CelRedstoneVTemp6(),
             CelRedstoneVTemp7(),
             CelRedstoneVBCMTemp1(),
             CelRedstoneVBCMTemp2(),
             CelRedstoneVBCMTemp3(),
             CelRedstoneVBCMTemp4(),
             CelRedstoneVBCMTemp5(),
             CelRedstoneVBCMTemp6(),
             CelRedstoneVBCMTemp7(),
             CelRedstoneVBCMTemp8(),
             CelRedstoneVBCMTemp9(),
             CelRedstoneVFan1(),
             CelRedstoneVFan2(),
             CelRedstoneVFan3(),
             CelRedstoneVFan4(),
             CelRedstoneVFan5(),
            )

class CelRedstoneVPWMs():

    units = (CelRedstoneVPWM1(),)

    def __init__(self):
        return
        # Some Celestica Boxes (specially ones from SuperMicro), have PSUs where
        # the PSU fan speed in managed by hardware and cannot be managed by software elegantly.
        # More on this in CM-9627. So if the said PSU is there, dont manage the PSU fan from
        # PWM. In all other cases, the fan speed is managed by software.
        if not CelRedstoneVPSU1().is_hw_managed:
            self.units = self.units + (CelRedstoneVPWM2(),)

        if not CelRedstoneVPSU2().is_hw_managed:
            self.units = self.units + (CelRedstoneVPWM3(),)

'''
   Celestica Redstone-V LEDs
       System:
         green - all systems ok
         yellow - fan missing or broken, psu missing or broken
       PSU1:
         green - psu1 installed and working
         off - psu1 missing or broken
       PSU2:
         green - psu2 installed and working
         off - psu2 missing or broken
'''

class CelRedstoneVSystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System LED'
    led_location = 'led_system'
    good_led_color = "green"
    bad_led_color = 'yellow_blinking'

class CelRedstoneVLEDs():
    units = (CelRedstoneVSystemLED(),
             )

class CelRedstoneV(cumulus.platform.Platform):
    name = 'cel,redstone_v'
    chip = cumulus.platform.MaverickChip()
    switch = CelRedstoneVSwitch(chip)
    cpld = CelRedstoneVCPLD()
    statusleds = CelRedstoneVLEDs()
    pwms = CelRedstoneVPWMs()
    sensors = CelRedstoneVSensors()

Platform = CelRedstoneV
