#
# Copyright (c) 2018 Cumulus Networks, Inc.  All rights reserved.
#

import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class DellS5048FSFP28(cumulus.platform.Tomahawk25GPort):
    _set_led = '''
;;; DellS5048FSFP28
;;
;;; Each 25G port has one bi-color amber/green LED.
;;;
;;; We use:
;;;   green = on with link, flashed on with activity
;;;   amber = always off
;;;
;;;                        bit 1
;;;                   0             1
;;;            +-------------+-------------+
;;;          0 |    green    | amber flash |
;;;   bit 0    +-------------+-------------+
;;;          1 |    amber    |     off     |
;;;            +-------------+-------------+
;;;
set_led:
        call    set_port_led
        ret
'''

    _set_led_common = '''
set_port_led:
        port a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand                        ; activity_green
        tinv
        pushst  PORTSTATUS_LINKEN   ; link_green
        tand
        tinv
        pop                         ; push the same value twice
        push    cy
        pack
        push    cy
        pack
        ret

set_led_off:
        pushst  PORTSTATUS_ONE
        pack
        pushst  PORTSTATUS_ONE
        pack
        ret
'''

    def __init__(self, label, serdes, leds, rx_polarity_flip=None, tx_polarity_flip=None,
                       tx_lane_map=None, rx_lane_map=None, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp28')
        cumulus.platform.Tomahawk25GPort.__init__(self,
                                                  label=label,
                                                  serdes=serdes,
                                                  phy_types=None,
                                                  phy_addrs=None,
                                                  phy_layer=sfp.type.upper(),
                                                  leds=leds,
                                                  rx_lane_map=rx_lane_map,
                                                  tx_lane_map=tx_lane_map,
                                                  rx_polarity_flip=rx_polarity_flip,
                                                  tx_polarity_flip=tx_polarity_flip,
                                                  pre_emphasis=pre_emphasis,
                                                  unit=0,
                                                  oversub=True)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        call = (fn_name, self._set_led, led_port)

        for led in led_index_list:
            set_calls[led] = call
            set_calls[led+1] = None

        return set_calls

class DellS5048FQSFP28(cumulus.platform.Tomahawk100GPort):
    _set_led = '''
;;; DellS5048FQSFP28
;;;
;;; Each QSFP28 port has four bi-color amber/green LEDs.
;;;
;;; For 100G, 50G, and 40G modes, we use only the first LED (green) to
;;; indicate link/activity.
;;;
set_led:
        call    set_port_led
        call    set_led_off
        call    set_led_off
        call    set_led_off
        ret
'''
    _set_led_2x = '''
;;; DellS5048FQSFP28_2x
;;;
;;;  For 2x50G mode, we use the first and third LEDs, one for each port.
;;;
set_led:
        call    set_port_led
        call    set_led_off
        inc     a
        inc     a
        call    set_port_led
        call    set_led_off
        ret
'''
    _set_led_4x = '''
;;; DellS5048FQSFP28_4x
;;;
;;;  For 4x25G and 4x10G mode, we use all four LEDs, one for each port.
;;;
set_led:
        call    set_port_led
        inc     a
        call    set_port_led
        inc     a
        call    set_port_led
        inc     a
        call    set_port_led
        ret
'''

    def __init__(self, label, serdes, leds,
                 rx_lane_map=None, tx_lane_map=None,
                 rx_polarity_flip=None, tx_polarity_flip=None, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.Tomahawk100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   leds=leds,
                                                   rx_lane_map=rx_lane_map,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   pre_emphasis=pre_emphasis,
                                                   unit=0,
                                                   oversub=True)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if ((mode in ('4x10G', '4x25G') and (sub != 0)) or
            ((mode == "2x50G") and (sub == 1 or sub == 3))):
            # For 4x modes, only update LEDs for the first sub-interface,
            # and for 2x mode, only update LEDs for the first and third
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        if mode in ('4x10G', '4x25G'):
            call = (fn_name + "_4x", self._set_led_4x, led_port)
        elif mode in ('2x50G'):
            call = (fn_name + "_2x", self._set_led_2x, led_port)
        elif mode in ('40G', '50G', '100G'):
            call = (fn_name, self._set_led, led_port)

        set_calls[led_index_list[0]] = call
        for x in range(1,8):
            set_calls[led_index_list[0]+x] = None

        return set_calls

class DellS5048FLED1(cumulus.ledup.TomahawkLEDController):
    led_functions = (DellS5048FSFP28._set_led_common)
    num_leds = 72

class DellS5048FLED0(cumulus.ledup.TomahawkLEDController):
    led_functions = (DellS5048FSFP28._set_led_common)
    num_leds = 72

# Dell S5048F switch ports

class DellS5048FSwitch(cumulus.platform.BCMSwitch):
    ledup = (DellS5048FLED1, DellS5048FLED0)
    _sfp28 = DellS5048FSFP28
    _qsfp28 = DellS5048FQSFP28

    has_phy_ucode_spi_rom = False
    has_phy_ucode_mdio = False

    ports = (
        # TSCF10
        _sfp28(label="1",  serdes=( 40,), leds=(1,  0), tx_lane_map=0x2031, rx_lane_map=0x0213, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="2",  serdes=( 41,), leds=(1,  2), tx_lane_map=0x2031, rx_lane_map=0x0213, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="3",  serdes=( 42,), leds=(1,  4), tx_lane_map=0x2031, rx_lane_map=0x0213, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="4",  serdes=( 43,), leds=(1,  6), tx_lane_map=0x2031, rx_lane_map=0x0213, tx_polarity_flip=True,  rx_polarity_flip=False),

        # TSCF16
        _sfp28(label="5",  serdes=( 64,), leds=(1,  8), tx_lane_map=0x3210, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True),
        _sfp28(label="6",  serdes=( 65,), leds=(1, 10), tx_lane_map=0x3210, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True),
        _sfp28(label="7",  serdes=( 66,), leds=(1, 12), tx_lane_map=0x3210, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True),
        _sfp28(label="8",  serdes=( 67,), leds=(1, 14), tx_lane_map=0x3210, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True),

        # TSCF12
        _sfp28(label="9",  serdes=( 48,), leds=(1, 16), tx_lane_map=0x0132, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="10", serdes=( 49,), leds=(1, 18), tx_lane_map=0x0132, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True),
        _sfp28(label="11", serdes=( 50,), leds=(1, 20), tx_lane_map=0x0132, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="12", serdes=( 51,), leds=(1, 22), tx_lane_map=0x0132, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),

        # TSCF17
        _sfp28(label="13", serdes=( 68,), leds=(1, 24), tx_lane_map=0x0123, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="14", serdes=( 69,), leds=(1, 26), tx_lane_map=0x0123, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="15", serdes=( 70,), leds=(1, 28), tx_lane_map=0x0123, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="16", serdes=( 71,), leds=(1, 30), tx_lane_map=0x0123, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),

        # TSCF13
        _sfp28(label="17", serdes=( 52,), leds=(1, 32), tx_lane_map=0x0123, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="18", serdes=( 53,), leds=(1, 34), tx_lane_map=0x0123, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="19", serdes=( 54,), leds=(1, 36), tx_lane_map=0x0123, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="20", serdes=( 55,), leds=(1, 38), tx_lane_map=0x0123, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),

        # TSCF20
        _sfp28(label="21", serdes=( 80,), leds=(1, 40), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="22", serdes=( 81,), leds=(1, 42), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="23", serdes=( 82,), leds=(1, 44), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="24", serdes=( 83,), leds=(1, 46), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False),

        # TSCF24
        _sfp28(label="25", serdes=( 96,), leds=(0,  0), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="26", serdes=( 97,), leds=(0,  2), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="27", serdes=( 98,), leds=(0,  4), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="28", serdes=( 99,), leds=(0,  6), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False),

        # TSCF0
        _sfp28(label="29", serdes=(  0,), leds=(0,  8), tx_lane_map=0x3210, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True),
        _sfp28(label="30", serdes=(  1,), leds=(0, 10), tx_lane_map=0x3210, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True),
        _sfp28(label="31", serdes=(  2,), leds=(0, 12), tx_lane_map=0x3210, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True),
        _sfp28(label="32", serdes=(  3,), leds=(0, 14), tx_lane_map=0x3210, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True),

        # TSCF25
        _sfp28(label="33", serdes=(100,), leds=(0, 16), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True),
        _sfp28(label="34", serdes=(101,), leds=(0, 18), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True),
        _sfp28(label="35", serdes=(102,), leds=(0, 20), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True),
        _sfp28(label="36", serdes=(103,), leds=(0, 22), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True),

        # TSCF1
        _sfp28(label="37", serdes=(  4,), leds=(0, 24), tx_lane_map=0x3210, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="38", serdes=(  5,), leds=(0, 26), tx_lane_map=0x3210, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="39", serdes=(  6,), leds=(0, 28), tx_lane_map=0x3210, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="40", serdes=(  7,), leds=(0, 30), tx_lane_map=0x3210, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=False),

        # TSCF26
        _sfp28(label="41", serdes=(104,), leds=(0, 32), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="42", serdes=(105,), leds=(0, 34), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="43", serdes=(106,), leds=(0, 36), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="44", serdes=(107,), leds=(0, 38), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False),

        # TSCF2
        _sfp28(label="45", serdes=(  8,), leds=(0, 40), tx_lane_map=0x3210, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True),
        _sfp28(label="46", serdes=(  9,), leds=(0, 42), tx_lane_map=0x3210, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="47", serdes=( 10,), leds=(0, 44), tx_lane_map=0x3210, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True),
        _sfp28(label="48", serdes=( 11,), leds=(0, 46), tx_lane_map=0x3210, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True),

        # TSCF21
        _qsfp28(label="49", serdes=( 84,  85,  86,  87), leds=(1, 48), tx_lane_map=0x1302, rx_lane_map=0x1032, tx_polarity_flip=0x1, rx_polarity_flip=0xd),

        # TSCF5
        _qsfp28(label="50", serdes=( 20,  21,  22,  23), leds=(0, 48), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=0x1, rx_polarity_flip=0x8),

        # TSCF8
        _qsfp28(label="51", serdes=( 32,  33,  34,  35), leds=(1, 56), tx_lane_map=0x1302, rx_lane_map=0x2031, tx_polarity_flip=0xe, rx_polarity_flip=0xd),

        # TSCF27
        _qsfp28(label="52", serdes=(108, 109, 110, 111), leds=(0, 56), tx_lane_map=0x1230, rx_lane_map=0x3210, tx_polarity_flip=0xe, rx_polarity_flip=0x7),

        # TSCF9
        _qsfp28(label="53", serdes=( 36,  37,  38,  39), leds=(1, 64), tx_lane_map=0x2031, rx_lane_map=0x1302, tx_polarity_flip=0xb, rx_polarity_flip=0xd),

        # TSCF29
        _qsfp28(label="54", serdes=(116, 117, 118, 119), leds=(0, 64), tx_lane_map=0x0213, rx_lane_map=0x0213, tx_polarity_flip=0xe, rx_polarity_flip=0x2),
)

# sysfs paths

class DellS5048FCPLD(cumulus.cpld.SysFS):
    path = '/sys/bus/platform/devices/dellemc_s5048f_smf.0/hwmon'
    if os.path.exists(path):
        files = [ f for f in os.listdir(path) if f.startswith('hwmon') ]
        if len(files):
            path = os.path.join(path, files[0])
                    
# temp sensors

class DellS5048FSMFTemp(cumulus.sensors.Temp_Unit):
    driver_path = '/sys/devices/platform/dellemc_s5048f_smf.0'
    temp_fan_min = None
    temp_fan_max = None

# U2900 - CPU onboard sensor
class DellS5048FSMFTemp1(DellS5048FSMFTemp):
    name = 'Temp1'
    description = 'CPU onboard sensor'
    driver_hwmon = ['temp1']
    temp_max = 95
    temp_crit = 120

# U44 - Switch onboard sensor 2
class DellS5048FSMFTemp2(DellS5048FSMFTemp):
    name = 'Temp2'
    description = 'Switch onboard sensor 2'
    driver_hwmon = ['temp2']

# U4 - System inlet 1
class DellS5048FSMFTemp3(DellS5048FSMFTemp):
    name = 'Temp3'
    description = 'System inlet 1'
    driver_hwmon = ['temp3']
    temp_max = 50

# U2 - System inlet 2
class DellS5048FSMFTemp4(DellS5048FSMFTemp):
    name = 'Temp4'
    description = 'System inlet 2'
    driver_hwmon = ['temp4']

# U38 - Switch onboard sensor 1
class DellS5048FSMFTemp5(DellS5048FSMFTemp):
    name = 'Temp5'
    description = 'Switch onboard sensor 1'
    driver_hwmon = ['temp6']

# U2900 - System inlet for back to front airflow
class DellS5048FSMFTemp6(DellS5048FSMFTemp):
    name = 'Temp6'
    description = 'System inlet for B2F airflow'
    driver_hwmon = ['temp9']

# PSU temp sensors

class DellS5048FSMFPSU1Temp1(DellS5048FSMFTemp):
    name = 'PSU1Temp1'
    driver_hwmon = ['temp11']
    psu = 'PSU1'

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state != 'OK'):
            return -1
        return cumulus.sensors.Temp_Unit.find_and_add_attrs(self)

class DellS5048FSMFPSU2Temp1(DellS5048FSMFTemp):
    name = 'PSU2Temp1'
    driver_hwmon = ['temp12']
    psu = 'PSU2'

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state != 'OK'):
            return -1
        return cumulus.sensors.Temp_Unit.find_and_add_attrs(self)

# Fans

class DellS5048FSMFFan(cumulus.sensors.Fan_Unit):
    rpmmode = 1
    fan_max = 24000
    fan_min = 5000
    driver_path = '/sys/devices/platform/dellemc_s5048f_smf.0'

class DellS5048FSMFFan1(DellS5048FSMFFan):
    name = 'Fan1'
    description = 'Fan Tray 1, Fan 1'
    cpld_hwmon = ['fan1']
    driver_hwmon = ['fan1']

class DellS5048FSMFFan2(DellS5048FSMFFan):
    name = 'Fan2'
    description = 'Fan Tray 1, Fan 2'
    cpld_hwmon = ['fan2']
    driver_hwmon = ['fan2']

class DellS5048FSMFFan3(DellS5048FSMFFan):
    name = 'Fan3'
    description = 'Fan Tray 2, Fan 1'
    cpld_hwmon = ['fan3']
    driver_hwmon = ['fan3']

class DellS5048FSMFFan4(DellS5048FSMFFan):
    name = 'Fan4'
    description = 'Fan Tray 2, Fan 2'
    cpld_hwmon = ['fan4']
    driver_hwmon = ['fan4']

class DellS5048FSMFFan5(DellS5048FSMFFan):
    name = 'Fan5'
    description = 'Fan Tray 3, Fan 1'
    cpld_hwmon = ['fan5']
    driver_hwmon = ['fan5']

class DellS5048FSMFFan6(DellS5048FSMFFan):
    name = 'Fan6'
    description = 'Fan Tray 3, Fan 2'
    cpld_hwmon = ['fan6']
    driver_hwmon = ['fan6']

class DellS5048FSMFFan7(DellS5048FSMFFan):
    name = 'Fan7'
    description = 'Fan Tray 4, Fan 1'
    cpld_hwmon = ['fan7']
    driver_hwmon = ['fan7']

class DellS5048FSMFFan8(DellS5048FSMFFan):
    name = 'Fan8'
    description = 'Fan Tray 4, Fan 2'
    cpld_hwmon = ['fan8']
    driver_hwmon = ['fan8']

class DellS5048FSMFPSU1Fan1(DellS5048FSMFFan):
    name = 'PSU1Fan1'
    description = 'PSU 1 Fan 1'
    cpld_hwmon = ['fan9']
    driver_hwmon = ['fan9']
    psu = 'PSU1'

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state != 'OK'):
            return -1
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)
    
class DellS5048FSMFPSU2Fan1(DellS5048FSMFFan):
    name = 'PSU2Fan1'
    description = 'PSU 2 Fan 1'
    cpld_hwmon = ['fan10']
    driver_hwmon = ['fan10']
    psu = 'PSU2'

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state != 'OK'):
            return -1
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

# PSUs

class DellS5048FSMFPSU1(cumulus.sensors.PSU_Unit):
    name = 'PSU1'
    driver_path = '/sys/devices/platform/dellemc_s5048f_smf.0/hwmon'
    if os.path.exists(driver_path):
        files = [ f for f in os.listdir(driver_path) if f.startswith('hwmon') ]
        if len(files):
            driver_path = os.path.join(driver_path, files[0])
    cpld_path = driver_path

    cpld_hwmon = ['psu_pwr1']
    driver_hwmon = ['psu1']

class DellS5048FSMFPSU2(cumulus.sensors.PSU_Unit):
    name = 'PSU2'
    driver_path = '/sys/devices/platform/dellemc_s5048f_smf.0/hwmon'
    if os.path.exists(driver_path):
        files = [ f for f in os.listdir(driver_path) if f.startswith('hwmon') ]
        if len(files):
            driver_path = os.path.join(driver_path, files[0])
    cpld_path = driver_path

    cpld_hwmon = ['psu_pwr2']
    driver_hwmon = ['psu2']

class DellS5048FSensors():
    units = (DellS5048FSMFTemp1(),
             DellS5048FSMFTemp2(),
             DellS5048FSMFTemp3(),
             DellS5048FSMFTemp4(),
             DellS5048FSMFTemp5(),
             DellS5048FSMFTemp6(),
             DellS5048FSMFPSU1(),
             DellS5048FSMFPSU2(),
             DellS5048FSMFPSU1Temp1(),
             DellS5048FSMFPSU2Temp1(),
             DellS5048FSMFFan1(),
             DellS5048FSMFFan2(),
             DellS5048FSMFFan3(),
             DellS5048FSMFFan4(),
             DellS5048FSMFFan5(),
             DellS5048FSMFFan6(),
             DellS5048FSMFFan7(),
             DellS5048FSMFFan8(),
             DellS5048FSMFPSU1Fan1(),
             DellS5048FSMFPSU2Fan1(),
             )

# Dell S5048F class

class DellS5048F(cumulus.platform.Platform):
    name = 'dellemc,s5048f_c2538'
    chip = cumulus.platform.TomahawkPlus_56967_Chip()
    switch = DellS5048FSwitch(chip)
    cpld = DellS5048FCPLD()
    pwms = None
    sensors = DellS5048FSensors()
    statusleds = None

Platform = DellS5048F
