
# Copyright 2017.  Cumulus Networks, Inc.
#
import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
import json
import syslog

class DeltaAG7648SFPp(cumulus.platform.TridentTwo10GPort):
    _set_led = '''
;;; DeltaAG7648SFPp
;;
set_led:
;;; Each 10G port has two LEDs. One for link and one for activity.
;;; The link LED is dual-color green/yellow
;;; The activity LED is just green
;;; Three bits to pack all active high:
;;;   0: link_green
;;;   1: link_yellow
;;;   2: activity_green
;;; We use:
;;;   link_green on with link
;;;   link_yellow always off
;;;   activity_green flashes on with activity
;;;
        port a

        pushst  PORTSTATUS_LINKEN   ; link_green
        pack                        ;  - on with link

        pushst  PORTSTATUS_ZERO     ; link_yellow
        pack                        ;  - off

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand                        ; activity_green
        pack                        ;  - flash on with activity

        ret
'''
    def __init__(self, label, serdes, leds, rx_polarity_flip=None, tx_polarity_flip=None,
                 tx_lane_map=None, rx_lane_map=None):
        cumulus.platform.TridentTwo10GPort.__init__(self,
                                                    label=label,
                                                    serdes=serdes,
                                                    phy_types=None,
                                                    phy_addrs=None,
                                                    phy_layer='SFP+',
                                                    leds=leds,
                                                    rx_lane_map=rx_lane_map,
                                                    tx_lane_map=tx_lane_map,
                                                    rx_polarity_flip=rx_polarity_flip,
                                                    tx_polarity_flip=tx_polarity_flip,
                                                    unit=0)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        call = (fn_name, self._set_led, led_port)

        for led in led_index_list:
            set_calls[led] = call
            set_calls[led+1] = None
            set_calls[led+2] = None

        return set_calls


class DeltaAG7648QSFPp(cumulus.platform.TridentTwo40GPort):

    _set_led = '''
;;; DeltaAG7648QSFPp
;;;
;;; 40G ports have 4 green/yellow LEDs, two bits per LED.
;;; For 40G mode, we use only the first LED to indicate link/activity.
;;;    0: link_green   (active high)
;;;    1-7: off
;;;  We use the link_green led for link and activity
;;; The other 3 LEDs will be off.
;;;
set_led:
        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor                     ; on with link
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        ld      a, 7
qsfp_pack_loop:
        port 1
        pushst  PORTSTATUS_ZERO
        pack
        sub     a, 1
        jnz     qsfp_pack_loop

        ret
'''
    _set_led_4x10 = '''
;;; DeltaAG7648QSFPp_4x10
;;;
;;;  For 4x10G mode, we use an LED for each port. Each port has
;;;  a single green/yellow LED:
;;;    0: link_green   (active high)
;;;    1: link_yellow  (active high)
;;;  We use the link_green led for link and activity
;;;  link_yellow is always off
;;
set_led:
        call    set_qsfp_4x10_led
        inc     a
        call    set_qsfp_4x10_led
        inc     a
        call    set_qsfp_4x10_led
        inc     a
        call    set_qsfp_4x10_led

        ret

set_qsfp_4x10_led:
        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor                     ; link_green on with link
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        pushst  PORTSTATUS_ZERO ; link_yellow off
        pack

        ret
'''
    def __init__(self, label, serdes, leds, rx_lane_map=None, tx_lane_map=None, rx_polarity_flip=None, tx_polarity_flip=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp+')
        cumulus.platform.TridentTwo40GPort.__init__(self,
                                                    label=label,
                                                    serdes=serdes,
                                                    phy_types=None,
                                                    phy_addrs=None,
                                                    phy_layer=sfp.type.upper(),
                                                    leds=leds,
                                                    rx_lane_map=rx_lane_map,
                                                    rx_polarity_flip=rx_polarity_flip,
                                                    tx_lane_map=tx_lane_map,
                                                    tx_polarity_flip=tx_polarity_flip,
                                                    unit=0)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if (mode == "4x10G") and (sub != 0):
            # For 4x10G mode only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        if mode == "4x10G":
            call = (fn_name + "_4x10", self._set_led_4x10, led_port)
        elif mode == "40G":
            call = (fn_name, self._set_led, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0]+1] = None
        set_calls[led_index_list[0]+2] = None
        set_calls[led_index_list[0]+3] = None
        set_calls[led_index_list[0]+4] = None
        set_calls[led_index_list[0]+5] = None
        set_calls[led_index_list[0]+6] = None
        set_calls[led_index_list[0]+7] = None


        return set_calls


class DeltaAG7648LED_1_36(cumulus.ledup.TridentTwoLEDController):
    num_leds = 112


class DeltaAG7648LED_37_54(cumulus.ledup.TridentTwoLEDController):
    num_leds = 84

class DeltaAG7648Switch(cumulus.platform.BCMSwitch):
    ledup = (DeltaAG7648LED_1_36, DeltaAG7648LED_37_54)
    _sfp = DeltaAG7648SFPp
    _qsfp = DeltaAG7648QSFPp

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (
        _sfp(label="1", serdes=(44,), leds=(0, 0),    rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="2", serdes=(45,), leds=(0, 3),    rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="3", serdes=(46,), leds=(0, 6),    rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="4", serdes=(47,), leds=(0, 9),    rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="5", serdes=(48,), leds=(0, 12),   rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="6", serdes=(49,), leds=(0, 15),   rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="7", serdes=(50,), leds=(0, 18),   rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="8", serdes=(51,), leds=(0, 21),   rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="9", serdes=(12,), leds=(0, 24),   rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="10", serdes=(13,), leds=(0, 27),  rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="11", serdes=(14,), leds=(0, 30),  rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="12", serdes=(15,), leds=(0, 33),  rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="13", serdes=(20,), leds=(0, 36),  rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="14", serdes=(21,), leds=(0, 39),  rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="15", serdes=(22,), leds=(0, 42),  rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="16", serdes=(23,), leds=(0, 45),  rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="17", serdes=(24,), leds=(0, 48),  rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="18", serdes=(25,), leds=(0, 51),  rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="19", serdes=(26,), leds=(0, 54),  rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="20", serdes=(27,), leds=(0, 57),  rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="21", serdes=(28,), leds=(0, 60),  rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="22", serdes=(29,), leds=(0, 63),  rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="23", serdes=(30,), leds=(0, 66),  rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="24", serdes=(31,), leds=(0, 69),  rx_lane_map=0x3210, tx_lane_map=0x0123, rx_polarity_flip=True),
        _sfp(label="25", serdes=(52,), leds=(0, 72),  rx_lane_map=0x3210, tx_lane_map=0x1032),
        _sfp(label="26", serdes=(53,), leds=(0, 75),  rx_lane_map=0x3210, tx_lane_map=0x1032),
        _sfp(label="27", serdes=(54,), leds=(0, 78),  rx_lane_map=0x3210, tx_lane_map=0x1032),
        _sfp(label="28", serdes=(55,), leds=(0, 81),  rx_lane_map=0x3210, tx_lane_map=0x1032),
        _sfp(label="29", serdes=(56,), leds=(0, 84),  rx_lane_map=0x3210, tx_lane_map=0x0123),
        _sfp(label="30", serdes=(57,), leds=(0, 87),  rx_lane_map=0x3210, tx_lane_map=0x0123),
        _sfp(label="31", serdes=(58,), leds=(0, 90),  rx_lane_map=0x3210, tx_lane_map=0x0123),
        _sfp(label="32", serdes=(59,), leds=(0, 93),  rx_lane_map=0x3210, tx_lane_map=0x0123),
        _sfp(label="33", serdes=(60,), leds=(0, 96),  rx_lane_map=0x2301, tx_lane_map=0x1032),
        _sfp(label="34", serdes=(61,), leds=(0, 99),  rx_lane_map=0x2301, tx_lane_map=0x1032),
        _sfp(label="35", serdes=(62,), leds=(0, 102), rx_lane_map=0x2301, tx_lane_map=0x1032),
        _sfp(label="36", serdes=(63,), leds=(0, 105), rx_lane_map=0x2301, tx_lane_map=0x1032),
        _sfp(label="37", serdes=(64,), leds=(1, 0),   rx_lane_map=0x2301, tx_lane_map=0x3021),
        _sfp(label="38", serdes=(65,), leds=(1, 3),   rx_lane_map=0x2301, tx_lane_map=0x3021),
        _sfp(label="39", serdes=(66,), leds=(1, 6),   rx_lane_map=0x2301, tx_lane_map=0x3021),
        _sfp(label="40", serdes=(67,), leds=(1, 9),   rx_lane_map=0x2301, tx_lane_map=0x3021),
        _sfp(label="41", serdes=(68,), leds=(1, 12),  rx_lane_map=0x2301, tx_lane_map=0x3021),
        _sfp(label="42", serdes=(69,), leds=(1, 15),  rx_lane_map=0x2301, tx_lane_map=0x3021),
        _sfp(label="43", serdes=(70,), leds=(1, 18),  rx_lane_map=0x2301, tx_lane_map=0x3021),
        _sfp(label="44", serdes=(71,), leds=(1, 21),  rx_lane_map=0x2301, tx_lane_map=0x3021),
        _sfp(label="45", serdes=(72,), leds=(1, 24),  rx_lane_map=0x2301, tx_lane_map=0x1032),
        _sfp(label="46", serdes=(73,), leds=(1, 27),  rx_lane_map=0x2301, tx_lane_map=0x1032),
        _sfp(label="47", serdes=(74,), leds=(1, 30),  rx_lane_map=0x2301, tx_lane_map=0x1032),
        _sfp(label="48", serdes=(75,), leds=(1, 33),  rx_lane_map=0x2301, tx_lane_map=0x1032),

        _qsfp(label="49", serdes=(100, 101, 102, 103), leds=(1, 36), tx_lane_map=0x2031, rx_lane_map=0x0132,
              rx_polarity_flip=0x5, tx_polarity_flip=0x2),
        _qsfp(label="50", serdes=(96, 97, 98, 99), leds=(1, 44), tx_lane_map=0x3021, rx_lane_map=0x1320,
              rx_polarity_flip=0xd, tx_polarity_flip=0xb),
        _qsfp(label="51", serdes=(108, 109, 110, 111), leds=(1, 52), tx_lane_map=0x0213, rx_lane_map=0x0132,
              rx_polarity_flip=0x5, tx_polarity_flip=0x8),
        _qsfp(label="52", serdes=(104, 105, 106, 107), leds=(1, 60), tx_lane_map=0x2031, rx_lane_map=0x0132,
              rx_polarity_flip=0x3, tx_polarity_flip=0x3),
        _qsfp(label="53", serdes=(80, 81, 82, 83), leds=(1, 68), tx_lane_map=0x0132, rx_lane_map=0x0132,
              rx_polarity_flip=0xb, tx_polarity_flip=0x0),
        _qsfp(label="54", serdes=(76, 77, 78, 79), leds=(1, 76), tx_lane_map=0x2031, rx_lane_map=0x1320,
              rx_polarity_flip=0xd, tx_polarity_flip=0x3),
    )



class DeltaAG7648CPLD(cumulus.cpld.SysFS):
    path = '/sys/bus/platform/devices/delta_ag7648_cpld.0'

DeltaAG7648_i2c_path = '/sys/bus/i2c/devices/'

''' tmp75: temp sensor 1 '''
DeltaAG7648_i2c_10_4d_path = DeltaAG7648_i2c_path + '10-004d'

''' tmp75: temp sensor 2 '''
DeltaAG7648_i2c_11_4c_path = DeltaAG7648_i2c_path + '11-004c'

''' tmp75: temp sensor 3 '''
DeltaAG7648_i2c_11_4d_path = DeltaAG7648_i2c_path + '11-004d'

''' tmp75: temp sensor 4 '''
DeltaAG7648_i2c_11_4e_path = DeltaAG7648_i2c_path + '11-004e'

''' dps460: PSU1 Temp/Fan '''
DeltaAG7648_i2c_14_59_path = DeltaAG7648_i2c_path + '14-0059'

''' dps460: PSU2 Temp/Fan '''
DeltaAG7648_i2c_14_58_path = DeltaAG7648_i2c_path + '14-0058'

''' max6620: Fan Controller 1 '''
DeltaAG7648_i2c_11_29_path = DeltaAG7648_i2c_path + '11-0029'

''' max6620: Fan Controller 2 '''
DeltaAG7648_i2c_11_2a_path = DeltaAG7648_i2c_path + '11-002a'

''' ltc4215: PSU1 '''
DeltaAG7648_i2c_11_42_path = DeltaAG7648_i2c_path + '11-0042'

''' ltc4215: PSU2 '''
DeltaAG7648_i2c_11_40_path = DeltaAG7648_i2c_path + '11-0040'


class DeltaAG7648Temp1(cumulus.sensors.Temp_Unit):
    # U54, Hardware shutdown, Fan Controlled
    name = 'Temp1'
    description = cumulus.sensors.Temp_Unit.mac_board_sensor_name
    driver_path = DeltaAG7648_i2c_11_4c_path
    driver_hwmon = ['temp1']
    temp_otp_hw = True


class DeltaAG7648Temp1F2B(DeltaAG7648Temp1):
    temp_max = 66
    temp_crit = 77
    temp_fan_min = 40
    temp_fan_max = 65


class DeltaAG7648Temp1B2F(DeltaAG7648Temp1):
    temp_max = 63
    temp_crit = 70
    temp_fan_min = 33
    temp_fan_max = 57


class DeltaAG7648Temp2(cumulus.sensors.Temp_Unit):
    # U245, Hardware shutdown, Fan Controlled
    name = 'Temp2'
    description = 'Board Sensor near QSFP+ cage'
    driver_path = DeltaAG7648_i2c_11_4e_path
    driver_hwmon = ['temp1']
    temp_otp_hw = True


class DeltaAG7648Temp2F2B(DeltaAG7648Temp2):
    temp_max = 52
    temp_crit = 60
    temp_fan_min = 23
    temp_fan_max = 48


class DeltaAG7648Temp2B2F(DeltaAG7648Temp2):
    temp_max = 51
    temp_crit = 60
    temp_fan_min = 24
    temp_fan_max = 49


class DeltaAG7648Temp3(cumulus.sensors.Temp_Unit):
    # U244, Hardware shutdown, Fan Controlled
    name = 'Temp3'
    description = 'Board Sensor near SFP+ cage'
    driver_path = DeltaAG7648_i2c_11_4d_path
    driver_hwmon = ['temp1']
    temp_otp_hw = True


class DeltaAG7648Temp3F2B(DeltaAG7648Temp3):
    temp_max = 60
    temp_crit = 71
    temp_fan_min = 34
    temp_fan_max = 58


class DeltaAG7648Temp3B2F(DeltaAG7648Temp3):
    temp_max = 62
    temp_crit = 69
    temp_fan_min = 32
    temp_fan_max = 56


class DeltaAG7648Temp4(cumulus.sensors.Temp_Unit):
    # U57, Software shutdown
    name = 'Temp4'
    description = 'Board Sensor near CPU'
    driver_path = DeltaAG7648_i2c_10_4d_path


class DeltaAG7648Temp4F2B(DeltaAG7648Temp4):
    temp_max = 54
    temp_crit = 64
    temp_fan_min = 35
    temp_fan_max = 50


class DeltaAG7648Temp4B2F(DeltaAG7648Temp4):
    temp_max = 49
    temp_crit = 61
    temp_fan_min = 35
    temp_fan_max = 45


class DeltaAG7648CT(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    temp_fan_min = 75
    temp_max = 93
    temp_crit = 98


class DeltaAG7648Temp5(DeltaAG7648CT):
    name = 'Temp5'
    driver_hwmon = ['temp2']


class DeltaAG7648Temp6(DeltaAG7648CT):
    name = 'Temp6'
    driver_hwmon = ['temp3']


class DeltaAG7648PSUTemp(cumulus.sensors.Temp_Unit):
    temp_fan_min = 30
    temp_fan_max = 52
    temp_max = 55
    temp_crit = 61
    psu = None

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if state == 'ABSENT':
            return -1
        return cumulus.sensors.Temp_Unit.find_and_add_attrs(self)


class DeltaAG7648Temp7(DeltaAG7648PSUTemp):
    name = 'PSU1Temp1'
    description = 'PSU1 Temp Sensor'
    driver_path = DeltaAG7648_i2c_14_59_path
    driver_hwmon = ['temp1']
    psu = 'PSU1'


class DeltaAG7648Temp8(DeltaAG7648PSUTemp):
    name = 'PSU2Temp1'
    description = 'PSU2 Temp Sensor'
    driver_path = DeltaAG7648_i2c_14_58_path
    driver_hwmon = ['temp1']
    psu = 'PSU2'


class DeltaAG7648T2Temp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path = '/cumulus/switchd/run'
    temp_fan_min = 56
    temp_fan_max = 80
    temp_max = 97
    temp_crit = 104


class DeltaAG7648T2Temp1(DeltaAG7648T2Temp):
    name = 'Temp7'
    driver_hwmon = ['temp1']


class DeltaAG7648T2Temp2(DeltaAG7648T2Temp):
    name = 'Temp8'
    driver_hwmon = ['temp2']


class DeltaAG7648T2Temp3(DeltaAG7648T2Temp):
    name = 'Temp9'
    driver_hwmon = ['temp3']


class DeltaAG7648T2Temp4(DeltaAG7648T2Temp):
    name = 'Temp10'
    driver_hwmon = ['temp4']


class DeltaAG7648T2Temp5(DeltaAG7648T2Temp):
    name = 'Temp11'
    driver_hwmon = ['temp5']


class DeltaAG7648T2Temp6(DeltaAG7648T2Temp):
    name = 'Temp12'
    driver_hwmon = ['temp6']


class DeltaAG7648T2Temp7(DeltaAG7648T2Temp):
    name = 'Temp13'
    driver_hwmon = ['temp7']


class DeltaAG7648T2Temp8(DeltaAG7648T2Temp):
    name = 'Temp14'
    driver_hwmon = ['temp8']


class DeltaAG7648T2Temp9(DeltaAG7648T2Temp):
    name = 'Temp15'
    driver_hwmon = ['temp9']


class DeltaAG7648Fan(cumulus.sensors.Fan_Unit):
    fan_max = 19000
    fan_min = 7000
    rpmmode = 1
    hwmon_search = False


class DeltaAG7648PSUFan(cumulus.sensors.Fan_Unit):
    fan_max = 18000
    fan_min = 6800
    minpwm = 102
    psu = None

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if state == 'ABSENT':
            return -1
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)


class DeltaAG7648Fan1(DeltaAG7648Fan):
    name = 'Fan1'
    description = 'Fan Tray 1'
    driver_path = DeltaAG7648_i2c_11_2a_path
    driver_hwmon = ['fan1']
    cpld_hwmon = ['fan_1']
    pwm_hwmon = ['pwm1']


class DeltaAG7648Fan2(DeltaAG7648Fan):
    name = 'Fan2'
    description = 'Fan Tray 1'
    driver_path = DeltaAG7648_i2c_11_2a_path
    driver_hwmon = ['fan2']
    cpld_hwmon = ['fan_1']
    pwm_hwmon = ['pwm2']


class DeltaAG7648Fan3(DeltaAG7648Fan):
    name = 'Fan3'
    description = 'Fan Tray 2'
    driver_path = DeltaAG7648_i2c_11_29_path
    driver_hwmon = ['fan1']
    cpld_hwmon = ['fan_2']
    pwm_hwmon = ['pwm1']


class DeltaAG7648Fan4(DeltaAG7648Fan):
    name = 'Fan4'
    description = 'Fan Tray 2'
    driver_path = DeltaAG7648_i2c_11_29_path
    driver_hwmon = ['fan2']
    cpld_hwmon = ['fan_2']
    pwm_hwmon = ['pwm2']


class DeltaAG7648Fan5(DeltaAG7648Fan):
    name = 'Fan5'
    description = 'Fan Tray 3'
    driver_path = DeltaAG7648_i2c_11_29_path
    driver_hwmon = ['fan3']
    cpld_hwmon = ['fan_3']
    pwm_hwmon = ['pwm3']


class DeltaAG7648Fan6(DeltaAG7648Fan):
    name = 'Fan6'
    description = 'Fan Tray 3'
    driver_path = DeltaAG7648_i2c_11_29_path
    driver_hwmon = ['fan4']
    cpld_hwmon = ['fan_3']
    pwm_hwmon = ['pwm4']


class DeltaAG7648Fan7(DeltaAG7648PSUFan):
    name = 'PSU1Fan1'
    description = 'PSU1 Fan'
    driver_path = DeltaAG7648_i2c_14_59_path
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    psu = 'PSU1'


class DeltaAG7648Fan8(DeltaAG7648PSUFan):
    name = 'PSU2Fan1'
    description = 'PSU2 Fan'
    driver_path = DeltaAG7648_i2c_14_58_path
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    psu = 'PSU2'


class DeltaAG7648PSU(cumulus.sensors.PSU_Unit):
    init_done = False

    def __init__(self):
        cumulus.sensors.PSU_Unit.__init__(self)
        self.add_state('OK', self.ok_handler)

    def ok_handler(self):
        if self.init_done:
            return
        err_states = ['BAD', 'ABSENT'];
        if self.attrs['prev_state'] in err_states:
            subprocess.call(["/sbin/rmmod", "dps460"])
            subprocess.call(["/sbin/modprobe", "dps460"])
            self.init_done = True


class DeltaAG7648PSU1(DeltaAG7648PSU):
    name = 'PSU1'
    cpld_hwmon = ['psu_pwr1']


class DeltaAG7648PSU2(DeltaAG7648PSU):
    name = 'PSU2'
    cpld_hwmon = ['psu_pwr2']


def delta_ag7648_airflow():
    fan_type_count = {'F2B': 0, 'B2F': 0}
    # Default to F2B type
    fan_type = 1
    b_info = ''

    try:
        plat = subprocess.check_output(('platform-detect', '-m'))
    except OSError:
        return None
    if plat.strip() != 'delta,ag7648':
        return None
    for fan in ['fan1', 'fan2', 'fan3']:
        try:
            b_info = subprocess.check_output(['/usr/cumulus/bin/decode-syseeprom', '-j', '-t', fan])
            if b_info != '':
                json_dict = json.loads(b_info)
                tlv_valid = json_dict['validTlvInfoHeader']
                if tlv_valid:
                    temp = int(json_dict['tlv']['Service Tag']['value'])
                    if temp == 1:
                        fan_type_count['F2B'] += 1
                    if temp == 2:
                        fan_type_count['B2F'] += 1
                else:
                    continue 
            else:
                continue
        except Exception:
            b_info = ''

    if fan_type_count['F2B'] == 0 and fan_type_count['B2F'] == 0:
        syslog.syslog(syslog.LOG_WARNING, "Airflow types (Front-to-Back/Back-to-Front) for all the fans could "
                                          "not be detected, defaulting to F2B")
    elif fan_type_count['F2B'] in xrange(1, 4) and fan_type_count['B2F'] == 0:
        fan_type = 1
    elif fan_type_count['B2F'] in xrange(1, 4) and fan_type_count['F2B'] == 0:
        fan_type = 2
    else:
        syslog.syslog(syslog.LOG_WARNING, "Fans with different airflow types (Front-to-Back/Back-to-Front) detected ")

    if fan_type == 2:
        return 'B2F'
    else:
        return 'F2B'


class DeltaAG7648PWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'
    fans = (DeltaAG7648Fan1(),
            DeltaAG7648Fan2(),
            DeltaAG7648Fan3(),
            DeltaAG7648Fan4(),
            DeltaAG7648Fan5(),
            DeltaAG7648Fan6(),
            DeltaAG7648Fan7(),
            DeltaAG7648Fan8(),
            )

    temps = (DeltaAG7648Temp5(),
             DeltaAG7648Temp6(),
             DeltaAG7648Temp7(),
             DeltaAG7648Temp8(),
             DeltaAG7648T2Temp1(),
             DeltaAG7648T2Temp2(),
             DeltaAG7648T2Temp3(),
             DeltaAG7648T2Temp4(),
             DeltaAG7648T2Temp5(),
             DeltaAG7648T2Temp6(),
             DeltaAG7648T2Temp7(),
             DeltaAG7648T2Temp8(),
             DeltaAG7648T2Temp9(),
             )
    f2b_temps = (DeltaAG7648Temp1F2B(),
                 DeltaAG7648Temp2F2B(),
                 DeltaAG7648Temp3F2B(),
                 DeltaAG7648Temp4F2B(),
                 )
    b2f_temps = (DeltaAG7648Temp1B2F(),
                 DeltaAG7648Temp2B2F(),
                 DeltaAG7648Temp3B2F(),
                 DeltaAG7648Temp4B2F(),
                 )
    # Default to F2B if airflow cant be figured out.
    if delta_ag7648_airflow() == 'B2F':
        temps = temps + b2f_temps
    else:
        temps = temps + f2b_temps



class DeltaAG7648PWMs():
    units = (DeltaAG7648PWM1(),
             )


class DeltaAG7648Sensors():
    units = (DeltaAG7648PSU1(),
             DeltaAG7648PSU2(),
             DeltaAG7648Temp5(),
             DeltaAG7648Temp6(),
             DeltaAG7648Temp7(),
             DeltaAG7648Temp8(),
             DeltaAG7648Fan1(),
             DeltaAG7648Fan2(),
             DeltaAG7648Fan3(),
             DeltaAG7648Fan4(),
             DeltaAG7648Fan5(),
             DeltaAG7648Fan6(),
             DeltaAG7648Fan7(),
             DeltaAG7648Fan8(),
             DeltaAG7648T2Temp1(),
             DeltaAG7648T2Temp2(),
             DeltaAG7648T2Temp3(),
             DeltaAG7648T2Temp4(),
             DeltaAG7648T2Temp5(),
             DeltaAG7648T2Temp6(),
             DeltaAG7648T2Temp7(),
             DeltaAG7648T2Temp8(),
             DeltaAG7648T2Temp9(),
             )
    f2b_temps = (DeltaAG7648Temp1F2B(),
                 DeltaAG7648Temp2F2B(),
                 DeltaAG7648Temp3F2B(),
                 DeltaAG7648Temp4F2B(),
                 )
    b2f_temps = (DeltaAG7648Temp1B2F(),
                 DeltaAG7648Temp2B2F(),
                 DeltaAG7648Temp3B2F(),
                 DeltaAG7648Temp4B2F(),
                 )
    # Default to F2B if airflow cant be figured out.
    if delta_ag7648_airflow() == 'B2F':
        units = units + b2f_temps
    else:
        units = units + f2b_temps


class DeltaAG7648SystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'Diag'
    led_location = 'led_system'
    bad_led_color = 'yellow'


class DeltaAG7648PowerLED(cumulus.sensors.StatusLED_Unit):
    name = 'Power'
    sources = ['PSU1', 'PSU2']
    led_location = 'led_power'
    bad_led_color = 'yellow_blinking'


class DeltaAG7648FanLED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4', 'Fan5', 'Fan6']
    led_location = 'led_fan'
    bad_led_color = 'yellow_blinking'


class DeltaAG7648Fan1LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 1'
    sources = ['Fan1', 'Fan2']
    led_location = 'led_fan_tray_1'
    bad_led_color = 'yellow'


class DeltaAG7648Fan2LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 2'
    sources = ['Fan3', 'Fan4']
    led_location = 'led_fan_tray_2'
    bad_led_color = 'yellow'


class DeltaAG7648Fan3LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 3'
    sources = ['Fan5', 'Fan6']
    led_location = 'led_fan_tray_3'
    bad_led_color = 'yellow'


class DeltaAG7648LEDs():
    units = (DeltaAG7648SystemLED(),
             DeltaAG7648FanLED(),
             DeltaAG7648PowerLED(),
             DeltaAG7648Fan1LED(),
             DeltaAG7648Fan2LED(),
             DeltaAG7648Fan3LED(),
             )


class DeltaAG7648(cumulus.platform.Platform):
    name = 'delta,ag7648'
    chip = cumulus.platform.TridentTwo_56854_Chip()
    switch = DeltaAG7648Switch(chip)
    cpld = DeltaAG7648CPLD()
    statusleds = DeltaAG7648LEDs()
    pwms = DeltaAG7648PWMs()
    sensors = DeltaAG7648Sensors()

Platform = DeltaAG7648
