#
# Copyright 2016.  Cumulus Networks, Inc.
#

import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class QuantaLY2RQSFPp(cumulus.platform.Trident40GPort):
    _set_led = '''
;;; QuantaLY2RQSFPp
;;;
;;; Two bits per port:
;;;
;;; 1st bit pushed is always zero
;;;
;;; 2n bit pushed on stack controls green LED:
;;; 0 - off
;;; 1 - on
;;;
set_led:
        port a

        pushst PORTSTATUS_ZERO
        pack

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        ret

'''
    _set_led_4x10 = '''
;;;
;;; Input : register A, logical port
;;; Input / Output : register B, accumulated port status

;;; For each port accumulate 2 bits of information in register B:
;;;   register B[0] - link enabled
;;;   register B[1] - activity (TX _OR_ RX)

SUB_INTF_LINKEN     equ 0x0  ; link enabled
SUB_INTF_ACTIVITY   equ 0x1  ; activity

sub_intf_info:

        ;;; push OR of current TX/RX activity onto stack
        port a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor

        ;;; push accumlated link activity onto stack
        tst B, SUB_INTF_ACTIVITY
        push cy

        ;;; OR accumlated activity with current activity and store
        ;;; result back in register B
        tor
        pop
        bit B, SUB_INTF_ACTIVITY

        ;;; push current LINKEN onto stack
        pushst  PORTSTATUS_LINKEN

        ;;; push accumlated link enable onto stack
        tst B, SUB_INTF_LINKEN
        push cy

        ;;; OR accumlated LINKEN with current LINKEN and store
        ;;; result back in register B
        tor
        pop
        bit B, SUB_INTF_LINKEN

        ret

;;; QuantaLY6QSFPp_4x10
;;; OR together status of 4 ports for 4x10G QSFP port
;;; Two LED bits per port: yellow(active high), green(active low)
;;; Input: register A - first logical port of 4-port group
;;; Modified: register B - used as scratch space
set_led:

        port a
        pushst PORTSTATUS_ZERO
        pack

        ;;; initialize B
        ld  b,0

        ;;; load sub-0 info
        call    sub_intf_info

        ;;; load sub-1 info
        inc a
        call    sub_intf_info

        ;;; load sub-2 info
        inc a
        call    sub_intf_info

        ;;; load sub-3 info
        inc a
        call    sub_intf_info

        ;;; push accumlated link activity onto stack
        tst B, SUB_INTF_ACTIVITY
        push    cy

        ;;; combine with BLINK state
        push    (BLINK_STATE_P)
        tand
        tinv    ; flash off with activity

        ;;; push accumlated link enable onto stack
        tst B, SUB_INTF_LINKEN
        push    cy

        ;;; AND activity-blink state with accumlated LINKEN
        tand
        pack

        ret

'''
    def __init__(self, label, serdes, leds):
        cumulus.platform.Trident40GPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_types=None,
                                                 phy_addrs=None,
                                                 leds=leds,
                                                 unit=0)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if (mode == "4x10G") and (sub != 0):
            # For 4x10G mode only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, 0)
        if mode == "4x10G":
            call = (fn_name + "_4x10", self._set_led_4x10, led_port)
        elif mode == "40G":
            call = (fn_name, self._set_led, led_port)

        led_index = led_index_list[0]
        set_calls[led_index] = call
        set_calls[led_index + 1] = None
        set_calls[led_index + 2] = None
        set_calls[led_index + 3] = None
        set_calls[led_index + 4] = None
        set_calls[led_index + 5] = None
        set_calls[led_index + 6] = None
        set_calls[led_index + 7] = None

        return set_calls

class QuantaLY2RSFPp(cumulus.platform.Trident10GPort):
    _set_led = '''
;;; QuantaLY2RSFPp
;;;
;;; Two bits per port:
;;;
;;; 1st bit pushed should always be zero;
;;;
;;; 2nd bit pushed on stack controls green LED:
;;; 0 - off
;;; 1 - on
;;;
set_led:
        port 1
        pushst PORTSTATUS_ZERO
        pack

        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        ret
'''
    def __init__(self, label, serdes, leds):
        cumulus.platform.Trident10GPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_type=None,
                                                 phy_addr=None,
                                                 phy_layer='SFP+',
                                                 leds=leds,
                                                 unit=0)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        call = (fn_name, self._set_led, self.hw_port_num(mode, 0))
        for led in led_index_list:
            set_calls[led] = call
            set_calls[led + 1] = None

        return set_calls

class QuantaLY2RLED(cumulus.ledup.TridentLEDController):
    num_leds = 64

class QuantaLY2RSwitch(cumulus.platform.BCMSwitch):
    ledup = QuantaLY2RLED
    _sfp = QuantaLY2RSFPp
    _qsfp = QuantaLY2RQSFPp
    has_phy_ucode_spi_rom = False
    has_phy_ucode_mdio = False
    ports = (
        _sfp(label="1",  serdes=5,  leds=(0, 0)),
        _sfp(label="2",  serdes=7,  leds=(0, 2)),
        _sfp(label="3",  serdes=4,  leds=(0, 4)),
        _sfp(label="4",  serdes=6,  leds=(0, 6)),
        _sfp(label="5",  serdes=11, leds=(0, 8)),
        _sfp(label="6",  serdes=9,  leds=(0, 10)),
        _sfp(label="7",  serdes=8,  leds=(0, 12)),
        _sfp(label="8",  serdes=10, leds=(0, 14)),
        _sfp(label="9",  serdes=12, leds=(0, 16)),
        _sfp(label="10", serdes=13, leds=(0, 18)),
        _sfp(label="11", serdes=15, leds=(0, 20)),
        _sfp(label="12", serdes=14, leds=(0, 22)),
        _sfp(label="13", serdes=17, leds=(0, 24)),
        _sfp(label="14", serdes=19, leds=(0, 26)),
        _sfp(label="15", serdes=16, leds=(0, 28)),
        _sfp(label="16", serdes=18, leds=(0, 30)),
        _sfp(label="17", serdes=20, leds=(0, 32)),
        _sfp(label="18", serdes=21, leds=(0, 34)),
        _sfp(label="19", serdes=23, leds=(0, 36)),
        _sfp(label="20", serdes=22, leds=(0, 38)),
        _sfp(label="21", serdes=24, leds=(0, 40)),
        _sfp(label="22", serdes=25, leds=(0, 42)),
        _sfp(label="23", serdes=27, leds=(0, 44)),
        _sfp(label="24", serdes=26, leds=(0, 46)),
        _sfp(label="25", serdes=29, leds=(0, 48)),
        _sfp(label="26", serdes=31, leds=(0, 50)),
        _sfp(label="27", serdes=28, leds=(0, 52)),
        _sfp(label="28", serdes=30, leds=(0, 54)),
        _sfp(label="29", serdes=32, leds=(0, 56)),
        _sfp(label="30", serdes=33, leds=(0, 58)),
        _sfp(label="31", serdes=35, leds=(0, 60)),
        _sfp(label="32", serdes=34, leds=(0, 62)),
        _sfp(label="33", serdes=43, leds=(1, 0)),
        _sfp(label="34", serdes=41, leds=(1, 2)),
        _sfp(label="35", serdes=40, leds=(1, 4)),
        _sfp(label="36", serdes=42, leds=(1, 6)),
        _sfp(label="37", serdes=45, leds=(1, 8)),
        _sfp(label="38", serdes=47, leds=(1, 10)),
        _sfp(label="39", serdes=46, leds=(1, 12)),
        _sfp(label="40", serdes=44, leds=(1, 14)),
        _sfp(label="41", serdes=48, leds=(1, 16)),
        _sfp(label="42", serdes=49, leds=(1, 18)),
        _sfp(label="43", serdes=51, leds=(1, 20)),
        _sfp(label="44", serdes=50, leds=(1, 22)),
        _sfp(label="45", serdes=52, leds=(1, 24)),
        _sfp(label="46", serdes=53, leds=(1, 26)),
        _sfp(label="47", serdes=55, leds=(1, 28)),
        _sfp(label="48", serdes=54, leds=(1, 30)),


        _qsfp(label="49", serdes=(56,57,58,59), leds=(1,32)),
        _qsfp(label="50", serdes=(60,63,61,62), leds=(1,40)),
        _qsfp(label="51", serdes=(64,65,66,67), leds=(1,48)),
        _qsfp(label="52", serdes=(68,71,69,70), leds=(1,56)),
    )

class QuantaLY2R_PSU(cumulus.sensors.PSU_Unit):
    def find_and_add_attrs(self):
        if self.get_gpio_value(self.psu_present) == '0':
            self.attrs['psu_pwr1_present'] = 1
        else:
            self.attrs['psu_pwr1_present'] = 0
        if self.get_gpio_value(self.psu_dc_ok) == '1':
            self.attrs['psu_pwr1_dc_ok'] = 1
        else:
            self.attrs['psu_pwr1_dc_ok'] = 0
        if self.attrs['psu_pwr1_present'] and self.attrs['psu_pwr1_dc_ok']:
            self.attrs['psu_pwr1_all_ok'] = 1
        return 0

class QuantaLY2R_Fan(cumulus.sensors.Fan_Unit):
    rpmmode = 1
    fan_min = 9500
    fan_max = 21000
    driver_path = '/sys/devices/soc.0/ffe03000.i2c/i2c-0/0-002e'

    def find_and_add_attrs(self):
        if self.get_gpio_value(self.fan_present) == '0':
            self.attrs['fan_0'] = 'installed, ok'
        else:
            self.attrs['fan_0'] = 'not_installed'
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

class QuantaLY2RFan1(QuantaLY2R_Fan):
    fan_present = 'gpio212'
    name = 'Fan1'
    driver_hwmon = ['fan1']

class QuantaLY2RFan2(QuantaLY2R_Fan):
    fan_present = 'gpio213'
    name = 'Fan2'
    driver_hwmon = ['fan2']

class QuantaLY2RFan3(QuantaLY2R_Fan):
    fan_present = 'gpio214'
    name = 'Fan3'
    driver_hwmon = ['fan3']

class QuantaLY2RFan4(QuantaLY2R_Fan):
    fan_present = 'gpio212'
    name = 'Fan4'
    driver_hwmon = ['fan5']

class QuantaLY2RFan5(QuantaLY2R_Fan):
    fan_present = 'gpio213'
    name = 'Fan5'
    driver_hwmon = ['fan6']

class QuantaLY2RFan6(QuantaLY2R_Fan):
    fan_present = 'gpio214'
    name = 'Fan6'
    driver_hwmon = ['fan7']

class QuantaLY2RPSU1(QuantaLY2R_PSU):
    name = 'PSU1'
    psu_present = 'gpio152'
    psu_dc_ok = 'gpio153'

class QuantaLY2RPSU2(QuantaLY2R_PSU):
    name = 'PSU2'
    psu_present = 'gpio136'
    psu_dc_ok = 'gpio137'

class QuantaLY2RSystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_location = 'gpio235'
    good_led_color = '1'  # Green
    bad_led_color = '0'   # Off

    def led_update(self, color):
        self.set_gpio_value(self.led_location, color)
        #print 'loc: %s color %s' %(self.led_location, color)

    def led_read(self):
        color = self.get_gpio_value(self.led_location)
        if color == '0':
            color = 'off'
        else:
            color = 'green'
        return self.name, color

class QuantaLY2RTemp(cumulus.sensors.Temp_Unit):
    temp_max = 68
    temp_crit = 80

class QuantaLY2RTemp1(QuantaLY2RTemp):
    name = 'Temp1'
    description = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path = '/sys/devices/pci0000:00/0000:00:00.0/0000:01:00.0'
    driver_hwmon = ['temp1']
    temp_max = 100
    temp_crit = 105

class QuantaLY2RTemp2(QuantaLY2RTemp):
    name = 'Temp2'
    description = "Left Corner near PHY"
    driver_path = '/sys/devices/soc.0/ffe03000.i2c/i2c-0/0-002e'
    driver_hwmon = ['temp1']

class QuantaLY2RTemp3(QuantaLY2RTemp):
    name = 'Temp3'
    description = "Left Side of Networking ASIC"
    driver_path = '/sys/devices/soc.0/ffe03000.i2c/i2c-0/0-002e'
    driver_hwmon = ['temp2']

class QuantaLY2RTemp4(QuantaLY2RTemp):
    name = 'Temp4'
    description = "Right Side of Networking ASIC"
    driver_path = '/sys/devices/soc.0/ffe03000.i2c/i2c-0/0-002e'
    driver_hwmon = ['temp3']

class QuantaLY2RTemp5(QuantaLY2RTemp):
    name = 'Temp5'
    description = "Fan Outlet Sensor located near CPU"
    driver_path = '/sys/devices/soc.0/ffe03000.i2c/i2c-0/0-002e'
    driver_hwmon = ['temp4']

class QuantaLY2RPWM1(cumulus.sensors.PWM_Unit):
    fans = (QuantaLY2RFan1(),
            QuantaLY2RFan2(),
            QuantaLY2RFan3(),
            QuantaLY2RFan4(),
            QuantaLY2RFan5(),
            QuantaLY2RFan6(),
            )
    temps = (QuantaLY2RTemp1(),
             QuantaLY2RTemp2(),
             QuantaLY2RTemp3(),
             QuantaLY2RTemp4(),
             QuantaLY2RTemp5(),
             )

class QuantaLY2RSensors():
    units = ( QuantaLY2RPSU1(),
              QuantaLY2RPSU2(),
              QuantaLY2RFan1(),
              QuantaLY2RFan2(),
              QuantaLY2RFan3(),
              QuantaLY2RFan4(),
              QuantaLY2RFan5(),
              QuantaLY2RFan6(),
              QuantaLY2RTemp1(),
              QuantaLY2RTemp2(),
              QuantaLY2RTemp3(),
              QuantaLY2RTemp4(),
              QuantaLY2RTemp5(),
              )

class QuantaLY2RPWMs():
    units = (QuantaLY2RPWM1(),
             )

class QuantaLY2RLEDs():
    units = (QuantaLY2RSystemLED(),
             )

class QuantaLY2R(cumulus.platform.Platform):
    name = 'quanta,ly2r'
    chip = cumulus.platform.TridentPlusChip()
    switch = QuantaLY2RSwitch(chip)
    statusleds = QuantaLY2RLEDs()
    pwms = QuantaLY2RPWMs()
    sensors = QuantaLY2RSensors()

Platform = QuantaLY2R
