#
# Copyright 2016.  Cumulus Networks, Inc.
#
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class QuantaLY6QSFPp(cumulus.platform.TridentTwo40GPort):
    _set_led = '''
; QuantaLY6QSFPp - green link, amber activity
set_led:
        port  a

        pushst PORTSTATUS_ZERO
        pack

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        pushst PORTSTATUS_ZERO
        pack
        pushst PORTSTATUS_ZERO
        pack
        pushst PORTSTATUS_ZERO
        pack
        pushst PORTSTATUS_ZERO
        pack
        pushst PORTSTATUS_ZERO
        pack
        pushst PORTSTATUS_ZERO
        pack

        ret

'''
    _set_led_4x10 = '''
;;;
;;; Input : register A, logical port
;;; Input / Output : register B, accumulated port status

;;; For each port accumulate 2 bits of information in register B:
;;;   register B[0] - link enabled
;;;   register B[1] - activity (TX _OR_ RX)

SUB_INTF_LINKEN     equ 0x0  ; link enabled
SUB_INTF_ACTIVITY   equ 0x1  ; activity

sub_intf_info:

        ;;; push OR of current TX/RX activity onto stack
        port a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor

        ;;; push accumlated link activity onto stack
        tst B, SUB_INTF_ACTIVITY
        push cy

        ;;; OR accumlated activity with current activity and store
        ;;; result back in register B
        tor
        pop
        bit B, SUB_INTF_ACTIVITY

        ;;; push current LINKEN onto stack
        pushst  PORTSTATUS_LINKEN

        ;;; push accumlated link enable onto stack
        tst B, SUB_INTF_LINKEN
        push cy

        ;;; OR accumlated LINKEN with current LINKEN and store
        ;;; result back in register B
        tor
        pop
        bit B, SUB_INTF_LINKEN

        ret

;;; QuantaLY6QSFPp_4x10
;;; OR together status of 4 ports for 4x10G QSFP port
;;; Two LED bits per port: yellow(active high), green(active low)
;;; Input: register A - first logical port of 4-port group
;;; Modified: register B - used as scratch space
set_led:

        port a
        pushst PORTSTATUS_ZERO
        pack

        ;;; initialize B
        ld  b,0

        ;;; load sub-0 info
        call    sub_intf_info

        ;;; load sub-1 info
        inc a
        call    sub_intf_info

        ;;; load sub-2 info
        inc a
        call    sub_intf_info

        ;;; load sub-3 info
        inc a
        call    sub_intf_info

        ;;; push accumlated link activity onto stack
        tst B, SUB_INTF_ACTIVITY
        push    cy

        ;;; combine with BLINK state
        push    (BLINK_STATE_P)
        tand
        tinv    ; flash off with activity

        ;;; push accumlated link enable onto stack
        tst B, SUB_INTF_LINKEN
        push    cy

        ;;; AND activity-blink state with accumlated LINKEN
        tand
        pack

        pushst PORTSTATUS_ZERO
        pack
        pushst PORTSTATUS_ZERO
        pack
        pushst PORTSTATUS_ZERO
        pack
        pushst PORTSTATUS_ZERO
        pack
        pushst PORTSTATUS_ZERO
        pack
        pushst PORTSTATUS_ZERO
        pack

        ret

'''
    def __init__(self, label, serdes, leds):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp+')
        cumulus.platform.TridentTwo40GPort.__init__(self,
                                                    label=label,
                                                    serdes=serdes,
                                                    phy_types=None,
                                                    phy_addrs=None,
                                                    phy_layer=sfp.type.upper(),
                                                    leds=leds,
                                                    oversub=True,
                                                    unit=0)
        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if (mode == "4x10G") and (sub != 0):
            # For 4x10G mode only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        if mode == "4x10G":
            call = (fn_name + "_4x10", self._set_led_4x10, led_port)
        elif mode == "40G":
            call = (fn_name, self._set_led, led_port)

        # Only one LED per QSFP on this platform
        led_index = led_index_list[0]
        set_calls[led_index] = call
        set_calls[led_index + 1 ] = None
        set_calls[led_index + 2 ] = None
        set_calls[led_index + 3 ] = None
        set_calls[led_index + 4 ] = None
        set_calls[led_index + 5 ] = None
        set_calls[led_index + 6 ] = None
        set_calls[led_index + 7 ] = None

        return set_calls

class QuantaLY6LED(cumulus.ledup.TridentTwoLEDController):
    num_leds = 128

class QuantaLY6Switch(cumulus.platform.BCMSwitch):
    ledup = QuantaLY6LED
    _qsfp = QuantaLY6QSFPp

    has_phy_ucode_spi_rom = False
    has_phy_ucode_mdio = False

    ports = (
        _qsfp(label="1", serdes=(  0,   1,   2,   3), leds=(0, 0*8)),
        _qsfp(label="2", serdes=(  4,   5,   6,   7), leds=(0, 1*8)),
        _qsfp(label="3", serdes=(  8,   9,  10,  11), leds=(0, 2*8)),
        _qsfp(label="4", serdes=( 12,  13,  14,  15), leds=(0, 3*8)),
        _qsfp(label="5", serdes=( 16,  17,  18,  19), leds=(0, 4*8)),
        _qsfp(label="6", serdes=( 20,  21,  22,  23), leds=(0, 5*8)),
        _qsfp(label="7", serdes=( 24,  25,  26,  27), leds=(0, 6*8)),
        _qsfp(label="8", serdes=( 28,  29,  30,  31), leds=(0, 7*8)),
        _qsfp(label="9", serdes=( 32,  33,  34,  35), leds=(0, 8*8)),
        _qsfp(label="10", serdes=( 36,  37,  38,  39), leds=(0, 9*8)),
        _qsfp(label="11", serdes=( 40,  41,  42,  43), leds=(0, 10*8)),
        _qsfp(label="12", serdes=( 44,  45,  46,  47), leds=(0, 11*8)),
        _qsfp(label="13", serdes=( 48,  49,  50,  51), leds=(0, 12*8)),
        _qsfp(label="14", serdes=( 52,  53,  54,  55), leds=(0, 13*8)),
        _qsfp(label="15", serdes=( 56,  57,  58,  59), leds=(0, 14*8)),
        _qsfp(label="16", serdes=( 60,  61,  62,  63), leds=(0, 15*8)),
        _qsfp(label="17", serdes=( 64,  65,  66,  67), leds=(1, 0*8)),
        _qsfp(label="18", serdes=( 68,  69,  70,  71), leds=(1, 1*8)),
        _qsfp(label="19", serdes=( 72,  73,  74,  75), leds=(1, 2*8)),
        _qsfp(label="20", serdes=( 76,  77,  78,  79), leds=(1, 3*8)),
        _qsfp(label="21", serdes=( 80,  81,  82,  83), leds=(1, 4*8)),
        _qsfp(label="22", serdes=( 84,  85,  86,  87), leds=(1, 5*8)),
        _qsfp(label="23", serdes=( 88,  89,  90,  91), leds=(1, 6*8)),
        _qsfp(label="24", serdes=( 92,  93,  94,  95), leds=(1, 7*8)),
        _qsfp(label="25", serdes=( 96,  97,  98,  99), leds=(1, 8*8)),
        _qsfp(label="26", serdes=(100, 101, 102, 103), leds=(1, 9*8)),
        _qsfp(label="27", serdes=(104, 105, 106, 107), leds=(1, 10*8)),
        _qsfp(label="28", serdes=(108, 109, 110, 111), leds=(1, 11*8)),
        _qsfp(label="29", serdes=(112, 113, 114, 115), leds=(1, 12*8)),
        _qsfp(label="30", serdes=(116, 117, 118, 119), leds=(1, 13*8)),
        _qsfp(label="31", serdes=(120, 121, 122, 123), leds=(1, 14*8)),
        _qsfp(label="32", serdes=(124, 125, 126, 127), leds=(1, 15*8)),
    )
