#
# Copyright (c) 2017 Cumulus Networks, Inc.  All Rights Reserved.
#

import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class QuantaLY7SFPp(cumulus.platform.Maverick10GPort):
    _set_led = '''
;;; Quanta LY7 SFP+ Port
;;;
;;; There are two bits per 10G port.
;;;
;;;                 1st bit
;;;                0        1
;;;            +-------+-------+
;;;          0 |  off  |  off  |
;;;  2nd bit   +-------+-------+
;;;          1 | green |  off  |
;;;            +-------+-------+
;;;
;;; First bit is always zero.  Second bit controls the green LED:
set_led:
        push    PORTSTATUS_ZERO ; first bit is always 0
        pack

        ld      b, (ID_ENABLE_P)
        tst     b, ID_ENABLE_BIT
        jc      id_mode

        port a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack
        ret
'''
    def __init__(self, label, serdes, leds, rx_lane_map, tx_lane_map,
                 rx_polarity_flip, tx_polarity_flip):
        cumulus.platform.Maverick10GPort.__init__(self,
                                                  label=label,
                                                  serdes=serdes,
                                                  phy_types=None,
                                                  phy_addrs=None,
                                                  phy_layer='SFP+',
                                                  leds=leds,
                                                  rx_lane_map=rx_lane_map,
                                                  tx_lane_map=tx_lane_map,
                                                  rx_polarity_flip=rx_polarity_flip,
                                                  tx_polarity_flip=tx_polarity_flip,
                                                  unit=0,
                                                  oversub=True)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        call = (fn_name, self._set_led, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0] + 1] = None

        return set_calls

class QuantaLY7QSFP28(cumulus.platform.Maverick100GPort):
    '''
;;; Quanta LY7 QSFP28 Port
;;;
;;; Each port has four LEDs and each LED has three bits to control its color.
;;;
;;;   bit 1  bit 2  bit 3  color
;;;  -------------------------------
;;;     0      0      0    white (only the first LED turns on)
;;;     1      0      0    yellow (only the first LED turns on)
;;;     0      1      0    blue (only the first LED turns on)
;;;     1      1      0    orange
;;;     0      0      1    green
;;;     1      0      1    reserved
;;;     0      1      1    reserved
;;;     1      1      1    off
    '''
    _set_led_common = '''
id_mode:
        push    0
        cmp     a, (ID_PORT_P)
        jnz     pack_it
        ld      b, (WIGGLE_STATE_P)
        and     b, (WAG_STATE_P)
        push    b
pack_it:
        pack
        ret

set_port_led:
        ld      b, (ID_ENABLE_P)
        tst     b, ID_ENABLE_BIT
        jc      id_mode

        port a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand                    ; if result is a 0, push 1, 1, 1 for off
                                ; is result is a 1, push 0, 0, 1 for green
        tinv
        pop
        push    CY
        push    CY
        pack
        pack
        pushst  PORTSTATUS_ONE
        pack
        ret

set_led_off:
        push    PORTSTATUS_ONE
        pack
        push    PORTSTATUS_ONE
        pack
        push    PORTSTATUS_ONE
        pack
        ret

'''

    _set_led_1x = '''
;;;
;;; In 1x mode, only the green LED of the first bi-color LED is used to
;;; indicate link/activity. The rest are off.
;;;
set_led:
        call    set_port_led

        call    set_led_off
        call    set_led_off
        call    set_led_off

        ret
'''

    _set_led_2x = '''
;;;
;;; In 2x mode, we use the green LED of the first and second RGB LED to
;;; indicate link/activity. The rest are off.
;;;
set_led:
        call    set_port_led

        inc     a
        inc     a
        call    set_port_led

        call    set_led_off
        call    set_led_off

        ret
'''

    _set_led_4x = '''
;;;
;;; In 4x mode, the green LEDs of all four RGB LEDs are used to
;;; indicate link/activity.
;;;
set_led:
        call    set_port_led
        inc     a
        call    set_port_led
        inc     a
        call    set_port_led
        inc     a
        call    set_port_led
        ret
'''

    def __init__(self, label, serdes, leds, rx_lane_map, tx_lane_map,
                 rx_polarity_flip, tx_polarity_flip):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.Maverick100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   leds=leds,
                                                   rx_lane_map=rx_lane_map,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   unit=0,
                                                   oversub=True)

        self.sfp = sfp

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if sub != 0:
            # For breakout modes only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        num_lp = self.num_logical_ports(mode)
        if num_lp == 4:
            call = (fn_name + "_4x", self._set_led_4x, led_port)
        elif num_lp == 2:
            call = (fn_name + "_2x", self._set_led_2x, led_port)
        else:
            call = (fn_name + "_1x", self._set_led_1x, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0]+1] = None
        set_calls[led_index_list[0]+2] = None
        set_calls[led_index_list[0]+3] = None
        set_calls[led_index_list[0]+4] = None
        set_calls[led_index_list[0]+5] = None
        set_calls[led_index_list[0]+6] = None
        set_calls[led_index_list[0]+7] = None

        return set_calls

class QuantaLY7LED(cumulus.ledup.MaverickLEDController):
    led_functions = (QuantaLY7QSFP28._set_led_common)
    num_leds = 128
    has_phys_id = True

class QuantaLY7Switch(cumulus.platform.BCMSwitch):
    ledup = QuantaLY7LED
    led_controllers = set([0, 1])
    _sfp = QuantaLY7SFPp
    _qsfp28 = QuantaLY7QSFP28
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (
        _sfp(label="1",  serdes=( 0,), leds=(0,  0), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="2",  serdes=( 1,), leds=(0,  2), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="3",  serdes=( 2,), leds=(0,  4), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="4",  serdes=( 3,), leds=(0,  6), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),

        _sfp(label="5",  serdes=( 4,), leds=(0,  8), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="6",  serdes=( 5,), leds=(0, 10), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="7",  serdes=( 6,), leds=(0, 12), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="8",  serdes=( 7,), leds=(0, 14), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),

        _sfp(label="9",  serdes=( 8,), leds=(0, 16), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="10", serdes=( 9,), leds=(0, 18), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="11", serdes=(10,), leds=(0, 20), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="12", serdes=(11,), leds=(0, 22), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),

        _sfp(label="13", serdes=(12,), leds=(0, 24), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="14", serdes=(13,), leds=(0, 26), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="15", serdes=(14,), leds=(0, 28), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="16", serdes=(15,), leds=(0, 30), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),

        _sfp(label="17", serdes=(16,), leds=(0, 32), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="18", serdes=(17,), leds=(0, 34), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="19", serdes=(18,), leds=(0, 36), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="20", serdes=(19,), leds=(0, 38), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),

        _sfp(label="21", serdes=(20,), leds=(0, 40), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="22", serdes=(21,), leds=(0, 42), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="23", serdes=(22,), leds=(0, 44), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="24", serdes=(23,), leds=(0, 46), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False,),

        _sfp(label="25", serdes=(36,), leds=(1,  0), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="26", serdes=(37,), leds=(1,  2), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="27", serdes=(38,), leds=(1,  4), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="28", serdes=(39,), leds=(1,  6), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),

        _sfp(label="29", serdes=(40,), leds=(1,  8), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="30", serdes=(41,), leds=(1, 10), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="31", serdes=(42,), leds=(1, 12), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="32", serdes=(43,), leds=(1, 14), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),

        _sfp(label="33", serdes=(44,), leds=(1, 16), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="34", serdes=(45,), leds=(1, 18), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="35", serdes=(46,), leds=(1, 20), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="36", serdes=(47,), leds=(1, 22), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),

        _sfp(label="37", serdes=(48,), leds=(1, 24), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="38", serdes=(49,), leds=(1, 26), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="39", serdes=(50,), leds=(1, 28), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="40", serdes=(51,), leds=(1, 30), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),

        _sfp(label="41", serdes=(52,), leds=(1, 32), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="42", serdes=(53,), leds=(1, 34), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="43", serdes=(54,), leds=(1, 36), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="44", serdes=(55,), leds=(1, 38), tx_lane_map=0x1032, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),

        _sfp(label="45", serdes=(56,), leds=(1, 40), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="46", serdes=(57,), leds=(1, 42), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="47", serdes=(58,), leds=(1, 44), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp(label="48", serdes=(59,), leds=(1, 46), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False,),

        _qsfp28(label="49", serdes=( 32, 33, 34, 35), leds=(0, 64), tx_lane_map=0x3012, rx_lane_map=0x1320, tx_polarity_flip=0x0000, rx_polarity_flip=0x000a,),
        _qsfp28(label="50", serdes=( 28, 29, 30, 31), leds=(0, 56), tx_lane_map=0x0132, rx_lane_map=0x0312, tx_polarity_flip=0x0009, rx_polarity_flip=0x000c,),

        _qsfp28(label="51", serdes=( 68, 69, 70, 71), leds=(1, 64), tx_lane_map=0x2301, rx_lane_map=0x3102, tx_polarity_flip=0x000f, rx_polarity_flip=0x0005,),
        _qsfp28(label="52", serdes=( 64, 65, 66, 67), leds=(1, 56), tx_lane_map=0x2301, rx_lane_map=0x0123, tx_polarity_flip=0x000c, rx_polarity_flip=0x000f,),
    )

vhwmon_path = '/sys/class/misc/vhwmon'

# Intel Rangeley CPU
class QuantaLY7CoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    no_shutdown = True

class QuantaLY7CPUTemp1(QuantaLY7CoreTemp):
    name = 'Temp1'
    description = 'Core 0'
    driver_hwmon = ['temp2']

class QuantaLY7CPUTemp2(QuantaLY7CoreTemp):
    name = 'Temp2'
    description = 'Core 1'
    driver_hwmon = ['temp3']

class QuantaLY7CPUTemp3(QuantaLY7CoreTemp):
    name = 'Temp3'
    description = 'Core 2'
    driver_hwmon = ['temp4']

class QuantaLY7CPUTemp4(QuantaLY7CoreTemp):
    name = 'Temp4'
    description = 'Core 3'
    driver_hwmon = ['temp5']

# Maverick - Internal Temp Sensors
class QuantaLY7MaverickTemp(cumulus.sensors.Temp_Unit):
    description  = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path  = '/cumulus/switchd/run'

class QuantaLY7MaverickTemp1(QuantaLY7MaverickTemp):
    name = 'Temp5'
    driver_hwmon = ['temp1']
    
class QuantaLY7MaverickTemp2(QuantaLY7MaverickTemp):
    name = 'Temp6'
    driver_hwmon = ['temp2']
    
class QuantaLY7MaverickTemp3(QuantaLY7MaverickTemp):
    name = 'Temp7'
    driver_hwmon = ['temp3']
            
class QuantaLY7MaverickTemp4(QuantaLY7MaverickTemp):
    name = 'Temp8'
    driver_hwmon = ['temp4']
    
class QuantaLY7MaverickTemp5(QuantaLY7MaverickTemp):
    name = 'Temp9'
    driver_hwmon = ['temp5']
    
class QuantaLY7MaverickTemp6(QuantaLY7MaverickTemp):
    name = 'Temp10'
    driver_hwmon = ['temp6']
    
class QuantaLY7MaverickTemp7(QuantaLY7MaverickTemp):
    name = 'Temp11'
    driver_hwmon = ['temp7']
    
class QuantaLY7MaverickTemp8(QuantaLY7MaverickTemp):
    name = 'Temp12'
    driver_hwmon = ['temp8']
    
class QuantaLY7MaverickTemp9(QuantaLY7MaverickTemp):
    name = 'Temp13'
    driver_hwmon = ['temp9']

# BMC Sensors
class QuantaLY7BMCTemp(cumulus.sensors.Temp_Unit):
    driver_path = vhwmon_path + '/bmc:temp'

class QuantaLY7BMCTemp1(QuantaLY7BMCTemp):
    name = 'Temp14'
    description = 'Temp_CPU'
    driver_hwmon = ['temp1']
    temp_max = 97

class QuantaLY7BMCTemp2F2B(QuantaLY7BMCTemp):
    name = 'Temp15'
    description = 'Temp_Ambient_0'
    driver_hwmon = ['temp2']
    temp_max = 68
    temp_crit = 71

class QuantaLY7BMCTemp2B2F(QuantaLY7BMCTemp):
    name = 'Temp15'
    description = 'Temp_Ambient_0'
    driver_hwmon = ['temp2']
    temp_max = 56
    temp_crit = 58

class QuantaLY7BMCTemp3F2B(QuantaLY7BMCTemp):
    name = 'Temp16'
    description = 'Temp_Ambient_1'
    driver_hwmon = ['temp3']
    temp_max = 67
    temp_crit = 70

class QuantaLY7BMCTemp3B2F(QuantaLY7BMCTemp):
    name = 'Temp16'
    description = 'Temp_Ambient_1'
    driver_hwmon = ['temp3']
    temp_max = 64
    temp_crit = 67

class QuantaLY7BMCTemp4F2B(QuantaLY7BMCTemp):
    name = 'Temp17'
    description = 'Temp_Ambient_2'
    driver_hwmon = ['temp4']
    temp_max = 56
    temp_crit = 58

class QuantaLY7BMCTemp4B2F(QuantaLY7BMCTemp):
    name = 'Temp17'
    description = 'Temp_Ambient_2'
    driver_hwmon = ['temp4']
    temp_max = 67
    temp_crit = 70

class QuantaLY7BMCTemp5F2B(QuantaLY7BMCTemp):
    name = 'Temp18'
    description = 'Temp_Ambient_3'
    driver_hwmon = ['temp5']
    temp_max = 67
    temp_crit = 70

class QuantaLY7BMCTemp5B2F(QuantaLY7BMCTemp):
    name = 'Temp18'
    description = 'Temp_Ambient_3'
    driver_hwmon = ['temp5']
    temp_max = 65
    temp_crit = 68

class QuantaLY7BMCTemp6F2B(QuantaLY7BMCTemp):
    name = 'Temp19'
    description = 'Temp_Ambient_4'
    driver_hwmon = ['temp6']
    temp_max = 72
    temp_crit = 75

class QuantaLY7BMCTemp6B2F(QuantaLY7BMCTemp):
    name = 'Temp19'
    description = 'Temp_Ambient_4'
    driver_hwmon = ['temp6']
    temp_max = 69
    temp_crit = 72

class QuantaLY7BMCTemp7F2B(QuantaLY7BMCTemp):
    name = 'Temp20'
    description = 'Temp_Ambient_5'
    driver_hwmon = ['temp7']
    temp_max = 74
    temp_crit = 77

class QuantaLY7BMCTemp7B2F(QuantaLY7BMCTemp):
    name = 'Temp20'
    description = 'Temp_Ambient_5'
    driver_hwmon = ['temp7']
    temp_max = 75
    temp_crit = 78

class QuantaLY7BMCTemp8F2B(QuantaLY7BMCTemp):
    name = 'Temp21'
    description = 'Temp_Ambient_6'
    driver_hwmon = ['temp8']
    temp_max = 70
    temp_crit = 73

class QuantaLY7BMCTemp8B2F(QuantaLY7BMCTemp):
    name = 'Temp21'
    description = 'Temp_Ambient_6'
    driver_hwmon = ['temp8']
    temp_max = 73
    temp_crit = 76

class QuantaLY7Fan(cumulus.sensors.Fan_Unit):
    driver_path = vhwmon_path + '/bmc:fan'
    no_pwm = True

class QuantaLY7Fan1(QuantaLY7Fan):
    name = 'Fan1'
    description = 'Fan 1_1'
    driver_hwmon = ['fan1']

class QuantaLY7Fan2(QuantaLY7Fan):
    name = 'Fan2'
    description = 'Fan 1_2'
    driver_hwmon = ['fan2']

class QuantaLY7Fan3(QuantaLY7Fan):
    name = 'Fan3'
    description = 'Fan 2_1'
    driver_hwmon = ['fan3']

class QuantaLY7Fan4(QuantaLY7Fan):
    name = 'Fan4'
    description = 'Fan 2_2'
    driver_hwmon = ['fan4']

class QuantaLY7Fan5(QuantaLY7Fan):
    name = 'Fan5'
    description = 'Fan 3_1'
    driver_hwmon = ['fan5']

class QuantaLY7Fan6(QuantaLY7Fan):
    name = 'Fan6'
    description = 'Fan 3_2'
    driver_hwmon = ['fan6']

class QuantaLY7Fan7(QuantaLY7Fan):
    name = 'Fan7'
    description = 'Fan 4_1'
    driver_hwmon = ['fan7']

class QuantaLY7Fan8(QuantaLY7Fan):
    name = 'Fan8'
    description = 'Fan 4_2'
    driver_hwmon = ['fan8']

class QuantaLY7Fan9(QuantaLY7Fan):
    name = 'Fan9'
    description = 'Fan 5_1'
    driver_hwmon = ['fan9']

class QuantaLY7Fan10(QuantaLY7Fan):
    name = 'Fan10'
    description = 'Fan 5_2'
    driver_hwmon = ['fan10']

class QuantaLY7PSU(cumulus.sensors.PSU_Unit):
    driver_hwmon = ['power1']

class QuantaLY7PSU1(QuantaLY7PSU):
    name = 'PSU1'
    driver_path = vhwmon_path + '/bmc:psu1'
    psu_ok = 0

    def get_status(self):
        val = self.attrs.get('power1_input', 0)
        if val != 0:
            val = 1
        self.attrs['psu_pwr1_all_ok'] = val
        QuantaLY7PSU1.psu_ok = val
        return cumulus.sensors.PSU_Unit.get_status(self)

class QuantaLY7PSU2(QuantaLY7PSU):
    name = 'PSU2'
    driver_path = vhwmon_path + '/bmc:psu2'
    psu_ok = 0

    def get_status(self):
        val = self.attrs.get('power1_input', 0)
        if val != 0:
            val = 1
        self.attrs['psu_pwr2_all_ok'] = val
        QuantaLY7PSU2.psu_ok = val
        return cumulus.sensors.PSU_Unit.get_status(self)

class QuantaLY7PSUFan(cumulus.sensors.Fan_Unit):
    driver_hwmon = ['fan1']
    no_pwm = True

    def get_status(self):
        if not self.psu.psu_ok:
            return self.update_status(None, 'not_ok')
        return cumulus.sensors.Fan_Unit.get_status(self)

class QuantaLY7PSU1Fan1(QuantaLY7PSUFan):
    name = 'PSU1Fan1'
    description = 'PSU1 Fan1'
    psu = QuantaLY7PSU1()
    driver_path = psu.driver_path

class QuantaLY7PSU2Fan1(QuantaLY7PSUFan):
    name = 'PSU2Fan1'
    description = 'PSU2 Fan1'
    psu = QuantaLY7PSU2()
    driver_path = psu.driver_path

def quanta_ly7_airflow():
    f2b_bytes = ['0x46', '0x42']
    b2f_bytes = ['0x42', '0x46']

    try:
        plat = subprocess.check_output(('/usr/bin/platform-detect', '-m'))
    except OSError:
        return None

    if plat.strip() != 'quanta,ly7_rglbmc':
        return None
    try:
        info = subprocess.check_output('/usr/cumulus/bin/decode-syseeprom')
    except:
        info = ''

    bytes = ''
    for l in info.split('\n'):
        if 'Vendor Extension' in l:
            bytes = l.split()[-2:]
            break

    if bytes == b2f_bytes:
        return 'B2F'
    elif bytes == f2b_bytes:
        return 'F2B'
    else:
        return None

class QuantaLY7Sensors():
    units = (QuantaLY7CPUTemp1(),
             QuantaLY7CPUTemp2(),
             QuantaLY7CPUTemp3(),
             QuantaLY7CPUTemp4(),
             QuantaLY7MaverickTemp1(),
             QuantaLY7MaverickTemp2(),
             QuantaLY7MaverickTemp3(),
             QuantaLY7MaverickTemp4(),
             QuantaLY7MaverickTemp5(),
             QuantaLY7MaverickTemp6(),
             QuantaLY7MaverickTemp7(),
             QuantaLY7MaverickTemp8(),
             QuantaLY7MaverickTemp9(),
             QuantaLY7BMCTemp1(),
             QuantaLY7Fan1(),
             QuantaLY7Fan2(),
             QuantaLY7Fan3(),
             QuantaLY7Fan4(),
             QuantaLY7Fan5(),
             QuantaLY7Fan6(),
             QuantaLY7Fan7(),
             QuantaLY7Fan8(),
             QuantaLY7Fan9(),
             QuantaLY7Fan10(),
             QuantaLY7PSU1(),
             QuantaLY7PSU2(),
             QuantaLY7PSU1Fan1(),
             QuantaLY7PSU2Fan1(),
             )
    f2b_temps = (QuantaLY7BMCTemp2F2B(),
                 QuantaLY7BMCTemp3F2B(),
                 QuantaLY7BMCTemp4F2B(),
                 QuantaLY7BMCTemp5F2B(),
                 QuantaLY7BMCTemp6F2B(),
                 QuantaLY7BMCTemp7F2B(),
                 QuantaLY7BMCTemp8F2B(),
                 )
    b2f_temps = (QuantaLY7BMCTemp2B2F(),
                 QuantaLY7BMCTemp3B2F(),
                 QuantaLY7BMCTemp4B2F(),
                 QuantaLY7BMCTemp5B2F(),
                 QuantaLY7BMCTemp6B2F(),
                 QuantaLY7BMCTemp7B2F(),
                 QuantaLY7BMCTemp8B2F(),
                 )
    # Default to F2B if airflow can't be figured out.
    if quanta_ly7_airflow() == 'B2F':
        units = units + b2f_temps
    else:
        units = units + f2b_temps

class QuantaLY7BMC(cumulus.platform.BMC):
    name = 'bmc'
    type = 'ipmibmc'

class QuantaLY7(cumulus.platform.Platform):
    name = 'quanta,ly7_rglbmc'
    chip = cumulus.platform.MaverickChip()
    switch = QuantaLY7Switch(chip)
    bmc = QuantaLY7BMC()
    sensors = QuantaLY7Sensors()
    statusleds = None
    pwms = None

Platform = QuantaLY7
