#!/usr/bin/python
# Copyright (C) 2017,2018 Cumulus Networks, Inc. all rights reserved
#
# This software is subject to the Cumulus Networks End User License
# Agreement available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt
#
# portvalidator.py -- validate ports.conf and return a list of any lines
#                     containing syntax or platform-dependent errors.
#
# The portvalidator class has two public methods for performing validation
# checks on /etc/cumulus/ports.conf files and displaying the results:
#
# import portwriteerrors
#     After validating, write a nicely formatted list of invalid lines and
#     line numbers to stdout.
#
# import portvalidator
#     Validates syntactic and platform/ASIC-dependent content of a ports.conf
#     file line-by-line.  Always performs syntax validation first, rejects
#     the file for further validation if any errors are found, otherwise
#     continues on to perform multiple platform/ASIC-dependent validations.
#
import sys
import argparse
import os
import logging
import syslog
import copy
import re
from collections import OrderedDict
from validate import Validator, ValidateError

import cumulus.platforms
import portconfig

def is_phantom_port(platform, port):
    '''
    On cel,seastone we have the unfortunate situation wherein a port
    that simply doesn't exist on the platform (i.e. is in the ASIC, but
    is not broken out to a front panel physical port and doesn't exist
    in the platform codebase) must be processed when seen in ports.conf.
    This "phantom" port test allows it to be detected on the fly easily.
    '''
    if port == "34" and platform.name == "cel,seastone":
        return True

    return False

class ArgParseError(RuntimeError):
    pass

class PortsValidationError(ValidateError):
    pass

class ConfigItems:
    def __init__(self, conf_file, platform, debug):
        self.platform = platform
        self.debug = debug
        self.pc = pc = portconfig.Config(platform, dummy=True)

#
# Processor classes.
#
# Each processor must presume that it and all other processors will process
# and postprocess each line.  To the extent that it is possible to do so,
# each processor must insulate its processing and any side effects from the
# others.  Early attempts to calculate when to `terminate' processing of a
# line from within an individual processor ended in inconvenience/mistakes.
# Running each line through all processors, and admitting up front that we
# are doing so, is a bit more computation-heavy but is definitely cleaner.
#

class GangModeProcessor:
    def __init__(self, pc, platform, debug):
        self.pc = pc
        self.platform = platform
        self.debug = debug
        self.gang_leader = None
        self.gang_range = 0
        self.gang_mode = None

    def insidegangrangecheck(self, soft_port, soft_mode):
        #
        # Check to see if a port previously marked ganged would include
        # this port in its gang_range.  If so, this port *must* have
        # an identical ganged mode ('/', same no. of ganged ports and
        # speed) and of course therefore not be disabled.
        #
        debug = self.debug
        if debug:
            print 'insidegangrangecheck: entered'
            print '    gang_range= %d' % (self.gang_range)

        reason = None
        if self.gang_range > 0:
            if debug:
                print 'gang_range > 0 case'
                print '    gang_leader= %s' % self.gang_leader
                print '    gang_range= %d' % self.gang_range
                print '    gang_mode= %s' % self.gang_mode

            if soft_mode != self.gang_mode:
                reason = "port %s is ganged by leader %s but is marked '%s' rather than '%s'" % (soft_port, self.gang_leader, soft_mode, self.gang_mode)

            #
            # Whether the soft mode is valid or not, we want to keep an
            # accurate count of our position within the gang range.
            #
            self.gang_range -= 1
            if self.gang_range == 0:
                self.gang_leader = None
                self.gang_mode = None

        if debug:
            print 'End of insidegangrangecheck()'
            print '    gang_leader= %s' % self.gang_leader
            print '    gang_range= %d' % self.gang_range
            print '    gang_mode= %s' % self.gang_mode
            print '    reason= %s' % (reason)

        return reason

    def find_hw_port(self, label):
        '''
        Find a platform port based upon the port's label. Return the port and
        its index in the list.
        '''
        index = 0
        for hwport in self.platform.switch.ports:
            if hwport.label == label and not hwport.is_fabric:
                return (hwport, index)
            index += 1
        return (None, -1)

    def newgangrangecheck(self, soft_port, soft_mode):
        #
        # Check for a gang leader, and record a few things if we have
        # found one.
        #
        debug = self.debug
        if debug:
            print 'newgangrangecheck: entered'

        reason = None

        if self.gang_leader is not None:
            return reason

        if is_phantom_port(self.platform, soft_port):
            return reason

        (hwport, index) = self.find_hw_port(soft_port)
        if hwport is None:
            reason = "port %s is not specified in the platform port list" % (soft_port,)
            return reason

        if not '/' in soft_mode:
            return reason

        if index % hwport.max_gang_size:
            reason = "port %s is specified as a gang leader with mode '%s' but is not on a %d-port boundary" % (soft_port, soft_mode, hwport.max_gang_size)
            return reason

        self.gang_leader = soft_port
        self.gang_range  = hwport.max_gang_size
        self.gang_mode   = soft_mode

        if debug:
            print 'New gang leader: %s' % self.gang_leader
            print '    range: %d'       % self.gang_range
            print '    mode: %s'        % self.gang_mode
            print '    reason= %s' % (reason)

        return reason

class SplitModeProcessor:
    def __init__(self, pc, platform, debug):
        self.pc = pc
        self.platform = platform
        self.debug = debug
        self.blocking_port = None
        self.num_split = 0
        self.blocked_adj_split_num = 0
        self.adj_label = None

    def insideblockedadjsplitcheck(self, soft_port, soft_mode):
        #
        # Check to see if a previously processed port blocks this one,
        # in which case this port *must* have mode `disable' or `disabled'.
        # Note that self.blocked_port will get set in newblockedadjsplitcheck()
        # for *any* split port, so it's necessary to verify here that any
        # blocking of an adjacent port could occur.
        #
        debug = self.debug
        if debug:
            print 'insideblockedadjsplitcheck: entered'
            print '    blocking_port= %s' % (self.blocking_port)
            print '    blocked_adj_split_num= %d' % (self.blocked_adj_split_num)
            print '    adj_label= %s' % (self.adj_label)

        reason = None

        if self.blocking_port is None:
            return reason

        #
        # First, find our port from its label.
        #
        found_port = False
        for port in self.platform.switch.ports:
            if port.label == soft_port:
                found_port = True
                break

        if found_port == False:
            self.blocking_port = None
            reason = "%s is not a valid port specifier for this platform" % (soft_port)
            return reason

        #
        # Be cautious.  It's possible for a port to have the attribute
        # `blocked_adj_split', but for its value to be `None'.
        # blocked_adj_split_num() assures us that an integer value has
        # been assigned, even if it's 0.
        #
        if hasattr(port, 'blocked_adj_split_num'):
            self.blocked_adj_split_num = port.blocked_adj_split_num()

        if self.blocked_adj_split_num == 0:
            self.blocking_port = None    # now that we know it doesn't block
            self.num_split = 0
            return reason

        if self.num_split < self.blocked_adj_split_num:
            self.blocking_port = None
            self.num_split = 0
            self.blocked_adj_split_num = 0
            return reason

        if soft_mode != 'disable' and soft_mode != 'disabled':
            reason = "%s is blocked by port %s but is marked '%s' rather than disable[d]" % (soft_port, self.blocking_port, soft_mode)

        if debug:
            print 'End of insideblockedadjsplitcheck()'
            print '    blocking_port= %s' % self.blocking_port
            print '    blocked_adj_split_num= %d' % self.blocked_adj_split_num
            print '    adj_label= %s' % self.adj_label
            print '    reason= %s' % (reason)

        self.blocking_port = None
        self.num_split = 0
        self.blocked_adj_split_num = 0
        return reason

    def newblockedadjsplitcheck(self, soft_port, soft_mode):
        #
        # Check for a new split port that blocks some number of
        # adjacent port(s), and record a few things if we have
        # found one.  This generally applies to Mellanox but is written
        # to depend only upon the presence of a `blocked_adj_split'
        # attribute so that that may be implemented on any platform
        # (even Broadcom) where it is relevant.
        #
        debug = self.debug
        if debug:
            print 'newblockedadjsplitcheck: entered'
            print '    blocking_port= %s' % (self.blocking_port)
            print '    blocked_adj_split_num= %d' % (self.blocked_adj_split_num)

        reason = None

        if not 'x' in soft_mode:
            return reason

        found_port = False
        for port in self.platform.switch.ports:
            if port.label == soft_port:
                found_port = True
                break

        if found_port == False:
            reason = "%s is not a valid port specifier for this platform" % (soft_port)
            return reason

        self.blocking_port = port.label
        self.num_split = int(soft_mode.split('x')[0])

        if debug:
            print 'End of newblockedadjsplitcheck'
            print '    reason= %s' % (reason)

        return reason

class MiscProcessor:
    '''
    Catchall validation processing.  Meant to capture corner cases that are
    not easily covered elsewhere, or to be a quick spot to add new cases
    before they are assimilated into other processors or get a new processor
    written for them eventually.

    Items include:
    1. Trident2 ASIC limitation of 52 logical ports per X or Y pipeline.
    2. switchd cannot tolerate all ports disabled on any platform (CM-18115).
    '''
    def __init__(self, pc, platform, debug):
        self.pc = pc
        self.platform = platform
        self.debug = debug
        # ---- Trident2 processing ----
        self.Xpipeline = {}
        self.Ypipeline = {}
        self.Xtotal_logical_ports = 0
        self.Ytotal_logical_ports = 0
        self.Xreported = 0
        self.Yreported = 0
        # ---- all ports disabled processing ----
        self.total_disabled_ports = 0
        self.enabled_port = 0
        self.num_ports = -1
        # ---- cel,seastone processing ----
        self.got33 = False
        self.got34 = False

        # ---- Trident2 processing ----
        if (isinstance(platform.switch.chip,
                cumulus.platform.TridentTwoChip) or
            isinstance(platform.switch.chip,
                cumulus.platform.TridentTwo_56850_Chip) or
            isinstance(platform.switch.chip,
                cumulus.platform.TridentTwo_56854_Chip) or
            isinstance(platform.switch.chip,
                cumulus.platform.TridentTwoPlusChip) or
            isinstance(platform.switch.chip,
                cumulus.platform.TridentTwoPlus_56860_Chip) or
            isinstance(platform.switch.chip,
                cumulus.platform.TridentTwoPlus_56864_Chip)):
            self.chip = 'Trident2'
        else:
            self.chip = None

    def Trident2logicalportcheck(self, soft_port, soft_mode):
        #
        # Count max logical ports contributed by `soft_mode' for this
        # line, and maintain the total in self.total_logical_ports.
        #
        debug = self.debug
        if debug:
            print 'Trident2logicalportcheck: entered'
            print '    self.Xpipeline= '
            print self.Xpipeline
            print '    self.Ypipeline= '
            print self.Ypipeline
            print '    self.Xtotal_logical_ports= %d' % (self.Xtotal_logical_ports)
            print '    self.Ytotal_logical_ports= %d' % (self.Ytotal_logical_ports)

        reason = None
        if self.chip != 'Trident2':
            return reason

        #
        # Compute which pipeline we're in first.
        # No explicit mention of this in the code, so until I can find
        # appropriate documentation for Trident2 it's:
        #
        #     port is in first half of ports => pipeline X
        #     port is in second half of ports => pipeline Y
        #
        found_port = False
        for port in self.platform.switch.ports:
            if port.label == soft_port:
                found_port = True
                if soft_mode == 'disable' or soft_mode == 'disabled':
                    num_logical_ports = 0
                else:
                    num_logical_ports = port.num_logical_ports(soft_mode)

                port_num = int(soft_port) - 1    # XXX: not future-proof ...
                if debug:
                    print "Computed port_num= %d len= %d" % (port_num, len(self.platform.switch.ports) / 2)
                if port_num < len(self.platform.switch.ports) / 2:
                    if debug:
                        print "Added '%s':%d to Xpipeline" % (soft_port, num_logical_ports)
                    self.Xpipeline[soft_port] = num_logical_ports
                    self.Xtotal_logical_ports += num_logical_ports
                else:
                    if debug:
                        print "Added '%s':%d to Ypipeline" % (soft_port, num_logical_ports)
                    self.Ypipeline[soft_port] = num_logical_ports
                    self.Ytotal_logical_ports += num_logical_ports

                break

        if debug:
            print 'End of Trident2logicalportcheck()'
            print '    self.Xpipeline= '
            print self.Xpipeline
            print '    self.Ypipeline= '
            print self.Ypipeline
            print '    self.Xtotal_logical_ports= %d' % (self.Xtotal_logical_ports)
            print '    self.Ytotal_logical_ports= %d' % (self.Ytotal_logical_ports)

        if found_port == False:
            reason = "%s is not a valid port specifier for this platform" % (soft_port)
            return reason

        if self.Xtotal_logical_ports > 52 and self.Xreported == 0:
            self.Xreported = 1
            reason = "Maximum of 52 logical ports exceeded for Trident2 X pipeline (ports 1-%d)" % (len(self.platform.switch.ports) / 2)
            return reason

        if self.Ytotal_logical_ports > 52 and self.Yreported == 0:
            self.Yreported = 1
            reason = "Maximum of 52 logical ports exceeded for Trident2 Y pipeline (ports %d-%d)" % (len(self.platform.switch.ports) / 2 + 1, len(self.platform.switch.ports))
            return reason

        return reason

    def seastoneportcheck(self, soft_port, soft_mode):
        #
        # Simple processing of the ports 33 and 34 on Celestica Seastone
        # platform (CM-19740).  The idea is to:
        #
        # 1. Allow "port 34" to appear in ports.conf, despite the fact that
        #    it's a phantom.  This is the primary reason for scattered
        #    is_phantom_port() checks throughout the validator code.
        # 2. Force "port 33" and "port 34" to be left disabled to allow full
        #    4-way breakout of other ports.  See detailed notes in CM-19740
        #    and CM-14501.
        #
        debug = self.debug
        platform = self.platform
        if debug:
            print 'seastoneportcheck: entered'
            print '    soft_port= %s' % (soft_port)
            print '    soft_mode= %s' % (soft_mode)
            print '    self.got33= %s' % (self.got33)
            print '    self.got34= %s' % (self.got34)

        reason = None

        if platform.name != 'cel,seastone':
            return reason

        if self.got33 and self.got34:
            return reason

        if soft_port == '33':
            self.got33 = True
            if soft_mode != 'disable' and soft_mode != 'disabled':
                reason = 'On platform %s port %s must be present and disabled [cf. RN-563]' % (platform.name, soft_port)
                return reason
        elif soft_port == '34':
            self.got34 = True
            if soft_mode != 'disable' and soft_mode != 'disabled':
                reason = 'On platform %s port %s must be present and disabled [cf. RN-563]' % (platform.name, soft_port)
                return reason

        if debug:
            print 'End of seastoneportcheck()'
            print '    soft_port= %s' % (soft_port)
            print '    soft_mode= %s' % (soft_mode)
            print '    self.got33= %s' % (self.got33)
            print '    self.got34= %s' % (self.got34)

        return reason

    def seastonepost(self):
        #
        # Postprocessor to check whether or not we saw mandatory ports
        # 33 and 34 in disabled state once all ports.conf lines were
        # processed.
        #
        debug = self.debug
        platform = self.platform
        reason = None

        if platform.name != 'cel,seastone':
            return reason

        #
        # Final check on declarations of port 33 and (phantom) port 34.
        #
        if not (self.got33 and self.got34):
            reason = 'On platform %s ports 33 and 34 must be present and disabled [cf. RN-563]' % (platform.name)

        if debug:
            print 'End of seastonepost()'
            print '    self.got33= %s' % (self.got33)
            print '    self.got34= %s' % (self.got34)
            if reason is not None:
                print '    reason= %s' % (reason)

        return reason

    def allportsdisabledcheck(self, soft_port, soft_mode):
        #
        # Simple verification that not all ports are marked disabled
        # (CM-18115).
        #
        debug = self.debug
        if debug:
            print 'allportsdisabledcheck: entered'
            print '    self.total_disabled_ports = %d' % (self.total_disabled_ports)
            print '    self.enabled_port = %d' % (self.enabled_port)
            print '    self.num_ports = %d' % (self.num_ports)

        reason = None
        if soft_mode == 'disable' or soft_mode == 'disabled':
            self.total_disabled_ports += 1
        else:
            self.enabled_port = 1

        if self.enabled_port:
            return reason

        if self.num_ports == -1:
            num_ports = 0
            for port in self.platform.switch.ports:
                if port.is_fabric:
                    continue
                num_ports += 1
            self.num_ports = num_ports

        if debug:
            print 'End of allportsdisabledcheck()'
            print '    self.total_disabled_ports = %d' % (self.total_disabled_ports)
            print '    self.enabled_port = %d' % (self.enabled_port)
            print '    self.num_ports = %d' % (self.num_ports)

        if self.total_disabled_ports >= self.num_ports:
            reason = "switchd does not tolerate all %d ports to be disabled" % (self.num_ports)
            return reason

        return reason

class Portvalidator:
    def __init__(self, file, debug):
        self.platform = platform = cumulus.platforms.probe()
        self.file = file
        self.debug = debug
        self.ci = ci = ConfigItems(file, platform, debug)
        self.gmp = GangModeProcessor(ci, platform, debug)
        self.smp = SplitModeProcessor(ci, platform, debug)
        self.mp = MiscProcessor(ci, platform, debug)

        #
        # Construct the ordered processors dictionary.
        #
        # Tags such as "gang-ins" (Gang mode inside a gang range) or
        # "gang-new" (newly spotted gang leader) are currently for
        # descriptive messages only.
        # Method belongs to the appropriate class as it indicates.
        # Classinstance is the `self' first argument to the method.
        #
        self.processors = OrderedDict([
            ("misc-trid2",[MiscProcessor.Trident2logicalportcheck, self.mp]),
#           ("misc-sea",  [MiscProcessor.seastoneportcheck, self.mp]),
            ("gang-new",  [GangModeProcessor.newgangrangecheck, self.gmp]),
            ("gang-ins",  [GangModeProcessor.insidegangrangecheck, self.gmp]),
            ("split-ins", [SplitModeProcessor.insideblockedadjsplitcheck, self.smp]),
            ("split-new", [SplitModeProcessor.newblockedadjsplitcheck, self.smp]),
            ("misc-allpd",[MiscProcessor.allportsdisabledcheck, self.mp]),
        ])

        #
        # Construct the ordered postprocessors dictionary.
        #
        # These are for any postprocessing that needs to be done after
        # all lines in the ports.conf file have been processed.
        #
        self.postprocessors = OrderedDict([
#           ("misc-sea",  [MiscProcessor.seastonepost, self.mp]),
        ])

# ----------------------------------------------------------
#
#   v a l i d a t e __ p o r t s __ m o d e
#
# ----------------------------------------------------------

    def validate_ports_mode(self, mode) :
        """
        Validates the right hand side of a single ports.conf line

        [1-MAXINT]=[4x10G|4x25G|2x50G|40G|50G|100G|100G/4|/1|2x|disabled|loopback]

        for syntactic correctness.  It does *not* validate whether the values
        given are possible, correct, or desirable for this platform.
        """
        debug = self.debug
        if debug :
            print 'Mode: %s' % mode
        if len(mode) == 0 :
            reason = 'blank mode specifier is invalid'
            if debug :
                print 'validate_ports_mode: %s' % reason
            return (-1, reason)

        if mode.startswith((' ', '\t')) :
            reason = 'mode starting with whitespace is invalid'
            if debug :
                print 'validate_ports_mode: %s' % reason
            return (-1, reason)

        stripped_mode = mode.strip()
        if len(stripped_mode) == 0 :
            reason = 'whitespace-only mode is invalid'
            if debug :
                print 'validate_ports_mode: %s' % reason
            return (-1, reason)

        mode = stripped_mode

        vtortok = Validator()

        # check for simple speeds
        speed_check = 'option("1G", "10G", "13G", "25G", "40G", "50G", "100G")'
        # check for ganged ports
        ganged_check = 'option("2x20G", "2x40G", "2x50G", "4x1G", "4x10G", "4x25G")'
        # check for split ports
        split_check = 'option("40G/4", "100G/4")'
        # check for shorthand gang and split ports
        shorthand_check = 'option("1x", "2x", "4x", "/1", "/2", "/4", "3/2")'
        # check for string options
        string_check = 'option("disable", "disabled", "loopback", "RJ45", "SFP")'

        try :
            vtortok.check(speed_check, mode)
        except ValidateError :
            if debug :
                print 'Speed check for mode %s failed.' % mode
        else :
            if debug :
                print 'validate_ports_mode: Speed check for mode %s passed.' % mode
            return (0, None)

        try :
            vtortok.check(ganged_check, mode)
        except ValidateError :
            if debug :
                print 'Ganged check for mode %s failed.' % mode
        else :
            if debug :
                print 'validate_ports_mode: Ganged check for mode %s passed.' % mode
            return (0, None)

        try :
            vtortok.check(split_check, mode)
        except ValidateError :
            if debug :
                print 'Split check for mode %s failed.' % mode
        else :
            if debug :
                print 'validate_ports_mode: Split check for mode %s passed.' % mode
            return (0, None)

        try :
            vtortok.check(shorthand_check, mode)
        except ValidateError :
            if debug :
                print 'Shorthand check for mode %s failed.' % mode
        else :
            if debug :
                print 'validate_ports_mode: Shorthand check for mode %s passed.' % mode
            return (0, None)

        try :
            vtortok.check(string_check, mode)
        except ValidateError :
            if debug :
                print 'String check for mode %s failed.' % mode
        else :
            if debug :
                print 'validate_ports_mode: String check for mode %s passed.' % mode
            return (0, None)

        reason = 'Mode checks for *G, 4x*G, *G/4, *x, /*, disabled all failed!'
        if debug :
            print 'validate_ports_mode: %s' % reason
        return (-1, reason)

# ----------------------------------------------------------
#
#          v a l i d a t e __ p o r t s __ l a b e l
#
# ----------------------------------------------------------

    def validate_ports_label(self, label):
        '''
        Validates the left hand side of a single ports.conf line, which
        describes a port label.

           m[1-MAXINT]p[1-MAXINT] | L[1-MAXINT] | [1-MAXINT]

        Three forms for the left hand side are accepted:
           1. Modular switch: m[1-MAXINT]p[1-MAXINT] e.g. m3p7
           2. Voyager switch: L[1-MAXINT] e.g. L2
           3. Fixed switch: [1-MAXINT] e.g. 12
        '''
        debug = self.debug
        if debug:
            print 'Port Label: %s' % label

        # Label may start with either 
        #   a. "m" followed by a positive integer followed by "p", or
        #   b. "L"
        # and then must contain a positive integer at the end of the string.
        pattern = '^((m[1-9]\d*p)|L)?[1-9]\d*$'
        result = re.match(pattern, label)
        if result is None:
            raise ValidateError()
        else:
            if debug:
                print 'Label matches valid label pattern: %s' % label

# ----------------------------------------------------------
#
#          v a l i d a t e __ p o r t s __ l i n e
#
# ----------------------------------------------------------

    def validate_ports_line(self, line_num, line):
        '''
        Validate a line that is expected to be roughly in the format:

           [#]<PortLabel>=[4x10G|4x25G|2x50G|40G|50G|100G|100G/4|/1|2x|disabled|loopback]

        Comments (starts with # after leading whitespace is stripped, followed
            by anything up to \n) are automatically valid.  The same is *not*
            true of inline comments following uncommented content in the same line.
        Returns a tuple (retval, reason).
            retval: 0 for valid, -1 for invalid
            reason: if retval == -1, a string explaining why it's invalid
                        or possibly why the validator couldn't validate.
        '''
        debug = self.debug
        if debug:
            print 'Line: %s' % line
        if len(line) == 0:
            if debug:
                print 'validate_ports_line: blank line was declared valid'
            return (0, None)

        stripped_line = line.strip()
        if len(stripped_line) == 0:
            if debug:
                print 'validate_ports_line: whitespace-only line was declared valid'
            return (0, None)

        if stripped_line[0] == '#':
            if (line_num == 1 and
                re.sub(r'\s+', '', stripped_line[1:]) == "validate=0"):

                #
                # Special comment on line 1 only to disable validation of
                # this file.  This warrants ASAP return via exception and
                # no error status will be reported at the top level.  The
                # traceback limit simply inhibits printing the ugly traceback
                # which would have resulted from the exception.
                #
                sys.tracebacklimit = 0
                raise PortsValidationError()

            if debug:
                print 'validate_ports_line: comment was declared valid'
            return (0, None)
        elif stripped_line[0] == '=':
            reason = 'validate_ports_line: blank port specifier is not valid'
            if debug:
                print 'validate_ports_line: %s' % reason
            return (-1, reason)

        line_list = stripped_line.split('=')
        length = len(line_list)
        if length != 2:
            reason = 'malformed line did not look like <port>=<mode>'
            if debug:
                print 'validate_ports_line: %s' % reason
            return (-1, reason)

        #
        # Now we have exactly one = sign, i.e at least it's of the form
        # port_label=mode
        # Validate the port label (substring before = sign).
        #
        if debug:
            print 'Port: %s' % line_list[0]

        try:
            self.validate_ports_label(line_list[0])
        except ValidateError:
            reason = 'Syntax check for port specifier failed on %s' % line_list[0]
            if debug:
                print 'validate_ports_line: %s' % reason
            return (-1, reason)
        else:
            if debug:
                print 'Syntax check for port label passed.'

        mode = line_list[1]
        try:
            (newvalue, reason) = self.validate_ports_mode(mode)
        except Exception, e:
            logging.exception("validate_ports_line: Couldn't validate mode '%s'" % mode)
        else:
            if newvalue < 0:
                if debug :
                    print 'validate_ports_line: %s' % reason
                return (newvalue, reason)
            else:
                if debug:
                    print 'Syntax check for port mode passed.'

        return (0, None)

    def portsyntaxwriteerrors(self, retlist):
        '''
        Write the list of either syntax or platform validator error lines
            (invalid lines returned in (line_num, line, reason) tuples of
            retlist[]) with line numbers to stdout in a nice format.

        Preferred interface is portwriteerrors(), since the same utility
            is used whether errors found are syntax or platform-dependent.
        '''
        #
        # First handle the metaerror case in which ports.conf is uninvolved.
        # By convention these tuples look like: (0, '', <reason string>)
        #
        (line_num, line, reason) = retlist[0]
        if len(retlist) == 1 and line_num == 0 and line == '':
            print 'Error: %s' % (reason)
            return

        #
        # Handle the usual case of multiple tuples in retlist
        #
        print 'Error: %d invalid lines found in %s:' % (len(retlist), self.file)

        for (line_num, line, reason) in retlist:
            line = line.rstrip('\r\n')
            print "[Line %d]:'%s'" % (line_num, line)
            print "  Invalid because: %s" % reason

    def portsyntaxvalidator(self):
        '''
        Read `file' into the lines[] list, stripping trailing carriage returns
            and newlines as we go.  Then validate lines one at a time, and for
            those failing validation add them to a list of tuples containing
            (line number, line, reason).  The list of tuples is returned.
        '''
        debug = self.debug
        retlist = []

        lines = [line.rstrip('\r\n') for line in open(self.file)]

        for line_num, line in enumerate(lines, start=1):
            try:
                (vterror, reason) = self.validate_ports_line(line_num, line)
            except PortsValidationError:
                #
                # This means we found `validate=0' on line 1 and no
                # validation is to be performed.  Just re-raise the
                # exception and get out NOW.
                #
                raise PortsValidationError()
            except Exception:
                logging.exception("This ports.conf line (%d) broke the syntax validator: '%s'" % (line_num, line))
            else:
                if vterror < 0:
                    retlist.append((line_num, line, reason))

        return retlist

    def validate_platform_port_line(self, line, processed_ports):
        #
        # Validate the port specifier (substring before = sign) first.
        # By this point it's valid syntactically, but is it
        # appropriate for this platform?
        #
        # Side effects:
        #    May append a port specifier to processed_ports[] at its
        #        sole discretion
        #
        # Returns a reason string:
        #    None: port specifier is valid
        #    Otherwise: Reason port specifier was found to be invalid
        #
        debug = self.debug
        [soft_port, soft_mode] = line.strip().split('=')
        if debug:
            print 'Port: %s' % soft_port

        #
        # First check to see if this port is a duplicate (i.e. we
        # have already seen it at least once and inserted it into
        # processed_ports[]).  If so, don't process it again -
        # just move on to the next line.
        #
        found_port = False
        for port in processed_ports:
            if port == soft_port:
                found_port = True
        if found_port:
            reason = "Duplicate line for port %s found" % (soft_port)
            return reason

        # It's not a dup - process it.
        platform = self.platform
        for port in platform.switch.ports:
            if port.is_fabric:
                continue
            if port.label == soft_port:
                found_port = True
                break

        if not found_port:
            #
            # Hack to allow any phantom ports to be processed.
            #
            if is_phantom_port(platform, soft_port):
                if debug:
                    print 'validate_platform_port_line: Found phantom port %s' % soft_port
            else:
                reason = "%s is not a valid port specifier for this platform" % (soft_port)
                processed_ports.append(soft_port)
                return reason

        processed_ports.append(soft_port)
        return None

    def validate_platform_mode_line(self, line, processed_ports):
        #
        # Validate the mode specifier (substring after = sign) first.
        # By this point it's valid syntactically, but is it
        # appropriate for this platform?
        #
        # Side effects:
        #
        # Returns a reason string:
        #    None: mode is valid
        #    Otherwise: Reason mode was found to be invalid
        #
        debug = self.debug
        common_modes = ['disable', 'disabled', 'loopback', 'RJ45', 'SFP']
        [soft_port, soft_mode] = line.strip().split('=')
        if debug:
            print 'Mode: %s' % soft_mode

        #
        # Setup for main mode processing loop.
        #
        platform = self.platform
        found_port = False
        for port in platform.switch.ports:
            if port.is_fabric or port.label != soft_port:
                continue
            found_port = True
            break

        if found_port == False:
            #
            # Hack to allow any phantom ports to be processed.
            #
            # XXX: It's worth considering just bypassing the entire
            #   processing loop for phantom port lines here - what's the
            #   point of it anyway?  The phantom port checks scattered
            #   around the code are there to stay anyway, and once
            #   they're in place and working I don't see much need for a
            #   Seastone processor any longer.  Bypassing would give us
            #   zero chance of barfing on phantom port issues in other
            #   processors and would make it easier for everyone to
            #   maintain and extend the validator.
            #
            if is_phantom_port(platform, soft_port):
                if debug:
                    print 'validate_platform_mode_line: Processing phantom port %s' % soft_port
            else:
                reason = "%s is not a valid port specifier for this platform" % (soft_port)
                return reason

        #
        # First check to see if the soft mode is in the list of modes
        # *for this port* on this platform.  Common soft modes such as
        # `disable[d]', `loopback' and `RJ45' get a free ride on this
        # check (XXX: are they ever forbidden on *any* port on *any*
        # platform?)
        #
        if debug:
            print 'Before completed and common mode checks'

        if not is_phantom_port(platform, soft_port):
            if not (soft_mode in port.modes or soft_mode in common_modes):
                completed_mode = self.config_map[soft_port]
                if completed_mode in port.modes:
                    if debug:
                        print 'Completing original soft mode %s to %s' % (soft_mode, completed_mode)
                    soft_mode = completed_mode
                else:
                    reason = 'Mode %s is not supported for port %s' % (soft_mode, soft_port)
                    return reason

        #
        # Compact main mode processing loop, calls processors
        # with standard arguments in our desired sequence.
        #
        if debug:
            print 'Top of mode processing loop'

        for processor_tag, processor_list in self.processors.items():
            processor, classinst = processor_list
            reason = processor(classinst, soft_port, soft_mode)
            if reason is not None:
                if debug:
                    print 'Terminating after %s' % (processor_tag)
                    print '    reason is %s' % (reason)
                return reason

        if debug:
            print 'Bottom of mode processing loop'

        return None

    def complete_config_map(self):
        '''
        We need something like the config_map that we get from the Config
        class to do platform validation.  Unfortunately, that map is only
        sparsely filled in for ports that actually appear in the ports.conf
        file.  For validation, we really want a map that is complete in
        that it includes all nonfabric ports for the platform, including
        any with default modes that didn't appear in ports.conf, so we
        won't have to worry about missing keys.
        For that reason we build it out here.

        It's worth mentioning that this must follow our syntax validation
        check on ports.conf - it catches some cases that ConfigParser
        does not, such as `=' alone on a line - making map completion easier.
        '''

        #
        # First get a working copy of the config_map we already have.
        #
        pc = self.ci.pc
        file = self.file
        platform = self.platform
        debug = self.debug

        ports_conf = open(file).read()
        pc.config(ports_conf)
        self.config_map = copy.deepcopy(pc.config_map)

        #
        # Then plow through all the ports to complete our map.
        #
        for port in platform.switch.ports:
            if port.is_fabric:
                continue

            found_port = False
            for cfglabel in self.config_map.keys():
                if port.label == cfglabel:
                    found_port = True
                    if debug:
                        print 'complete_config_map: found config_map key %s' % (cfglabel)
                    break

            if found_port == False:
                self.config_map[port.label] = port.default_mode
                if debug:
                    print 'complete_config_map: added missing config_map (k,v) (%s,%s)' % (port.label, port.default_mode)

        self.config_map = pc._complete_modes(self.config_map)
        if debug:
            print 'complete_config_map: final config_map:'
            print self.config_map

    def portplatformvalidator(self, forced_platform=None):
        '''
        Read `file' into the lines[] list, stripping trailing carriage returns
            and newlines as we go.  Then validate lines one at a time and
            keep track of validation state machine info (physical ports
            processed, logical ports consumed per pipeline and per switch, etc.)
            For those failing validation, add a descriptive entry to the list
            of tuples containing (line number, line, reason).  The list
            (retlist) of tuples is returned.

        For each tuple:
            line_num: (int) line number of invalid line in ports.conf file
            line: (str) copy of invalid line
            reason: (str) explanation of why line is invalid or possibly
                        why it couldn't be validated.
        '''

        debug = self.debug
        file = self.file

        #
        # Perform the early disqualification cases.
        # These generally produce metaerrors that have nothing to do with
        # ports.conf but prevent further validation for some reason anyway.
        #
        platform = self.platform
        retlist = []
        if platform.switch == None:
            reason = 'Platform type %s has no switch' % (platform.name)
            retlist.append((0, '', reason))
            return retlist

        if len(platform.switch.ports) == 0:
            if platform.name == 'cumulus,vx':
                #
                # No switch ports are expected on cumulus,vx - return w/no error
                #
                return retlist

            reason = 'Platform type %s has no switch ports defined' % (platform.name)
            retlist.append((0, '', reason))
            return retlist

        #
        # Initialise the config_map from portconfig.
        #
        self.complete_config_map()

        if debug:
            print "Valid modes for platform %s by port are:" % (platform.name)

            for port in platform.switch.ports:
                if port.is_fabric:
                    continue
                sys.stdout.write('%s: ' % (port.label))
                if len(port.modes) > 0:
                    print port.modes
                    sys.stdout.write('Complete: %s\n' % self.config_map[port.label])
                else:
                    sys.stdout.write('\n')

                if hasattr(port, 'blocked_adj_split_num'):
                    self.blocked_adj_split_num = port.blocked_adj_split_num()
                    sys.stdout.write('Blocked: ')
                    print self.blocked_adj_split_num

        #
        # The main platform validation loop.
        # Read in ports.conf lines one at a time, process, and
        # update counts/stored info for validation state machine.
        #
        # It's worth noting that this file is presumed to have
        # passed a syntax check, so lines should be well-formed
        # with <port>=<mode> syntax and have reasonable values
        # for those fields.
        #
        processed_ports = []
        lines = [line.rstrip('\r\n') for line in open(file)]

        for line_num, line in enumerate(lines, start=1):
            if debug:
                print 'Line: %s' % line
            if len(line) == 0:
                if debug:
                    print 'portplatformvalidator: blank line was declared valid'
                continue

            stripped_line = line.strip()
            if len(stripped_line) == 0:
                if debug:
                    print 'portplatformvalidator: whitespace-only line was declared valid'
                continue

            if stripped_line[0] == '#':
                if debug:
                    print 'portplatformvalidator: comment was declared valid'
                continue

            #
            # Port specifier validation.
            #
            reason = self.validate_platform_port_line(line, processed_ports)
            if reason is not None:
                retlist.append((line_num, line, reason))

            #
            # Mode specifier validation.
            #
            reason = self.validate_platform_mode_line(line, processed_ports)
            if reason is not None:
                retlist.append((line_num, line, reason))

        #
        # Postprocessing loop, calls postprocessors for any final stuff.
        #
        if debug:
            print 'Top of postprocessing loop'

        for processor_tag, processor_list in self.postprocessors.items():
            postprocessor, classinst = processor_list
            reason = postprocessor(classinst)
            if reason is not None:
                if debug:
                    print 'Terminating after %s' % (processor_tag)
                    print '    reason is %s' % (reason)
                retlist.append((line_num, line, reason))

        if debug:
            print 'Bottom of postprocessing loop'

        if debug:
            print "Processed soft ports for file %s:" % (file)
            print processed_ports

        return retlist

    def portvalidator(self):
        '''
        Perform first syntactic, then platform/ASIC-dependent validation
            on ports.conf (self.file).
        Build a list of tuples containing (line number, line, reason) from
            portsyntaxvalidator(); if any syntax errors are found, return
            the list (retlist) without performing platform validation.  If
            no syntax errors were found, build retlist from
            portplatformvalidator().  In either case the list (retlist) is
            returned and the caller may then use portwriteerrors(retlist)
            to nicely format the list for output to stdout.
        '''
        debug = self.debug

        try:
            retlist = self.portsyntaxvalidator()
        except PortsValidationError:
            print 'validate=0 line found, no validation will be performed'
            return []

        if len(retlist) > 0:
            return retlist

        retlist = self.portplatformvalidator()
        return retlist

    def portwriteerrors(self, retlist):
        self.portsyntaxwriteerrors(retlist)
