#
# Copyright 2014.  Cumulus Networks, Inc.
#
# switchconfiggen.py --
#   Platform dependent switch config generation.
#
#   This script is invoked from update-ports prior to its porttab/phytab/etc. creation.
#   For a given platform, switchconfiggen is free to generate any dynamic config files
#   on which update-ports/switchd may depend.

import os
import time
import sys
import subprocess

import cumulus.portconfig
import cumulus.platforms

try:
    import xml.etree.cElementTree as ET
except ImportError:
    import xml.etree.ElementTree as ET


_mlx_switch_config_template_dir = '/etc/cumulus/'
_mlx_switch_config_template_base = 'switch_config_template'
_mlx_switch_config_ext = '.xml'
_mlx_switch_config_template_full = ''.join([_mlx_switch_config_template_dir, 
                                            _mlx_switch_config_template_base,
                                            _mlx_switch_config_ext])
_mlx_switch_config_install_dir = '/etc/mlx/'
_mlx_switch_config_base = 'switch_config'
_mlx_switch_config_install_full = ''.join([_mlx_switch_config_install_dir, 
                                           _mlx_switch_config_base,
                                           _mlx_switch_config_ext])


########################################################################
#
# Plaform specific switch config generation
#
########################################################################
def switchconfiggen(sdk_config, 
                    ports_conf_fname='/etc/cumulus/ports.conf',
                    autogen_comment='',
                    verbose=False):

    global _mlx_switch_config_template_dir
    global _mlx_switch_config_template_base
    global _mlx_switch_config_template_full
    global _mlx_switch_config_base
    global _mlx_switch_config_ext
    global _mlx_switch_config_install_dir
    global _mlx_switch_config_install_full

    # Mellanox platform
    #
    if sdk_config.platform.switch.chip.sw_base == 'mlx':

        # Create the install directory for switch_config.xml file
        # if it does not already exist
        #
        if not os.path.exists(_mlx_switch_config_install_dir):
            os.makedirs(_mlx_switch_config_install_dir)
            if not os.path.exists(_mlx_switch_config_install_dir):
                sys.stderr.write('Unable to create directory %s\n' 
                                 % _mlx_switch_config_install_dir)
                sys.exit(-1)

        # Parse the initial XML template contents
        #
        tree = ET.ElementTree(file=_mlx_switch_config_template_full)
        if not tree:
            sys.stderr.write('XML parse error on %s\n' 
                             % _mlx_switch_config_template_full)
            sys.exit(-1)

        # Locate swids section
        #
        swids_tag = 'swids'
        swids_elem = tree.find(swids_tag)
        if swids_elem is None:
            sys.stderr.write('error: %s missing section %s\n' 
                             % (_mlx_switch_config_template_full, swids_tag))
            sys.exit(-1)

        my_swid = int(swids_elem.text.split(',')[0])

        # Locate device-mac-address section
        #
        dev_mac_tag = 'device/device-mac-address'
        dev_mac_elem = tree.find(dev_mac_tag)
        if dev_mac_elem is None:
            sys.stderr.write('error: %s missing section %s\n' 
                             % (_mlx_switch_config_template_full, dev_mac_tag))
            sys.exit(-1)

        # Locate sections concerning ports
        #
        num_pports_tag = 'device/number-of-physical-ports'
        num_pports_elem = tree.find(num_pports_tag)
        if num_pports_elem is None:
            sys.stderr.write('error: %s missing section %s\n' 
                             % (_mlx_switch_config_template_full, num_pports_tag))
            sys.exit(-1)

        ports_list_tag = 'device/ports-list'
        ports_list_elem = tree.find(ports_list_tag)
        if ports_list_elem is None:
            sys.stderr.write('error: %s missing section %s\n' 
                             % (_mlx_switch_config_template_full, ports_list_tag))
            sys.exit(-1)

        # Prepend autogen comment
        #
        tree.getroot().insert(0, ET.Comment(autogen_comment))

        # Create intermediate dictionary of active (configured) port structures
        #
        if verbose:
            sys.stdout.write('creating intermediate active ports dictionary\n')

        active_ports = {}
        for port in sdk_config.platform.switch.ports:

            # sdk_config.config_map contains a dict of port label=speed_mode
            # mappings obtained from ports.conf
            #
            if port.configurable(sdk_config.config_map):
                active_ports[port.label] = port

        # Instantiate port-info elements and add to XML tree
        #
        if verbose:
            sys.stdout.write('instantiating port-info elements for %s\n' 
                             % _mlx_switch_config_install_full)


        # Instantiate in the order of sdk_config.platform.switch.ports (i.e., 
        # ascending order from first port label to last) so switchd can build
        # its internal port table in the same predictive order.
        #
        total_num_pports = 0
        for port in sdk_config.platform.switch.ports:
            curr_port = active_ports.get(port.label)
            if curr_port is None:
                continue

            # Check to see if current port would have been blocked
            # by its adjacent splitable port.
            #
            blocked_adj_split_num = curr_port.blocked_adj_split_num()
            if blocked_adj_split_num > 0:
                adj_port = active_ports.get(curr_port.adj_label)
                if not adj_port is None:
                    adj_speed_mode = sdk_config.config_map.get(adj_port.label)
                    if adj_port.num_logical_ports(adj_speed_mode) >= blocked_adj_split_num:
                        sys.stderr.write('%s: port %s is not available when port %s is split into %u ports\n' 
                                         % (ports_conf_fname, curr_port.label,
                                            adj_port.label, blocked_adj_split_num))
                        sys.exit(-1)


            # Validate speed mode
            #
            speed_mode = sdk_config.config_map.get(curr_port.label)
            num_lports = curr_port.num_logical_ports(speed_mode)
            if num_lports == 0:
                sys.stderr.write('%s: port %s unsupported speed mode %s\n' 
                                 % (ports_conf_fname, curr_port.label, speed_mode))
                sys.exit(-1)

            # Ensure the split is legal in terms of number of resulting ports
            #
            num_splits_allowed = len(curr_port.hw_ports)
            if num_lports > num_splits_allowed:
                if num_splits_allowed < 2:
                    sys.stderr.write('%s: splitting port %s is not allowed\n' 
                                     % (ports_conf_fname, curr_port.label))
                else:
                    sys.stderr.write('%s: port %s can only be split into %u ports\n' 
                                     % (ports_conf_fname, curr_port.label, num_splits_allowed))
                sys.exit(-1)

            # Instantiate new port-info element for XML file, one for each num_lports
            #
            total_num_pports += num_lports
            for p in range(num_lports):

                port_info_elem = ET.SubElement(ports_list_elem, 'port-info')
                port_label_elem   = ET.SubElement(port_info_elem, 'port-label')
                local_port_elem   = ET.SubElement(port_info_elem, 'local-port')
                mapping_mode_elem = ET.SubElement(port_info_elem, 'mapping-mode')
                width_elem        = ET.SubElement(port_info_elem, 'width')
                lanes_elem        = ET.SubElement(port_info_elem, 'lanes')
                lane_module_elem  = ET.SubElement(port_info_elem, 'lane-to-module')
                port_mode_elem    = ET.SubElement(port_info_elem, 'port-mode')
                port_speed_elem   = ET.SubElement(port_info_elem, 'port-speed')
                swid_elem         = ET.SubElement(port_info_elem, 'swid')
                autoneg_elem      = ET.SubElement(port_info_elem, 'autoneg')

                # port-label: Label port number (port number on the front panel)
                #
                port_label_elem.text = curr_port.label

                # local-port: Local port number
                #
                local_port_elem.text = str(curr_port.hw_intf_num(speed_mode, p))

                # mapping-mode: 0 - port is not active , 1 - active port
                #
                mapping_mode_elem.text = '1'

                # port-mode: 0 - stacking disabled, 1 - stacking enabled, 2 - tca connector, 3 - cpu port
                #
                port_mode_elem.text = '0'

                # swid: Port's SWID number (SWID number should be in SWIDs section , zero based)
                #
                swid_elem.text = str(my_swid)

                # port-speed
                #
                port_speed_elem.text = str(curr_port.speed_bmap(speed_mode))

                # lanes: which lanes are allocated for this port 
                #
                lanes_elem.text = curr_port.xml_lanes(speed_mode, p)

                # width
                #
                width_elem.text = str(curr_port.serdes_width(speed_mode))

                # lane-to-module: Lane to module (Lane<i> -> Module<i>)
                #
                lane_module_elem.text = curr_port.xml_modules(speed_mode, p)

                # autoneg: 0 - not enabled, 1 - enabled
                #
                autoneg_elem.text = "1"


        # Update number-of-physical-ports
        #
        num_pports_elem.text = str(total_num_pports)

        # Update device-mac-address from the macbase
        #
        dev_mac_elem.text = '%x:%x:%x:%x:%x:%x' % \
                            (sdk_config.macbase[0], sdk_config.macbase[1], sdk_config.macbase[2], 
                             sdk_config.macbase[3], sdk_config.macbase[4], sdk_config.macbase[5]) 

        # Mellanox Spectrum requires that the macbase be aligned on a 64-address
        # boundary and that 128 MACs be allocated
        if "/mlx/Spectrum" in sdk_config.platform.switch.chip.chip_dir_list:
            if sdk_config.macrange < 64:
                sys.stderr.write('*** The ONIE EEPROM on this switch is not compatible with Cumulus Linux ***\n')
                sys.stderr.write('***    The number of MAC addresses allocated is %d.\n' % (sdk_config.macrange,))
                sys.stderr.write('***    There must be at least 64 MAC addresses allocated.\n')
                sys.stderr.write('***    Contact Mellanox to obtain a switch with an EEPROM which is compatible with Cumulus Linux.\n')
                sys.exit(-1)
            if sdk_config.macbase[5] & 0x3F:
                sys.stderr.write('*** The ONIE EEPROM on this switch is not compatible with Cumulus Linux ***\n')
                sys.stderr.write('***    The base MAC address is %s.\n' % (dev_mac_elem.text,))
                sys.stderr.write('***    The least significant byte (%x) may not have any of the 6 least significant bits set.\n' % (sdk_config.macbase[5],))
                sys.stderr.write('***    Contact Mellanox to obtain a switch with an EEPROM which is compatible with Cumulus Linux.\n')
                sys.exit(-1)

        if verbose:
            sys.stdout.write('device-mac-address %s\n' % dev_mac_elem.text)

        # Generate final switch_config.xml file
        #
        if verbose:
            sys.stdout.write('writing %s\n' % _mlx_switch_config_install_full)

        tree.write(_mlx_switch_config_install_full)
