#!/usr/bin/env python
# Copyright 2017 Cumulus Networks, Inc.
#
# Neighbor manager packet helpers
#
# dpkt is used for most packet parsing and construction. however only basic ipv6
# parsing is available via dpkt. This module supplements dpkt where necessary.
# XXX: overtime we should deprecate this module and use dpkt entirely (when it
# is ready)

import binascii
import ctypes
import dpkt
import ipaddr
import logging
import socket

# Reference iana.org
IP6_ALL_NODES_ADDR = 'FF02::1'
IP6_ALL_NODES_LL_ADDR = '33:33:00:00:00:01'

# Reference RFC4861
ICMP6_OPT_SRC_LL_ADDR = 1
ICMP6_OPT_TGT_LL_ADDR = 2

allNodesMcastAddr = ipaddr.IPAddress(IP6_ALL_NODES_ADDR)
allNodesMcastAddrStr = socket.inet_pton(socket.AF_INET6, str(allNodesMcastAddr))
libc_handle = ctypes.cdll.LoadLibrary('libc.so.6')

# from include/uapi/linux/if_packet.h
class struct_sockaddr_ll(ctypes.Structure):
    _fields_ = [('sll_family', ctypes.c_ushort),
                ('sll_protocol', ctypes.c_short),
                ('sll_ifindex', ctypes.c_int),
                ('sll_hatype', ctypes.c_ushort),
                ('sll_pkttype', ctypes.c_ubyte),
                ('sll_halen', ctypes.c_ubyte),
                ('sll_addr', ctypes.c_ubyte * 8)]

# from include/uapi/linux/uio.h - data buf will passed in via this
class struct_iovec(ctypes.Structure):
    _fields_ = [("iov_base", ctypes.c_void_p),
                ("iov_len", ctypes.c_size_t)]

# from include/linux/socket.h (struct user_msghdr)
class struct_msghdr(ctypes.Structure):
    _fields_ = [('msg_name', ctypes.c_void_p),
                ('msg_namelen', ctypes.c_uint32),
                ('msg_iov', ctypes.POINTER(struct_iovec)),
                ('msg_iovlen', ctypes.c_size_t),
                ('msg_control', ctypes.c_void_p),
                ('msg_controllen', ctypes.c_size_t),
                ('msg_flags', ctypes.c_int)]

# XXX: In the review i rxed a comment to make this module standalone. but i
# don't want to pass the logger into every function or have a hacky c-like
# registration mechanism. I will explore a native python soln. for this post
# 3.3.0. Reference: CCR-6113
logger = logging.getLogger("neighmgrd")

# Reference: RFC4681, Section 4.6.1
class Icmp6OptLLAddr(dpkt.dpkt.Packet):
    '''
    user is expected to fill in all three fields
    '''
    __hdr__ = (('type', 'B', 0),
               ('len', 'B', 0),
               ('lladdr', '6s', ''))

# Reference: RFC4681, Section 4.3
class Icmp6NS(dpkt.dpkt.Packet):
    '''
    User must fill in tgt ip6 address
    '''
    __hdr__ = (('rsvd', '4s', b'x00' * 4),
               ('tgt', '16s', ''))

# Reference: RFC4681, Section 4.4
class Icmp6NA(dpkt.dpkt.Packet):
    '''
    User must fill in tgt ip6 address
    '''
    __hdr__ = (('rso', 'B', 0),
               ('rsvd', '3s', b'x00' * 3),
               ('tgt', '16s', ''))
    # rso bit fields
    R_BIT_POS = 7
    S_BIT_POS = 6
    O_BIT_POS = 5

    def rso_bit_get(self, bitPos):
        (self.rso >> bitPos) & 1

    def rso_bit_set(self, bitPos, bitVal):
        self.rso = self.rso | (bitVal << bitPos)

# Reference: RFC4681, Section 4
class Icmp6(dpkt.dpkt.Packet):
    '''
    User must provide type.
    ip6 computes TCP, UDP and ICMP6 csums so we don't bother to do it here.
    '''
    __hdr__ = (('type', 'B', 0),
               ('code', 'B', 0),
               ('sum', 'H', 0))

    # ICMP6 options:
    # Currently we are only interested in tgt_ll_addr and src_ll_addr.
    # we will silently ignore the rest as recommended by the RFC.
    _my_opt_types = {ICMP6_OPT_SRC_LL_ADDR:Icmp6OptLLAddr, ICMP6_OPT_TGT_LL_ADDR:Icmp6OptLLAddr}
    class _Icmp6Opt(dpkt.dpkt.Packet):
        __hdr__ = (('type', 'B', 0),
                   ('len', 'B', 0))

    def _parse_opts(self, pktclass):
        # we will leave the original data unaltered in case the pkt needs
        # to be rebuilt. and the parsed icmp6_opts will be returned as side
        # band info (opts).
        # XXX: post 3.3.0 i will add bytes method to regen using opts and
        # flush data. it will be easier to upstream that way.
        opt_data = pktclass.data
        pktclass.opts = []

        # parse the options (0 or more)
        while opt_data:
            if len(opt_data) < len(self._Icmp6Opt.__hdr__):
                # malformed opt_data - doesn't have a complete option header
                pktclass.opts = []
                break

            # read in just the opt header and validate it
            my_opt = self._Icmp6Opt(opt_data[:len(self._Icmp6Opt.__hdr__)])
            if my_opt.len < 1:
                # malformed option - rfc 4861 says that the min acceptable
                # length is 8 bytes
                pktclass.opts = []
                break
            # option length is expressed in units of 8 octets
            my_opt_len = 8 * my_opt.len
            if len(opt_data) < my_opt_len:
                # malformed opt_data - doesn't have a complete option
                pktclass.opts = []
                break

            # if this is an option we care abt parse and cache it in
            # pktclass.opts else silently ignore
            if my_opt.type in self._my_opt_types:
                pktclass.opts.append(self._my_opt_types[my_opt.type](opt_data[:my_opt_len]))
            # remove the option that we just read from the opt_data
            opt_data = opt_data[my_opt_len:]

    # ICMP6 type:
    # currently we are only interested in NS and NA. RS, RA and REDIRECT can
    # be added later as needed.
    _typesw = {dpkt.icmp6.ND_NEIGHBOR_SOLICIT:Icmp6NS, dpkt.icmp6.ND_NEIGHBOR_ADVERT:Icmp6NA}

    def unpack(self, buf):
        dpkt.Packet.unpack(self, buf)

        try:
            self.data = self._typesw[self.type](self.data)
            self._parse_opts(self.data)
            setattr(self, self.data.__class__.__name__.lower(), self.data)
        except (KeyError, dpkt.UnpackError):
            pass

# Monkey-patch dpkt.icmp6.ICMP6 and update base class with my protocol class.
# Reason for doing this is because dpkt.dpkt.ICMP6 is not ready for use (it
# is a cut-paste of v4 ICMP and massively messed up).
#
# I will try to send the maintainer a fix later.
dpkt.icmp6.ICMP6 = Icmp6
dpkt.ip6.IP6.set_proto(dpkt.ip.IP_PROTO_ICMP6, Icmp6)

def mactobinary(mac):
    '''
    Converts a MAC address of the form 1122.3344.5566 or 11:22:33:44:55:66 to a
    6-byte packed array.
    '''
    try:
        retVal = binascii.unhexlify(mac.replace(".", "").replace(":",""))
    except TypeError:
        retVal = None
    return retVal

def macBytes2Dotted(macStr):
    return ':'.join('%02x' % ord(b) for b in macStr)

def Icmp6GratNAGen(addr, lladdr):
    """
    Generates an unsolicted Neighbor advertisement packet
    Ethernet header:
        SMAC = lladdr
        DMAC = all-nodes-mcast-mac

    IP header:
        SIP = addr
        DIP = all-nodes-mcast

    ICMP6 NA header:
        R = 1 (router)
        S = 0 (solicited)
        O = 1 (override)
        target = addr

    ICMP6 TGT_LL_ADDR option:
        data = lladdr
    """
    # we build the packet inner to outer
    # build ip-options for ICMP6; len here is expressed in units of 8 octets
    optLLAddr = Icmp6OptLLAddr(type=ICMP6_OPT_TGT_LL_ADDR,
                               len=1,
                               lladdr=lladdr)

    # build NA with ip-options as data
    addrStr = socket.inet_pton(socket.AF_INET6, str(addr))
    na = Icmp6NA(tgt=addrStr, data=optLLAddr)
    # set override bit
    na.rso_bit_set(na.O_BIT_POS, 1)
    # set router bit
    na.rso_bit_set(na.R_BIT_POS, 1)

    # build icmp6 with NA as data
    icmp6 = Icmp6(type=dpkt.icmp6.ND_NEIGHBOR_ADVERT, data=na)

    # build ip6 with icmp6 as data
    ip6 = dpkt.ip6.IP6(src=addrStr, dst=allNodesMcastAddrStr,
                       nxt=dpkt.ip.IP_PROTO_ICMP6, plen=len(icmp6), hlim=255,
                       data=icmp6)

    # build ethernet with with ip6 as data
    eth = dpkt.ethernet.Ethernet(dst=mactobinary(IP6_ALL_NODES_LL_ADDR),
                                 src=lladdr, type = dpkt.ethernet.ETH_TYPE_IP6,
                                 data=ip6)

    if logger.isEnabledFor(logging.DEBUG):
        logger.debug("GARP: %r generated" % eth)
    return str(eth)

def nd_pkt_to_learnt_addr(eth):
    addr = lladdr = None
    ip6 = eth.data

    if not isinstance(ip6.data, Icmp6):
        # not a ICMP6 packet that we care abt
        return addr, lladdr

    icmp6 = ip6.data
    if isinstance(icmp6.data, Icmp6NS):
        ns = icmp6.data
        # There are a few cases where src_ll_addr option may NOT be present
        # 1. for DAD (in this case ip6.src will also be unspec)
        # 2. if unicast NS the rfc says src_ll_addr SHOULD (i.e. not MUST) be
        #    included.
        # The opt is a MUST for multicast solicited NSs
        for opt in ns.opts:
            if opt.type == ICMP6_OPT_SRC_LL_ADDR:
                # for NS use SIP in the ip6 header
                addr = ip6.src
                lladdr = opt.lladdr
                break
    elif isinstance(icmp6.data, Icmp6NA):
        na = icmp6.data
        # There are a few cases where tgt_ll_addr option may NOT be present
        # 1. If the NA was in response to a unicast NS this option maynot be
        # present. It is unclear how the snooper would learn the neigh in
        # such a case.
        # XXX: I should also use the O bit to control the neigh_add REPLACE
        # flag. I will address that post 3.3.0 (some thought is needed to
        # mimic remote host).
        # XXX: I should also use the R bit to update the neigh entry router
        # flag. I will address that post 3.3.0 (some thought is needed to
        # handle interaction with the RS).
        for opt in na.opts:
            if opt.type == ICMP6_OPT_TGT_LL_ADDR:
                addr = na.tgt
                lladdr = opt.lladdr
                break

    if addr:
        addr = socket.inet_ntop(socket.AF_INET6, addr)
        addr = ipaddr.IPAddress(addr)
        lladdr = macBytes2Dotted(lladdr)
    return addr, lladdr

def sendmsg(sock, ifIndex, protocol, packet, packet_len):
    iov = struct_iovec()
    iov.iov_base = ctypes.cast(packet, ctypes.c_void_p)
    iov.iov_len = packet_len

    sockaddr = struct_sockaddr_ll()
    sockaddr.sll_family = socket.AF_PACKET
    sockaddr.sll_protocol = socket.htons(protocol)
    sockaddr.sll_ifindex = ifIndex
    sockaddr.sll_halen = 6
    # XXX - is the sockaddr DMAC or SMAC? just doing DMAC for now
    if packet_len >= sockaddr.sll_halen:
        ctypes.memmove(sockaddr.sll_addr, iov.iov_base, sockaddr.sll_halen)

    msghdr = struct_msghdr()
    msghdr.msg_name = ctypes.cast(ctypes.pointer(sockaddr), ctypes.c_void_p)
    msghdr.msg_namelen = ctypes.sizeof(sockaddr)
    msghdr.msg_iov = ctypes.pointer(iov)
    msghdr.msg_iovlen = 1
    msghdr.msg_control = ctypes.cast(0, ctypes.c_void_p)
    msghdr.msg_controllen = 0
    msghdr.msg_flags = 0

    msg = ctypes.cast(ctypes.pointer(msghdr), ctypes.c_void_p)
    sent_len = libc_handle.sendmsg(sock.fileno(), msg, 0)
    logger.debug("Sent %d bytes to ifIndex %d" % (sent_len, ifIndex))
