#!/usr/bin/env python
# Copyright 2017 Cumulus Networks, Inc.
#
# Stream socket library
# For sample usage see neighmgrctl.

import struct

'''
1. This is a simple sock library that abstracts the partial-read, partial-write
handling required for stream sockets.
2. All communication uses the following format - length,payload. So the
rxer is expected to read the first four bytes to determine the length of the
payload. And then read "length" bytes to get the complete message generated by
the sender before acting on it.
3. To use the lib do the following -
a. Create a StreamInfo for every sock connection (on both sides)
b. For recv -
   - When the sock becomes readable get the number of bytes to read via
     get_read_data_len() and use that as the recv maxLen.
   - Feed the data rxed to proc_read_data(). When one whole message is available
     proc_read_data will return a non-empty string.
c. For send -
   - To send data to the remote end use build_write_data() for generating a
     message from a string. That info is stashed into writeBuf and can be
     used when the sock becomes writeable.
   - Remove the written bytes using remove_written_data()
'''

class StreamInfo:
    '''
    Each accepted/connected socket will have the following info associated
    with it
    '''

    def __init__(self, sock, addr):
        self.sock = sock
        self.addr = addr

        # once a socket is created setup the offsets needed for reading
        # one complete message from the other side
        self._start_read_data()
        self.writeBuf = ''

    def _start_read_data(self):
        # first 4 bytes indicate payload length; so always start by reading
        # first 4
        self.readTryLen = 4
        # bytes read so far
        self.readBuf = ''
        # once the message length is know we will start reading the message
        # itself
        self.cmdReadInProg = False

    def get_read_data_len(self):
        '''
        this api is used to determine the max length that should be read
        from the stream socket
        '''
        return self.readTryLen - len(self.readBuf)

    def proc_read_data(self, data):
        '''
        this api is called every time new data is read from the socket and
        added to readBuf
        '''
        self.readBuf += data
        if len(self.readBuf) < self.readTryLen:
            # more to read
            return None

        if self.cmdReadInProg:
            # we have completed reading the command
            cmdStr = self.readBuf
            self._start_read_data()
            return cmdStr

        # we have just read the length of the command; now to the business of
        # reading the actual command
        self.readTryLen = struct.unpack('!I', self.readBuf)[0]
        if self.readTryLen <= 0:
            # something went wrong; start over
            self._start_read_data()
            return None
        self.readBuf = ''
        self.cmdReadInProg = True
        return None

    def build_write_data(self, payload):
        '''
        this api is used to build the socket message from a input or outpt
        string
        '''
        self.writeBuf += struct.pack('!I', len(payload)) + payload

    def remove_written_data(self, numSent):
        '''
        this api is used to remove the bytes already sent to the remote end
        '''
        self.writeBuf = self.writeBuf[numSent:]
